#include "code/framework/anim/Anim.h"
#include <iostream>
#include <string>
#include <vector>
#include <list>
#include "code/framework/utils.h"
#include "code/framework/check/PrettyFileLogger.h"

using namespace std;
using namespace utils;

/////////////////////////////////////////////
//                 STATIC
/////////////////////////////////////////////

//a mettre pour que le fichier connaisse la variable
list<ImageAndAttr *> Anim::anim_loaded_images;


//static methods
int Anim::getNbBufferedImages()
{
	return (int)anim_loaded_images.size();	
}

//returns NULL if not found.
ImageAndAttr *Anim::getImageWithName(string name)
{

	for(list<ImageAndAttr *>::iterator it = Anim::anim_loaded_images.begin(); it!=Anim::anim_loaded_images.end(); it++) 
	{
    	if((*it)->getName().compare(name)==0)
    	{
    		return *it;//&(*it);
    	}
	}
	
	return NULL;
}

//make sure it's not already here
ImageAndAttr *Anim::bufferImage(ImageAndAttr *iaa)
{
	//pour éviter les réallocations completes lors d'un pushback
	//on lui dit de le faire tous les 10
	/*if(anim_loaded_images.capacity()==anim_loaded_images.size())
	{
		anim_loaded_images.reserve(anim_loaded_images.size()+10);
	}*/
	

	anim_loaded_images.push_back(iaa);

	return iaa;
}

void Anim::unbufferImage(ImageAndAttr *iaa)
{
	if(iaa==NULL)
		fatalError("Anim::unbufferImage : iaa est NULL");
		
	for(list<ImageAndAttr *>::iterator it = anim_loaded_images.begin(); it!=anim_loaded_images.end(); it++)
	{
		//string s=(*it).getName();
		if((*it)->getName().compare(iaa->getName())==0)
		{
			it=anim_loaded_images.erase(it);
			delete iaa;
			return;
		}
	}
}

void Anim::logFilesInBuffer()
{
	for(list<ImageAndAttr *>::iterator it = anim_loaded_images.begin(); it!=anim_loaded_images.end(); it++)
	{
		ImageAndAttr *iat=*it;
		PFL::logMessage(iat->getName() + " ("+toString(iat->getNbRef())+")");
	}
}




////////////////////////////////////////////////
//				   INSTANCE
////////////////////////////////////////////////

//constructeur
Anim::Anim(unsigned int nbImages, string header, string where, float delay_total, short transparency, int maxLoops)
{

	
	//on réserve de la place dans le vecteur
	m_frames.reserve(nbImages);
	
	m_currentFrame=1; //l'image courante //TODO: utiliser un iterator?
	m_nbImages=0; //le nombre d'images que contient l'animation, incrémenté par addImage
	m_loop=1; //le numero de loop en cours.
	m_running=false; //est ce que l'animation est en marche?
	m_nbTicks=0.0f; //ou en on est dans le nbre de ticks actuellement
	m_maxLoops=maxLoops; //combien de boucles elle doit faire
	
	m_delay_total=delay_total; //le temps total
	m_delay_frames=delay_total/nbImages; //temps entre chaque frame
	m_delay_left=0.0f; //temps qu'on a parcouru en plus lors d'une boucle, pris en compte dans le temps passé pour la prochaine
	
	if(where.empty())
		where="";
	for(unsigned int i=1;i<=nbImages;i++)
	{
		string filename=where+header+toString(i)+".png";
		addImage(filename, transparency);
	}
	
	m_x=0;
	m_y=0;
}

Anim::Anim(string filename, float delay_total, short transparency, int maxLoops)
{
	m_currentFrame=1; //l'image courante //TODO: utiliser un iterator?
	m_nbImages=0; //le nombre d'images que contient l'animation, incrémenté par addImage
	m_loop=1; //le numero de loop en cours.
	m_running=false; //est ce que l'animation est en marche?
	m_nbTicks=0; //ou en on est dans le nbre de ticks actuellement
	m_maxLoops=maxLoops; //combien de boucles elle doit faire
	
	m_delay_total=delay_total; //le temps total
	m_delay_frames=delay_total; //temsp entre chaque frame
	m_delay_left=0.0f;
	
	addImage(filename, transparency);
	
	m_x=0;
	m_y=0;
}

Anim::Anim(OSL_IMAGE *image, string imageId, float delay_total, int maxLoops)
{
	m_currentFrame=1; //l'image courante //TODO: utiliser un iterator?
	m_nbImages=0; //le nombre d'images que contient l'animation, incrémenté par addImage
	m_loop=1; //le numero de loop en cours.
	m_running=false; //est ce que l'animation est en marche?
	m_nbTicks=0; //ou en on est dans le nbre de ticks actuellement
	m_maxLoops=maxLoops; //combien de boucles elle doit faire
	
	m_delay_total=delay_total; //le temps total
	m_delay_frames=delay_total; //temsp entre chaque frame
	m_delay_left=0.0f;
	
	addImage(image, imageId);
	
	m_x=0;
	m_y=0;
}

Anim::Anim(string filename, OSL_PIXELFORMATS transparency)
{	
	m_currentFrame=1; //l'image courante 
	m_nbImages=0; //le nombre d'images que contient l'animation, incrémenté par addImage
	m_loop=1; //le numero de loop en cours.
	m_running=false; //est ce que l'animation est en marche?
	m_nbTicks=0; //ou en on est dans le nbre de ticks actuellement
	
	//0 pour qu'on ne puisse jamais la demarrer
	m_maxLoops=0; //combien de boucles elle doit faire
	
	m_delay_total=0.0f; //le temps total
	m_delay_frames=0.0f; //temsp entre chaque frame
	m_delay_left=0.0f;
	
	addImage(filename, transparency);
	
	m_x=0;
	m_y=0;
}

Anim::Anim(OSL_IMAGE *image, string imageID)
{	
	m_currentFrame=1; //l'image courante 
	m_nbImages=0; //le nombre d'images que contient l'animation, incrémenté par addImage
	m_loop=1; //le numero de loop en cours.
	m_running=false; //est ce que l'animation est en marche?
	m_nbTicks=0; //ou en on est dans le nbre de ticks actuellement
	
	//0 pour qu'on ne puisse jamais la demarrer
	m_maxLoops=0; //combien de boucles elle doit faire
	
	m_delay_total=0.0f; //le temps total
	m_delay_frames=0.0f; //temsp entre chaque frame
	m_delay_left=0.0f;
	
	addImage(image, imageID);
	
	m_x=0;
	m_y=0;
}


void Anim::addImage(string filename, short transparency)
{
	ImageAndAttr *iaa=Anim::getImageWithName(filename);
	if(iaa==NULL) //non trouvé! il faut le mettre en buffer
	{
		iaa=new ImageAndAttr(filename, transparency);
		//mise en buffer
		bufferImage(iaa);
	}
	iaa->addRef(); //y en a 0 au début, faut que ca soit au moins a 1
		
	//on est sûr que iaa est bon!!
	//on l'ajoute au vecteur
	m_frames.push_back(iaa);
	m_nbImages++;
	
	m_delay_frames=m_delay_total/m_nbImages; //temps entre chaque frame
}

void Anim::addImage(OSL_IMAGE *image, string imageId)
{
	ImageAndAttr *iaa=Anim::getImageWithName(imageId);
	if(iaa==NULL) //non trouvé! il faut le mettre en buffer
	{
		iaa=new ImageAndAttr(image, imageId);
		//mise en buffer
		bufferImage(iaa);
	}
	iaa->addRef(); //y en a 0 au début, faut que ca soit au moins a 1
		
	//on est sûr que iaa est bon!!
	//on l'ajoute au vecteur
	m_frames.push_back(iaa);
	m_nbImages++;
	
	m_delay_frames=m_delay_total/m_nbImages; //temps entre chaque frame
}


//copie
Anim::Anim(const Anim &an)
{

	m_currentFrame=an.m_currentFrame;
	m_delay_frames=an.m_delay_frames;
	m_delay_total=an.m_delay_total;
	m_delay_left=an.m_delay_left;
	m_frames=an.m_frames; //vecteur de pointeurs donc copie directe ok.
	m_loop=an.m_loop;
	m_maxLoops=an.m_maxLoops;
	m_nbImages=an.m_nbImages;
	m_nbTicks=an.m_nbTicks;
	m_running=an.m_running;
	m_x=an.m_x;
	m_y=an.m_y;
	
	//il faut incrémenter le nombre de références sur les images
	for(vector<ImageAndAttr *>::iterator it = m_frames.begin(); it<m_frames.end(); it++)
	{
		(*it)->addRef();
	}
}

Anim& Anim::operator =(const Anim& an)
{

	m_currentFrame=an.m_currentFrame;
	m_delay_frames=an.m_delay_frames;
	m_delay_total=an.m_delay_total;
	m_delay_left=an.m_delay_left;
	m_frames=an.m_frames; //vecteur de pointeurs donc copie directe ok.
	m_loop=an.m_loop;
	m_maxLoops=an.m_maxLoops;
	m_nbImages=an.m_nbImages;
	m_nbTicks=an.m_nbTicks;
	m_running=an.m_running;
	m_x=an.m_x;
	m_y=an.m_y;
	
	//il faut incrémenter le nombre de références sur les images
	for(vector<ImageAndAttr *>::iterator it = m_frames.begin(); it<m_frames.end(); it++)
	{
		(*it)->addRef();
	}
    return *this;
}

//destructor
Anim::~Anim()
{

	for(vector<ImageAndAttr *>::iterator it = m_frames.begin(); it<m_frames.end(); it++) 
	{
    	string name=(*it)->getName();
    	ImageAndAttr *iaa=Anim::getImageWithName(name);
    	if(iaa!=NULL)
    	{
    		iaa->subRef();	
    		if(iaa->getNbRef()==0)
    		{
    			unbufferImage(iaa);
    		}
    	}
	}
	m_frames.clear();
}

float Anim::getDelayTotal()
{
	return m_delay_total;
}

float Anim::getDelayBetweenFrames()
{
	return m_delay_frames;
}

void Anim::setDelayTotal(float d)
{
	m_delay_total=d;
	m_delay_frames=d/m_nbImages;
}

void Anim::setNbLoops(int loops)
{
	m_maxLoops=loops;
}

int Anim::getNbImages()
{
	return m_nbImages;
}

void Anim::setReferencePoint(short x, short y)
{
	for(vector<ImageAndAttr *>::iterator it = m_frames.begin(); it<m_frames.end(); it++)
	{
		OSL_IMAGE * img=(*it)->getImage();
		img->centerX=x;
		img->centerY=y;
	}
}

void Anim::setReferencePointCenter()
{
	for(vector<ImageAndAttr *>::iterator it = m_frames.begin(); it<m_frames.end(); it++)
	{
		OSL_IMAGE * img=(*it)->getImage();
		oslImageSetRotCenter(img);
	}
}

//retourne l'image courante
OSL_IMAGE *Anim::getImage() const
{
	return m_frames[m_currentFrame-1]->getImage();
}

//retourne le nom de l'image courante
string Anim::getImageName() const
{
	return m_frames[m_currentFrame-1]->getName();
}

void Anim::start()
{
	if(m_maxLoops!=0)
		m_running=true;
	//sinon, c'est une animation avec 1 seule image, donc ca démarre jamais
	//ou alors l'utilisateur a mis 0 pour pas que ca avance
}

void Anim::stop()
{
	m_running=false;
}

void Anim::restart()
{
	reset();
	start();
}

void Anim::reset()
{
	m_currentFrame=1;
	m_loop=1;
	m_nbTicks=0.0f;
	m_delay_left=0.0f;
}

//void Anim::update()
//{
//	if(m_running)
//	{
//		m_nbTicks++;
//		if(m_nbTicks>m_delay)
//		{
//			m_nbTicks=0;
//			m_currentFrame++;
//			if(m_currentFrame>m_nbImages)
//			{
//				m_loop++;
//				if(m_maxLoops==-1 || m_loop<=m_maxLoops) //il peut continuer
//				{
//					m_currentFrame=1;
//				}
//				else //on arrete l'animation, m_maxLoops!=NULL && m_loop>=m_maxLoops.
//				{
//					m_currentFrame=m_nbImages; //on la met a la fin
//					m_loop=m_maxLoops; //on le met a la fin
//					m_running=false;
//				}
//			}	
//		}
//	}
//}

void Anim::update()
{
	if(m_running) //en marche et anim non bloquée (si anim bloquée, start() ne fera rien.
	{

		m_nbTicks++;

		float delay_passed=m_nbTicks+m_delay_left;

		
		while(delay_passed>=m_delay_frames) //tant que le temps est superieur a celui entre les images
		{

			//on avance d'une image
			m_nbTicks=0.0f;
			delay_passed-=m_delay_frames;

			m_currentFrame++;

			if(m_currentFrame>m_nbImages)
			{

				m_loop++;
				if(m_maxLoops==-1 || m_loop<=m_maxLoops) //il peut continuer
				{
					m_currentFrame=1;
				}
				else //on arrete l'animation, m_maxLoops!=-1 && m_loop>=m_maxLoops.
				{
					m_currentFrame=m_nbImages; //on la met a la fin
					m_loop=m_maxLoops; //on le met a la fin
					m_running=false;
				}
			}
			m_delay_left=delay_passed;
		}

	}

}

bool Anim::finished() const
{
	if(m_maxLoops==0)
		return true;
		
	if(m_maxLoops==-1)
		return false;
		
	if(m_loop==m_maxLoops && m_currentFrame==m_nbImages)
		return true;
		
	return false;
}

void Anim::drawXY(int x, int y)
{
	oslDrawImageXY(getImage(), x, y); //was oslDrawImageSimpleXY
}

void Anim::draw()
{
	oslDrawImageXY(getImage(), m_x, m_y);
}

void Anim::setX(int x)
{
	m_x=x;
}

void Anim::setY(int y)
{
	m_y=y;
}

int Anim::getX()
{
	return m_x;
}

int Anim::getY()
{
	return m_y;
}

State *Anim::savestate(State * state)
{
	state->m_x=m_x;
	state->m_y=m_y;
	return state;
}

void Anim::restorestate(State * state)
{
	m_x=state->m_x;
	m_y=state->m_y;
}


int Anim::getCurrentFrameNum() const
{
	return m_currentFrame;	
}

bool Anim::isRunning() const
{
	return m_running;	
}

void Anim::sync(Anim *anim)
{
	m_delay_total=anim->m_delay_total;
   	m_delay_frames=anim->m_delay_frames;
   	m_delay_left=anim->m_delay_left;
   	
   	m_currentFrame=anim->m_currentFrame;
   	m_loop=anim->m_loop;
   	m_running=anim->m_running;
   	m_nbTicks=anim->m_nbTicks;
}



























