#include "code/framework/bmpFont/BmpFont.h"
#include "code/framework/utils.h"
#include <string>
#include <oslib/oslib.h>
#include "code/framework/tree.hh"
#include <vector>
#include <list>
#include <fstream>
#include <iostream>
#include <sstream>
#include "code/framework/check/loaders.h"
#include <algorithm>

using namespace std;
using namespace utils;

//static
CharRowCol *BmpFont::chars; //contient le char, la ligne et la colonne correspondant 
								//au char numero i. commence a 0.
bool BmpFont::isInit=false; //lazy init //TODO: pas utile?
tree<short> *BmpFont::bytesTree; //contient les sommes de bytes 


//////////////////////////////////////////////
//            		STATIC
//////////////////////////////////////////////
void BmpFont::freeRes()
{
	if(isInit)
	{
		delete[] chars;
		delete bytesTree;
		isInit=false;	
	}	
}

void BmpFont::initBmpFont(istream *stream_chars, istream *stream_bytes)
{
	if(!isInit)
	{		

		
		chars=new CharRowCol[256];
		bytesTree=new tree<short>();
		
		//numero de colonne et de ligne
		for(int i=0;i<256; i++)
		{
			chars[i].setRow((short) i/16); //commence a 0
			chars[i].setCol(i%16); //commence a 0
		}
		
		//le caractère, en fait une string de plusieurs
		string line;
		while(getline(*stream_chars,line))
		{
			if(line.find("[") != 0) //ne pas prendre les commentaires en compte
			{
				if(line.size()>2) //ne pas prendre en compte les lignes vides, au minimum "a=b"
				{
					int indice=line.find("=");
					int key=toInt(line.substr(0, indice)); //longueur = indice
					string value=line.substr(indice+1, line.size()-1-indice);
					
					value=deleteAll(value, "\r");
					chars[key].setChar(value);
					
				}
			}
		}
		
		
		//les sommes de bytes:
		//initialisation de l'arbre de sommes de bytes
		tree<short>::iterator top, cur, loc;
		top=bytesTree->begin();
		
		//lecture dans le fichier
		while(std::getline(*stream_bytes,line))
		{
			if(line.find("[") != 0) //ne pas prendre les commentaires en compte
			{
				if(line.size()>2) //ne pas prendre en compte les lignes vides, au minimum "a=b"
				{
					int equalPos=line.find("="); //equalPos est aussi la longueur jusqu'a lui
					string sumPart=line.substr(0, equalPos);
					string resultPart=line.substr(equalPos+1, line.size()-1-equalPos);
					resultPart=deleteAll(resultPart, "\r");
					short result= (short) toInt(resultPart);
					
					
					//j'ai maintenant le résultat (result) et une string contenant des sommes (sumPart)
					//on va scinder les sommes via le signe "+"
					vector<string> strBytes=split(sumPart, "+");
					
					//ok j'ai les bytes sous forme de string je les convertis en short
					vector<short> bytes;
					for(unsigned int i=0; i<strBytes.size(); i++)
					{
						bytes.push_back((short) toInt(strBytes[i]));
					}
					
					//maintenant on rempli l'arbre
					cur=top; //position dans l'arbre, on commence en haut
					for(unsigned int byteNum=0; byteNum<bytes.size();byteNum++)
					{
						loc=find(bytesTree->begin(cur), bytesTree->end(cur), bytes[byteNum]); //on parcours les siblings a la recherche du byte
						if(loc!=bytesTree->end(cur)) //on a trouvé
						{
							cur=loc;
						}
						else //on n'a pas trouvé
						{
							cur=bytesTree->append_child(cur, bytes[byteNum]);
						}
					}
					bytesTree->append_child(cur, result);
				}
			}
		}
		
		isInit=true;
	}
}

void BmpFont::initBmpFont(string folder)
{
	if(!isInit)
	{
		//ifstream fichier_chars(string(folder+"chars.ini").c_str());
		
//		if(fichier_chars.fail())
//		{
//			fatalError("BmpFont::initBmpFont : cannot open "+folder+"chars.ini");
//		}
	
		ifstream fichier_chars;
		fwkOpenTextFile(folder+"chars.ini", &fichier_chars);
		
		//ifstream fichier_bytes(string(folder+"bytes.ini").c_str());
		
//		if(fichier_bytes.fail())
//		{
//			fatalError("BmpFont::initBmpFont : cannot open "+folder+"bytes.ini");
//		}
		
		ifstream fichier_bytes;
		fwkOpenTextFile(folder+"bytes.ini", &fichier_bytes);
		
		initBmpFont(&fichier_chars, &fichier_bytes);
		
		fichier_chars.close();
		fichier_bytes.close();		
	}
}

//faire un delete[] du tableau
unsigned short * BmpFont::loadSizes(string fileradix)
{
//	ifstream fichier_sizes((fileradix+".ini").c_str());
//	if(fichier_sizes.fail())
//	{
//		fatalError("BmpFont::loadSizes : cannot open "+fileradix+"chars.ini");
//	}
	
	ifstream fichier_sizes;
	fwkOpenTextFile(fileradix+".ini", &fichier_sizes);

	unsigned short * sizes = loadSizes(&fichier_sizes);
	fichier_sizes.close();
	
	return sizes;
}

unsigned short * BmpFont::loadSizes(istream *fichier_sizes)
{
	unsigned short * sizes = new unsigned short[256];
	string line;
	//ifstream fichier_sizes((fileradix+".ini").c_str());
	while(std::getline(*fichier_sizes, line))
	{
		if(line.find("[") != 0) //ne pas prendre les commentaires en compte
		{
			if(line.size()>2) //ne pas prendre en compte les lignes vides, au minimum "a=b"
			{
				int equalPos=line.find("="); //equalPos est aussi la longueur jusqu'a lui
				int key = toInt(line.substr(0, equalPos));
				string sizeStr=line.substr(equalPos+1, line.size()-1-equalPos);
				sizeStr=deleteAll(sizeStr, "\r");
				unsigned short size=(unsigned short)toInt(sizeStr);
				
				sizes[key]=size;		
			}
		}
	}
	
	//fichier_sizes.close();
	
	return sizes;
}

//pas oublier de désalouer avec delete laliste
list<short> * BmpFont::getIndicesFromStr(string str)
{


	initBmpFont(BMPFONT_INIT_DEFAULT_FOLDER);


	list<short> * table= new list<short>();
	int strLen=str.size();
	short index=0; //c'etait 1
	
	if(strLen==0)
		return table;
		
	tree<short>::iterator top, cur, loc;
	top=bytesTree->begin();
	

	while(index<strLen)
	{
		cur=top;
		unsigned char byte1=str[index];

		loc=find(bytesTree->begin(cur), bytesTree->end(cur), byte1);
		if(loc==bytesTree->end(cur)) //pas trouvé byte1 dans l'arbre
		{

			table->push_back(byte1);
			index++;
		}
		else //trouvé byte1
		{

			//on cherche le 2eme
			cur=loc;
			unsigned char byte2=str[index+1];

			loc=find(bytesTree->begin(cur), bytesTree->end(cur), byte2);
			if(loc==bytesTree->end(cur)) //pas trouvé
			{
				fatalError("Arbre de bytes mal rempli, 1 element sans fils!");
			}
			else //trouvé un 2eme
			{

				//faut chercher son fils
				//DANS L4ARBRE!!!
				//si son fils a n'a pas de fils b, le fils b est le résultat 
				cur=loc;
				int nbCh=bytesTree->number_of_children(loc);
				if(nbCh==0)
				{
					fatalError("Arbre de bytes mal rempli, 1 element avec un fils mais pas de petit fils!");
				}
				else if(nbCh==1) //un seul fils pour byte2, on le cherche
				{
					loc=bytesTree->child(loc, 0); //ca c le fils de byte2
					if(bytesTree->number_of_children(loc)==0) //c'est le résultat
					{
						table->push_back(*loc);
						index+=2;
					}
					else //byte2 a des fils, c'est un membre de l'adition
					{
						/*unsigned char byte3=str[index+2];
		
						loc=find(bytesTree.begin(cur), bytesTree.end(cur), byte3);
						if(loc==bytesTree.end(cur)) //pas trouvé
						{
							oslFatalError("Arbre de bytes mal rempli, 1 element avec un fils mais pas de petit fils!");
						}
						else //trouvé le byte3 à loc!
						{

							//s'il a pas de fils, c'est un résultat.
							//sinon, c'est une adition
							if(bytesTree.number_of_children(loc)==0)//c'etait un resultat
							{
	
								table->push_back(byte3);
								index+=2;
							}
							else //c'est un membre de l'adition
							{
								//faut trouver le fils unique
								unsigned short byte4=*bytesTree.child(loc, 0);
								table->push_back(byte4);
								index+=3;
							}
						}*/
						goto fils3; //ok c'est moche mais bon...
					}
				}
				else //2eme a plusieurs fils
				{
fils3:
					unsigned char byte3=str[index+2];
	
					loc=find(bytesTree->begin(cur), bytesTree->end(cur), byte3);
					if(loc==bytesTree->end(cur)) //pas trouvé
					{
						fatalError("Arbre de bytes mal rempli, 1 element avec un fils mais pas de petit fils!");
					}
					else //trouvé le byte3 à loc!
					{
			
						//s'il a pas de fils, c'est un résultat.
						//sinon, c'est une adition
						if(bytesTree->number_of_children(loc)==0)//c'etait un resultat
						{
				
							table->push_back(byte3);
							index+=2;
						}
						else //c'est un membre de l'adition
						{
							//faut trouver le fils unique
							unsigned short byte4=*bytesTree->child(loc, 0);
							table->push_back(byte4);
							index+=3;
						}
					}
				}				
			}
		}
	}

	return table;
}
//TODO: réécrire ce merdier la haut






/////////////////////////////////////////////
//				INSTANCE
/////////////////////////////////////////////


BmpFont::BmpFont(string where, string filename, short transparency, bool cursive)
{
	initBmpFont(BMPFONT_INIT_DEFAULT_FOLDER);
	
	string radix=where+filename;
	
	m_sizes=loadSizes(radix);
	//m_img = oslLoadImageFilePNG((char *)(radix+".png").c_str(), OSL_IN_RAM | OSL_SWIZZLED, transparency);
	m_img=fwkLoadImageFilePNG(radix+".png", OSL_IN_RAM | OSL_SWIZZLED, transparency);
	
	m_height=m_img->sizeY/16;
	m_width=m_img->sizeX/16;
	
	m_positions=new Pos[256];
	for(unsigned i=0; i<256; i++)
	{
		m_positions[i].x=(chars[i].getCol() * m_width) + ((m_width-m_sizes[i])/2);
		m_positions[i].y=chars[i].getRow() * m_height;	
	}
	
	m_dsizes=new unsigned short[256];
	for(unsigned i=0; i<256; i++)
	{
		m_dsizes[i] = m_width - ((m_width-m_sizes[i])/2);	
	}
	
	m_cursive=cursive;
	
	//m_img->angle=105;
}

BmpFont::BmpFont(OSL_IMAGE *img, istream *fichier_sizes, bool cursive)
{
	initBmpFont(BMPFONT_INIT_DEFAULT_FOLDER);
	
	m_sizes=loadSizes(fichier_sizes);
	m_img = img;
	
	m_height=m_img->sizeY/16;
	m_width=m_img->sizeX/16;
	
	m_positions=new Pos[256];
	for(unsigned i=0; i<256; i++)
	{
		m_positions[i].x=(chars[i].getCol() * m_width) + ((m_width-m_sizes[i])/2);
		m_positions[i].y=chars[i].getRow() * m_height;	
	}
	
	m_dsizes=new unsigned short[256];
	for(unsigned i=0; i<256; i++)
	{
		m_dsizes[i] = m_width - ((m_width-m_sizes[i])/2);	
	}
	
	m_cursive=cursive;
	
	//m_img->angle=105;
} 

BmpFont::BmpFont(const BmpFont &f)
{
	
	m_sizes = new unsigned short[256];
	for(unsigned int i=0;i<256;i++)
		m_sizes[i]=f.m_sizes[i];
		
	m_dsizes = new unsigned short[256];
	for(unsigned int i=0;i<256;i++)
		m_dsizes[i]=f.m_dsizes[i];
	
	m_positions=new Pos[256];
	for(unsigned int i=0;i<256;i++)
		m_positions[i]=f.m_positions[i];
	
	//m_img=oslCreateImageCopy(f.m_img, f.m_img->location);
	m_img=oslCreateSwizzledImage(f.m_img, f.m_img->location);
	
	m_width=f.m_width;
	m_height=f.m_height;
	m_cursive=f.m_cursive;
}

BmpFont& BmpFont::operator =(const BmpFont& f)
{
	//m_sizes = new unsigned short[256];
	for(unsigned int i=0;i<256;i++)
		m_sizes[i]=f.m_sizes[i];
		
	//m_dsizes = new unsigned short[256];
	for(unsigned int i=0;i<256;i++)
		m_dsizes[i]=f.m_dsizes[i];
	
	//m_positions=new Pos[256];
	for(unsigned int i=0;i<256;i++)
		m_positions[i]=f.m_positions[i];
	
	//m_img=oslCreateImageCopy(f.m_img, f.m_img->location);
	if(m_img!=NULL)
		oslDeleteImage(m_img);
	
	m_img=oslCreateSwizzledImage(f.m_img, f.m_img->location);
	
	m_width=f.m_width;
	m_height=f.m_height;
	
	m_cursive=f.m_cursive;
	
	return *this;
}

BmpFont::~BmpFont()
{
	delete[] m_sizes;
	delete[] m_dsizes;
	oslDeleteImage(m_img);
	delete[] m_positions;
}

OSL_IMAGE * BmpFont::getImage()
{
	return m_img;
}

unsigned short BmpFont::getTextSize(string text) const
{

	list<short> * indices = getIndicesFromStr(text);

	
	short sz=getTextDrawSize(indices);
	

	delete indices;
	
	return sz;
}

unsigned short BmpFont::getTextDrawSize(string text) const
{
	list<short> * indices = getIndicesFromStr(text);

	short sz=getTextDrawSize(indices);
	
	delete indices;
	
	return sz;
}

unsigned short BmpFont::getTextSize(list<short> * indices) const
{
	short sz=0;
	
	list<short>::iterator it;
	for (it=indices->begin(); it != indices->end(); it++)
	{
		short indice=*it;

		sz+=m_sizes[indice];
	}
	

	return sz;
}

unsigned short BmpFont::getTextDrawSize(list<short> *indices) const
{
	short sz=0;	
	short lastSize=0;
	short lastDsize=0;
	list<short>::iterator it;
	for (it=indices->begin(); it != indices->end(); it++)
	{
		short indice=*it;
		lastSize=m_sizes[indice];
		lastDsize=m_dsizes[indice];
		sz+=lastSize;
	}
	sz=sz-lastSize+lastDsize;
	
	return sz;
}

bool BmpFont::isCursive()
{
	return m_cursive;
}

unsigned short BmpFont::getHeight() const
{
	return m_height;
}

void BmpFont::drawChar(short indice, short x, short y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	oslSetImageTileSize(m_img,m_positions[indice].x,m_positions[indice].y,m_dsizes[indice],m_height);
	//oslDrawImageXY(m_img, x, y);
	fx::drawXY(m_img, x, y, fx1, fx2, fx3,fx4, fx5);
}


//tests
CharRowCol *BmpFont::getCRC(int indice)
{

	initBmpFont(BMPFONT_INIT_DEFAULT_FOLDER);

	return &chars[indice];
}

unsigned short BmpFont::getSize(int indice)
{
	return m_sizes[indice];	
}

unsigned short BmpFont::getDSize(int indice)
{
	return m_dsizes[indice];	
}








