#include "code/framework/check/Trackable.h"
#include "code/framework/utils.h"
#include "code/framework/check/loaders.h"
#include <iostream>
#include <fstream>

using namespace utils;
using namespace std;


map<string, int> Trackable::m_allocations;
map<string, int> Trackable::m_unallocations;
bool Trackable::m_istracking=false;

Trackable::Trackable(string classname, int id)
{
	
	if(id!=-1)
		m_classname=classname+"_"+toString(id);
	else	
		m_classname=classname;
	
	if(m_istracking)
	{
		map<string, int>::iterator it=m_allocations.find(m_classname);
		if(it!=m_allocations.end())
		{
			it->second++;
		}
		else
		{
			m_allocations.insert(pair<string, int>(m_classname, 1));
		}
	}
}

Trackable::~Trackable()
{
	if(m_istracking)
	{
		map<string, int>::iterator it=m_unallocations.find(m_classname);
		if(it!=m_unallocations.end())
		{
			it->second++;
		}
		else
		{
			m_unallocations.insert(pair<string, int>(m_classname, 1));
		}
	}
}

Trackable::Trackable(const Trackable & trackable) //copie
{
	m_classname=trackable.m_classname;
		
	if(m_istracking)
	{
		map<string, int>::iterator it=m_allocations.find(m_classname);
		if(it!=m_allocations.end())
		{
			it->second++;
		}
		else
		{
			m_allocations.insert(pair<string, int>(m_classname, 1));
		}
	}
}

Trackable& Trackable::operator=(const Trackable &trackable)
{
	m_classname=trackable.m_classname;
	
	if(m_istracking)
	{
		map<string, int>::iterator it=m_allocations.find(m_classname);
		if(it!=m_allocations.end())
		{
			it->second++;
		}
		else
		{
			m_allocations.insert(pair<string, int>(m_classname, 1));
		}
	}
	
	return *this;
}

void Trackable::dumpToFile(string filename)
{
	/*
	if(fileExists(filename))
	{
		if(sceIoRemove(filename)!=0) //failure
		{
			fatalError("cannot delete file '"+filename+"'");
		}
	}
	*/
	
	ofstream outfile;
	fwkOpenTextFile(filename, &outfile, ios_base::out|ios_base::trunc);
	
	
	for(map<string, int>::iterator it=m_allocations.begin();it!=m_allocations.end();it++)
	{
		pair<string, int> p=*it;
		int alloc=p.second;
		string key=p.first;
		
		outfile << key << " :   \t" << alloc << " \t";
		
		map<string, int>::iterator it_unalloc=m_unallocations.find(key);
		if(it_unalloc==m_unallocations.end())
		{
			outfile << "error " << endl;
		}
		else
		{
			int unalloc=it_unalloc->second;
			outfile << unalloc << " \t" << alloc-unalloc << " ";
			
			if(alloc-unalloc!=0)
			{
				outfile << "<<<<<<< MEMORY LEAK";
				message("memory leak detected with "+key);
			}
			
			outfile << endl;
		}
	}
	
	outfile.flush();
	outfile.close();
}

void Trackable::startTracking()
{
	m_istracking=true;
}

void Trackable::stopTracking()
{
	m_istracking=false;
}

void Trackable::zeroAll()
{
	for(map<string, int>::iterator it=m_allocations.begin();it!=m_allocations.end();it++)
	{
		it->second=0;
	}
	for(map<string, int>::iterator it=m_unallocations.begin();it!=m_unallocations.end();it++)
	{
		it->second=0;
	}
}




























