#include "code/framework/fx/impl/local/RotationEffect.h"

RotationEffect::RotationEffect(unsigned int revolutionTime, bool direction, int nbLoops)
{
	m_clockwise=direction;
	m_time=revolutionTime;
	m_curLoop=0;
	m_curTime=0;
	m_nbLoops=nbLoops;
	m_running=false;
}
 
	
void RotationEffect::start()
{
	m_running=true;
}
 
void RotationEffect::restart()
{
	m_curLoop=0;
	m_curTime=0;
	m_running=true;
}
 
void RotationEffect::reset()
{
	m_curLoop=0;
	m_curTime=0;
}
 
void RotationEffect::stop()
{
	m_running=false;
}
 
void RotationEffect::update()
{
	if(m_running)
	{
		m_curTime++;
		if(m_curTime==m_time)
		{
			m_curTime=0;
			m_curLoop++;	
		}
		if(finished())
		{
			m_running=false;	
			return;
		}
	}
	
}
 
bool RotationEffect::finished()
{
	if(m_nbLoops==-1) //infini
		return false; //ca fini jamais
	
	if(m_nbLoops==m_curLoop)
	{
		return true;
	}
	return false;
}
 
bool RotationEffect::isRunning()
{
	return m_running;
}
 
OSL_IMAGE * RotationEffect::applyOn(OSL_IMAGE *img)
{
	if(img==NULL)
		return NULL;
		
	unsigned int rot=int((m_curTime*360)/m_time);
	if(m_clockwise)
	{
		img->angle+=rot;
	}
	else
	{
		img->angle-=rot;
	}
	return img;
}

Orientable *RotationEffect::applyOnOrientable(Orientable *obj)
{
	if(obj==NULL)
		return NULL;
		
	unsigned int rot=int((m_curTime*360)/m_time);
	if(m_clockwise)
	{
		obj->setOrientation(obj->getOrientation()+rot);
	}
	else
	{
		obj->setOrientation(obj->getOrientation()-rot);
	}
	return obj;
}
 
RotationEffect* RotationEffect::clone() const
{
	return new RotationEffect(m_time, m_clockwise, m_nbLoops);
}

void RotationEffect::setRevolutionTime(unsigned int time)
{
	m_time=time;
}

unsigned int RotationEffect::getRevolutionTime()
{
	return m_time;
}

int RotationEffect::getOrientation()
{
	int rot=int((m_curTime*360)/m_time);
	if(m_clockwise)
	{
		return rot;
	}
	//else
	return -rot;
}















