#include "code/framework/fx/impl/local/SizeWowEffect.h"
#include "code/framework/utils.h"

using namespace utils;


//TODO: bug: sizewow se base sur la taille sans prise en compte de stretch

SizeWowEffect::SizeWowEffect(int shrinkPercentX, int growPercentX, int shrinkPercentY, int growPercentY, int growTime, int shrinkTime, bool startSmall, int nbLoops)
{
	m_currentLoop=0;
	m_currentTick=0;
	m_growing=startSmall;
	m_growTime=growTime;
	m_nbLoops=nbLoops;
	m_running=false;
	m_shrinkTime=shrinkTime;
	m_startSmall=startSmall;
	m_shrinkPercentX=shrinkPercentX;
	m_growPercentX=growPercentX;
	m_shrinkPercentY=shrinkPercentY;
	m_growPercentY=growPercentY;
	
//	if(m_shrinkPercentX>=100 || m_shrinkPercentY>=100)
	//	oslFatalError("cannot declare a sizewoweffect with 100 or more shrink %");
}
	
void SizeWowEffect::start()
{
	m_running=true;
}
 
void SizeWowEffect::restart()
{
	m_currentLoop=0;
	m_currentTick=0;
	m_growing=m_startSmall;
	m_running=true;
}

void SizeWowEffect::reset()
{
	m_currentLoop=0;
	m_currentTick=0;
	m_growing=m_startSmall;
}

void SizeWowEffect::stop()
{
	m_running=false;
}

void SizeWowEffect::update()
{
	if(m_running)
	{
		m_currentTick++;
		if	(
				(m_growing && m_currentTick==m_growTime) ||
				(!m_growing && m_currentTick==m_shrinkTime)
			)
		{
			m_currentTick=0;
			m_growing=!m_growing;
			m_currentLoop++;
			if(finished())
			{
				m_running=false;
				return;	
			}	
		}
	}
}

bool SizeWowEffect::finished()
{
	if(m_nbLoops==-1)//infini
		return false;
		
	if(m_currentLoop==m_nbLoops)
		return true;
	return false;
}

bool SizeWowEffect::isRunning()
{
	return m_running;
}

OSL_IMAGE * SizeWowEffect::applyOn(OSL_IMAGE *img)
{
	if(img==NULL)
		return NULL;
		
	int addX;
	
	int bigSizeX=int((img->stretchX*(100+m_growPercentX))/100.0f);
	int smallSizeX=int((img->stretchX*(100-m_shrinkPercentX))/100.0f);
	int gainX=bigSizeX-smallSizeX;
	
	int addY;
	int bigSizeY=int((img->stretchY*(100+m_growPercentY))/100.0f);
	int smallSizeY=int((img->stretchY*(100-m_shrinkPercentY))/100.0f);
	int gainY=bigSizeY-smallSizeY;
	
	if(m_growing)
	{		
		int shouldX=int(smallSizeX+(m_currentTick*gainX)/m_growTime);
		addX=shouldX-img->stretchX;
		
		int shouldY=int(smallSizeY+(m_currentTick*gainY)/m_growTime);
		addY=shouldY-img->stretchY;
			
	}
	else //diminue
	{
		int shouldX=int(bigSizeX-((m_currentTick*gainX)/m_shrinkTime));
		addX=shouldX-img->stretchX;
		
		int shouldY=int(bigSizeY-((m_currentTick*gainY)/m_shrinkTime));
		addY=shouldY-img->stretchY;
	}
	img->stretchX+=addX;
	img->stretchY+=addY;
	
	//to be safe
	if(img->stretchX<=0 || img->stretchY<=0)
		return NULL;
	
	return img;
	
}

Resizable *SizeWowEffect::applyOnResizable(Resizable *obj)
{
	//TODO implémenter sur resizable!
	return obj;
}

/*OSL_IMAGE * SizeWowEffect::unapplyOn(OSL_IMAGE *img)
{
	int addX;
	int bigSizeX=(int)(img->sizeX*(100+m_growPercentX))/100;
	int smallSizeX=(int)(img->sizeX*(100-m_shrinkPercentX))/100;
	int gainX=bigSizeX-smallSizeX;
	
	int addY;
	int bigSizeY=(int)(img->sizeY*(100+m_growPercentY))/100;
	int smallSizeY=(int)(img->sizeY*(100-m_shrinkPercentY))/100;
	int gainY=bigSizeY-smallSizeY;
	
	if(m_growing)
	{
		int shouldX=smallSizeX+(m_currentTick*gainX)/m_growTime;
		addX=shouldX-img->stretchX;
		
		int shouldY=smallSizeY+(m_currentTick*gainY)/m_growTime;
		addY=shouldY-img->stretchY;
			
	}
	else //diminue
	{
		int shouldX=bigSizeX-((m_currentTick*gainX)/m_shrinkTime);
		addX=shouldX-img->stretchX;
		
		int shouldY=bigSizeY-((m_currentTick*gainY)/m_shrinkTime);
		addY=shouldY-img->stretchY;
	}
	img->stretchX-=addX;
	img->stretchY-=addY;
	return img;
}*/






/*
OSL_IMAGE * SizeWowEffect::unapplyOn(OSL_IMAGE *img)
{
	int addX;
	int bigSize=(int)(img->sizeX*(100+m_growPercent))/100;
	int smallSize=(int)(img->sizeX*(100-m_shrinkPercent))/100;
	int gain=bigSize-smallSize;
	
	if(m_growing)
	{
		int shouldX=smallSize+(m_currentTick*gain)/m_growTime;
		addX=shouldX-img->stretchX;
	}
	else //diminue
	{
		int shouldX=bigSize-((m_currentTick*gain)/m_shrinkTime);
		addX=shouldX-img->stretchX;
	}
	img->stretchX-=addX;
	return img;
}
*/

SizeWowEffect* SizeWowEffect::clone() const
{
	return new SizeWowEffect(m_shrinkPercentX, m_growPercentX, m_shrinkPercentY, m_growPercentY, m_growTime, m_shrinkTime, m_startSmall, m_nbLoops);
}

int SizeWowEffect::getShrinkingTime()
{
	return m_shrinkTime;
}

void SizeWowEffect::setShrinkingTime(int time)
{
	m_shrinkTime=time;
}

int SizeWowEffect::getGrowingTime()
{
	return m_growTime;
}

void SizeWowEffect::setGrowingTime(int time)
{
	m_growTime=time;
}


int SizeWowEffect::getGrowPercentX()
{
	return m_growPercentX;
}

int SizeWowEffect::getShrinkPercentX()
{
	return m_shrinkPercentX;
}

int SizeWowEffect::getGrowPercentY()
{
	return m_growPercentY;
}

int SizeWowEffect::getShrinkPercentY()
{
	return m_shrinkPercentY;
}





