#include <vector>
#include <string>
#include "code/framework/menus/MenuItem.h"
#include "code/framework/utils.h"
#include "code/framework/bmpFont/BmpFont.h"
#include "code/framework/bmpFont/BmpText.h"
using namespace utils;

MenuItem::MenuItem(string title, BmpFont * f, int id, MenuItem * parent, int nbChildren, int nbValues)
{
	m_selectedValue=-1;
	m_selectedChild=-1;
	m_title=title;
	m_id=id;
	m_subMenus = new vector<MenuItem *>();
	if(nbChildren>0)
	{
		m_subMenus->reserve(nbChildren);
		m_selectedChild=0;	
	}
	m_values = new vector<int>();
	m_valueTitles= new vector<string>();
	if(nbValues>0)
	{
		m_selectedValue=0;
		m_values->reserve(nbValues);
		m_valueTitles->reserve(nbValues);
	}
	m_parent=parent;
	
	BmpText * bmpt=new BmpText(title, f);
	bmpt->compile(); //utile???
	m_image=bmpt->createSwizzledImageCopy();
	delete bmpt;
	oslSetImageRotCenter(m_image);
}

MenuItem::~MenuItem()
{
	vector<MenuItem *>::iterator it;
	for(it=m_subMenus->begin();it<m_subMenus->end();it++)
	{
		delete (*it);
	}
	delete m_subMenus;
	delete m_values;	
	delete m_valueTitles;

	oslDeleteImage(m_image);
}

MenuItem * MenuItem::getParent()
{
	return m_parent;
}

int MenuItem::getSelectedChildNum()
{
	return m_selectedChild;
}

MenuItem * MenuItem::getSelectedChild()
{
	return m_subMenus->at(m_selectedChild);
}

MenuItem * MenuItem::addSubMenu(MenuItem * sm)
{
	m_subMenus->push_back(sm);
	return sm;
}

MenuItem * MenuItem::gotoNextChild()
{
	if(m_subMenus->size()>0)
	{
		m_selectedChild++;
		if(m_selectedChild>=int(m_subMenus->size()))
			m_selectedChild=0;
		return m_subMenus->at(m_selectedChild);
	}
	return NULL;
}

MenuItem * MenuItem::gotoPreviousChild()
{
	if(m_subMenus->size()>0)
	{
		m_selectedChild--;
		if(m_selectedChild<0)
			m_selectedChild=m_subMenus->size()-1;
		return m_subMenus->at(m_selectedChild);
	}
	return NULL;
}

void MenuItem::addValue(string title, int value)
{
	m_values->push_back(value);
	m_valueTitles->push_back(title);
}

int MenuItem::getSelectedValue()
{
	return m_values->at(m_selectedValue);
}

unsigned int MenuItem::getNbValues()
{
	return m_values->size();
}

int MenuItem::gotoNextValue()
{
	if(m_values->size()>0)
	{
		m_selectedValue++;
		if(m_selectedValue>=int(m_values->size()))
			m_selectedValue=0;
		return m_selectedValue;
	}
	return -1;
}

int MenuItem::gotoPreviousValue()
{
	if(m_values->size()>0)
	{
		m_selectedValue--;
		if(m_selectedValue<0)
			m_selectedValue=m_values->size();
		return m_selectedValue;
	}
	return -1;
}

string MenuItem::getValueTitle()
{
	return m_valueTitles->at(m_selectedValue);
}

bool MenuItem::hasChildren()
{
	if(m_subMenus->size()>0)
		return true;
	return false;
}

bool MenuItem::hasValues()
{
	if(m_values->size()>0)
		return true;
	return false;
}

int MenuItem::getId()
{
	return m_id;
}

void MenuItem::setValue(int value)
{
	for(unsigned int i=0;i<m_values->size();i++)
	{
		if(m_values->at(i)==value)
		{
			m_selectedValue=i;
			return;	
		}	
	}
}

int MenuItem::getNbChildren()
{
	return int(m_subMenus->size());
}


MenuItem * MenuItem::getPreviousChild()
{

	if(m_subMenus->size()>0)
	{
		int childNum=m_selectedChild-1;
		if(childNum<0)
			childNum=m_subMenus->size()-1;
		return m_subMenus->at(childNum);
	}
	return NULL;
}

MenuItem * MenuItem::getNextChild()
{
	if(m_subMenus->size()>0)
	{
		int childNum=m_selectedChild+1;
		if(childNum>=int(m_subMenus->size()))
			childNum=0;
		return m_subMenus->at(childNum);
	}
	return NULL;
}

MenuItem * MenuItem::getAfterNextChild()
{
	if(m_subMenus->size()>0)
	{
		int childNum=m_selectedChild+1;
		if(childNum>=int(m_subMenus->size()))
			childNum=0;
		childNum++;
		if(childNum>=int(m_subMenus->size()))
			childNum=0;
		return m_subMenus->at(childNum);
	}
	return NULL;
}

string MenuItem::getTitle()
{
	return m_title;	
}

MenuItem * MenuItem::getBeforePreviousChild()
{
	if(m_subMenus->size()>0)
	{
		int childNum=m_selectedChild-1;
		if(childNum<0)
			childNum=m_subMenus->size()-1;
		childNum--;
		if(childNum<0)
			childNum=m_subMenus->size()-1;
		return m_subMenus->at(childNum);
	}
	return NULL;
}

OSL_IMAGE * MenuItem::getImage()
{
	return m_image;	
}


















