#ifndef PARTICLE_H_
#define PARTICLE_H_

#include <oslib/oslib.h>
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/interfaces/Drawable.h"
#include "code/framework/interfaces/Positionable.h"
#include "code/framework/interfaces/Playable.h"
#include "code/framework/anim/Anim.h"
//#include "code/framework/check/Trackable.h"

class Particle : public Drawable, public Positionable, public Playable//, public Trackable
{
	private:
		//OSL_IMAGE *m_img;
		Anim *m_anim;
		int m_x;
		int m_y;
		
		//jusqu'a 5 effets locaux appliqués
		AbstractLocalEffect *m_lfx1;
		AbstractLocalEffect *m_lfx2;
		AbstractLocalEffect *m_lfx3;
		AbstractLocalEffect *m_lfx4;
		AbstractLocalEffect *m_lfx5;
		
		//jusqu'a 2 effets globaux appliqués
		AbstractGlobalEffect *m_gfx1;		
		AbstractGlobalEffect *m_gfx2;
		
		int m_lifespan; //-1=jusqu'a ce que les effets soient terminés
		int m_lifeSpent;
	
	public:
		Particle(int x, int y, Anim *anim, int lifespan, AbstractLocalEffect *lfx1=NULL, AbstractLocalEffect *lfx2=NULL, AbstractLocalEffect *lfx3=NULL, AbstractLocalEffect *lfx4=NULL, AbstractLocalEffect *lfx5=NULL, AbstractGlobalEffect *gfx1=NULL, AbstractGlobalEffect *gfx2=NULL);
		Particle(int x, int y, Anim *anim, int lifespan, AbstractLocalEffect *lfx1, AbstractGlobalEffect *gfx1=NULL, AbstractGlobalEffect *gfx2=NULL);
		Particle(int x, int y, Anim *anim, int lifespan, AbstractGlobalEffect *gfx1, AbstractGlobalEffect *gfx2=NULL, AbstractLocalEffect *lfx1=NULL, AbstractLocalEffect *lfx2=NULL, AbstractLocalEffect *lfx3=NULL, AbstractLocalEffect *lfx4=NULL, AbstractLocalEffect *lfx5=NULL);
		~Particle();
		
		//drawable
		void drawXY(int x, int y, AbstractLocalEffect *fx1=NULL, AbstractLocalEffect *fx2=NULL, AbstractLocalEffect *fx3=NULL, AbstractLocalEffect *fx4=NULL, AbstractLocalEffect *fx5=NULL);
		void draw(AbstractLocalEffect *fx1=NULL, AbstractLocalEffect *fx2=NULL, AbstractLocalEffect *fx3=NULL, AbstractLocalEffect *fx4=NULL, AbstractLocalEffect *fx5=NULL);
		
		//playable
		void update();
		void start();
		bool isRunning();
		void reset();
		void restart();
		void stop();
		
		//positionable
		int getX();
		int getY();
		void setX(int x);
		void setY(int y);
		
		//statesavable
		State *savestate(State * state);
		void restorestate(State * state);
		
		bool isDead();
		
		AbstractGlobalEffect *getGlobalEffect1();
		AbstractGlobalEffect *getGlobalEffect2();
		
		int getLifeSpent();
		
		int getXwithFX();
		int getYwithFX();
};

//inline

inline void Particle::draw(AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	drawXY(m_x, m_y, fx1, fx2, fx3, fx4, fx5);
}


#endif /*PARTICLE_H_*/





