#include <oslib/oslib.h>
#include "code/framework/particles/complete/impl/ParticleGeneratorSegment.h"
#include "code/framework/utils.h"
#include "code/framework/fx/utils/AlphaParamsState.h"

ParticleGeneratorSegment::ParticleGeneratorSegment(	int x, int y, int direction, int scale, int nbParticles, int generationTime, int movingTime, int dist, int particleLifespan, AbstractAnimationFactory *animFactory, int nbLoops)
:AbstractParticleGenerator(x, y, direction, scale, nbParticles, generationTime, movingTime, dist, particleLifespan, animFactory, nbLoops)
{
	m_scale_2=int(m_scale/2);
	recalculateEdges();
}

ParticleGeneratorSegment::ParticleGeneratorSegment(	int x, int y, int direction, int scale, int nbParticles, int generationTime, int movingTime, int dist, int particleLifespan, string filename, short transparency, int nbLoops)
:AbstractParticleGenerator(x, y, direction, scale, nbParticles, generationTime, movingTime, dist, particleLifespan, filename, transparency, nbLoops)
{
	m_scale_2=int(m_scale/2);
	recalculateEdges();
}

ParticleGeneratorSegment::ParticleGeneratorSegment(	int x, int y, int direction, int scale, int nbParticles, int generationTime, int movingTime, int dist, int particleLifespan, Anim *anim, int nbLoops)
:AbstractParticleGenerator(x, y, direction, scale, nbParticles, generationTime, movingTime, dist, particleLifespan, anim, nbLoops)
{
	m_scale_2=int(m_scale/2);
	recalculateEdges();
}

ParticleGeneratorSegment *ParticleGeneratorSegment::clone() const
{
	ParticleGeneratorSegment *theclone = new ParticleGeneratorSegment(
		m_x, m_y, m_orientation, m_scale, 
		getThroughputNbParticles(), getThroughputFrameTime(), m_movingTime, 
		m_dist, m_particleLifespan, getAnimationFactory()->clone(), getNbLoops()
	);
	
	
	if(m_lfx1!=NULL)
		theclone->setLocalEffect1(m_lfx1->clone());
	if(m_lfx2!=NULL)
		theclone->setLocalEffect2(m_lfx2->clone());
	if(m_lfx3!=NULL)
		theclone->setLocalEffect3(m_lfx3->clone());
	if(m_lfx4!=NULL)
		theclone->setLocalEffect4(m_lfx4->clone());
	if(m_lfx5!=NULL)
		theclone->setLocalEffect5(m_lfx5->clone());
	if(m_gfx1!=NULL)
		theclone->setGlobalEffect1(m_gfx1->clone());
	if(m_gfx2!=NULL)
		theclone->setGlobalEffect2(m_gfx2->clone());
	
	return theclone;
}

ParticleGeneratorSegment::~ParticleGeneratorSegment()
{
	//le parent fait tout
}


Particle *ParticleGeneratorSegment::generateNextParticle()
{	
	//distance entre A et B au hasard
	int randDist=utils::random(0, m_scale);
	
	//point de depart
	int orientation=m_orientation+90; //de A a B
	int startY=m_ya+oslSini(-orientation, randDist);
	int startX=m_xa+oslCosi(-orientation, randDist);
	
	//creation des effets pour la particule
	AbstractLocalEffect *lfx1=NULL;	
	if(m_lfx1!=NULL)
		lfx1=m_lfx1->newLocalEffectInstance(startX, startY, m_dist, m_movingTime, m_orientation, m_scale, m_particleLifespan);
	AbstractLocalEffect *lfx2=NULL;
	if(m_lfx2!=NULL)
		lfx2=m_lfx2->newLocalEffectInstance(startX, startY, m_dist, m_movingTime, m_orientation, m_scale, m_particleLifespan);
	AbstractLocalEffect *lfx3=NULL;
	if(m_lfx3!=NULL)
		lfx3=m_lfx3->newLocalEffectInstance(startX, startY, m_dist, m_movingTime, m_orientation, m_scale, m_particleLifespan);
	AbstractLocalEffect *lfx4=NULL;
	if(m_lfx4!=NULL)
		lfx4=m_lfx4->newLocalEffectInstance(startX, startY, m_dist, m_movingTime, m_orientation, m_scale, m_particleLifespan);
	AbstractLocalEffect *lfx5=NULL;
	if(m_lfx5!=NULL)
		lfx5=m_lfx5->newLocalEffectInstance(startX, startY, m_dist, m_movingTime, m_orientation, m_scale, m_particleLifespan);
	AbstractGlobalEffect *gfx1=NULL;		
	if(m_gfx1!=NULL)
		gfx1=m_gfx1->newGlobalEffectInstance(startX, startY, m_dist, m_movingTime, m_orientation, m_scale, m_particleLifespan);	
	AbstractGlobalEffect *gfx2=NULL;	
	if(m_gfx2!=NULL)
		gfx2=m_gfx2->newGlobalEffectInstance(startX, startY, m_dist, m_movingTime, m_orientation, m_scale, m_particleLifespan);
	
	Particle * p=new Particle(m_x, m_y, generateAnimation(), m_particleLifespan, lfx1, lfx2, lfx3, lfx4, lfx5, gfx1, gfx2);
	
	return p;
}



void ParticleGeneratorSegment::setScale(int scale)
{
	AbstractParticleGenerator::setScale(scale);
	m_scale_2=int(m_scale/2);
	recalculateEdges();
}



void ParticleGeneratorSegment::setOrientation(int orientation)
{
	AbstractParticleGenerator::setOrientation(orientation);
	recalculateEdges();
}

void ParticleGeneratorSegment::recalculateEdges()
{
	int angle=-m_orientation-90; //- parce que le cercle trigo est inversé
	
	int cos=oslCosi(angle, m_scale_2);
	int sin=oslSini(angle, m_scale_2);
	
	//basé sur le fait que la pos du generateur est 0,0
	//A est en bas et B en haut quand orientation=0
	m_xa=-cos;
	m_ya=-sin;
	
	m_xb=cos;
	m_yb=sin;
}

int ParticleGeneratorSegment::getXA()
{
	return m_xa;
}

int ParticleGeneratorSegment::getYA()
{
	return m_ya;
}

int ParticleGeneratorSegment::getXB()
{
	return m_xb;
}

int ParticleGeneratorSegment::getYB()
{
	return m_yb;
}














