#include "code/framework/utils.h"
#include <string>
#include <stdlib.h>
#include <time.h>
#include <fstream>
#include <pspiofilemgr.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include "code/framework/check/loaders.h"
#include "code/framework/check/PrettyFileLogger.h"


using namespace std;

bool debugOn= false;

string utils::replaceAll(string in, string find, string replacement)
{
	string res=in;
	unsigned int pos=0;
	int findSz=find.size();
	while((pos=res.find(find))!=string::npos)
	{
		//trouvé à pos, on remplace
		res.replace(pos, findSz, replacement);
	}
	return res;
}

string utils::deleteAll(string in, string find)
{
	return replaceAll(in, find, string(""));
}

string utils::testReplace()
{
	string test="XabcdefXghijX";
	string repl="-_-";
	string find="X";
	string res=replaceAll(test, find, repl);
	return res;
}

string utils::testReadLine()
{
	ifstream fichier("res/bob.txt");
	string s;
	/*while(std::getline(fichier,s))
	{
		
	}*/
	
	getline(fichier, s);
	s=replaceAll(s, string("o"), string(""));
	s=deleteAll(s, string("\r"));
	return s;
}

void utils::debug(string msg)
{
	if(/*utils::*/debugOn)
		oslMessageBox(msg.c_str(), "debug", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, OSL_KEY_CIRCLE, OSL_MB_CANCEL, 0, 0));
	/*{
		oslInitMessageDialog(msg.c_str(), 0);
		//bool ok=false;
		
		while(oslGetDialogStatus() != OSL_DIALOG_NONE)
		{
			oslStartDrawing();
			oslClearScreen(RGB(0,0,0));
			oslDrawDialog();
			if(oslGetDialogStatus() == OSL_DIALOG_NONE)
            {
//            	int button = oslGetDialogButtonPressed();
//				if (button == 1 || button == 2 || button == 3)
//                {
//					ok=true;
//                }
				oslEndDialog();
            }
            oslEndDrawing();
            oslEndFrame();
        	oslSyncFrame();
		}
	}*/
}

void utils::enableDebug(bool enable)
{
	/*utils::*/debugOn=enable;
}

vector<string> utils::split(string str, string delim)
{
	vector<string> results;
	unsigned int cutAt;
	while((cutAt = str.find_first_of(delim)) != str.npos)
	{
		if(cutAt > 0)
		{
			results.push_back(str.substr(0,cutAt));
		}
		str = str.substr(cutAt+1);
	}
	if(str.length() > 0)
	{
		results.push_back(str);
	}
	return results;
}

//int utils::random(int min, int max) //inclus
//{
//	if(!randomIsInit)
//	{
//		srand(time(NULL));
//		randomIsInit=true;
//	}
//	int nb=max-min+1; //nombre de valeurs possibles
//	int r=rand()%nb + min;
//	return r; 
//}

long int utils::wavDuration(string wavfile) //en ms
{
	char riff[5];
	char wave[5];
	char fmt[5];
	char data[5];
	
	unsigned long int length=0, binary=0, stereo=0, bits=0, sample=0, bps=0, bpsam=0, bitsps=0, sound=0;
	int s=0;
	
	FILE *wav=fopen(wavfile.c_str(),"rb+");
	if(wav==NULL)
	{
		fatalError(("cannot open file "+wavfile).c_str());
	}
	
	/* To start the RIFF Chunk must be determined*/
	fread(riff,1,4,wav);
	riff[4]='\0';
	string str_riff=string(riff);
	if(str_riff.find("RIFF")!=0)
	{
	    fclose(wav);
		fatalError("error in header not wav file");
	}
	
	fread(&length,4,1,wav);
	
	fread(wave,1,4,wav);
	wave[4]='\0';
	string str_wave=string(wave);
	if(str_wave.find("WAVE")!=0)
	{
		fclose(wav);
		fatalError("error not in WAV format");
	}
	
	/* Secondly the FORMAT Chunk must be determined*/
	fread(fmt,1,4,wav);
	fmt[4]='\0';
	string str_fmt=string(fmt);
	if(str_fmt.find("fmt")!=0)
	{
	    fclose(wav);
		fatalError("error, no fmt chunk");
	}
	
	//TODO: fseek 16
	fread(&binary,4,1,wav);	
	fread(&bits,2,1,wav);
	fread (&stereo,2,1,wav);		
	fread(&sample,4,1,wav);
	fread(&bps,4,1,wav);
	
	
	fread(&bpsam,2,1,wav);
	if (bpsam==0x01)
	{
		//this is 8 bit mono
		s=1;
	}
	else if (bpsam==0X02)
	{
		//this is 8 bit stereo or 16 bit mono
		s=2;
	}		
	else if (bpsam==0X04)
	{
		//this is 16 bit stereo
		s=4;	
	}
	else
	{
		fatalError("impossible to determine bpsam of file");
	}
	
	fread(&bitsps,2,1,wav);
	
	/* Thirdly the DATA Chunk must be determined*/
	//must find it
	bool found=false;
	char c[1]={'0'};
	long int ft=ftell(wav);
	while(!found)
	{
		fread(c,1,1,wav);
		ft++;
		
		if(c[0]=='d')
		{
			fread(data,1,3,wav);
			data[3]='\0';
			string str_data=string(data);
			if (str_data.find("ata")==0)
			{
				found=true;
			}
			else
			{
				fseek(wav, ft, SEEK_SET);
			}
		}
	}
	
	
	fread(&sound,4,1,wav);
	
	fclose(wav);
	
	/* Convert the length of the file into milliseconds */
	float result=(float(sound)/float(s*sample))*1000.0f;
	
	return (long int)(result);
}

long int utils::bgmDuration(string bgmfile) //en ms
{
	OSL_SOUND *snd=oslLoadSoundFileBGM(bgmfile.c_str(), OSL_FMT_STREAM);
	
	long int bytes_total=snd->size;
	long int bytes_header=sizeof(BGM_FORMAT_HEADER);
	long int bytes_data=bytes_total-bytes_header;
		
	oslDeleteSound(snd);
	
	float divider=22050.0f;
	float result=(float(bytes_data)/divider)*1000.0f;
			
	return (long int)(result);
}

/*
void utils::drawImageOnImage(OSL_IMAGE *src, int srcX, int srcY, int width, int height, OSL_IMAGE *dest, int destX, int destY)
{
	oslLockImage(src);
	oslLockImage(dest);
	for(int x=0; x<width; x++)
	{
		for(int y=0; y<height; y++)
		{
			int putX=destX+x;
			int putY=destY+y;
			if(putX<dest->sizeX && putY<dest->sizeY)
			{
				int getX=srcX+x;
				int getY=srcY+y;
				
				int pixeldest=oslGetImagePixel(dest, putX, putY);
				int pixel8888dest=oslConvertColorEx(dest->palette, OSL_PF_8888, dest->pixelFormat, pixeldest);
				
				int dr, dg, db, da; //d: dest
				oslRgbaGet8888(pixel8888dest, dr, dg, db, da);
								
				int pixelsrc=oslGetImagePixel(src, getX, getY);
				int pixel8888src=oslConvertColorEx(src->palette, OSL_PF_8888, src->pixelFormat, pixelsrc);
				int sr, sg, sb, sa; //s: src
				oslRgbaGet8888(pixel8888src, sr, sg, sb, sa);
									
				int pixel8888result=0;
				if(sa+da>0)
				{
					int da_left=255-sa; //alpha left
					
					int rr, rg, rb, ra; //r: result
					
					//>>8 equivalent a div par 256
					//&0xff inutile
					rr=((dr*da_left + sr*sa)>>8)&0xff;
					rg=((dg*da_left + sg*sa)>>8)&0xff;
					rb=((db*da_left + sb*sa)>>8)&0xff;
					ra=sa + (((da_left*da)>>8)&0xff);
					pixel8888result=RGBA(rr,rg,rb,ra);
				}
				
				
				int pixelresult = oslConvertColorEx(dest->palette, dest->pixelFormat, OSL_PF_8888, pixel8888result);
				
				oslSetImagePixel(dest, putX, putY, pixelresult);
			}
		}
	}
	oslUnlockImage(dest);
	oslUnlockImage(src);
}
*/

void utils::drawImageOnImage(OSL_IMAGE *src, int srcX, int srcY, int width, int height, OSL_IMAGE *dest, int destX, int destY)
{
	oslLockImage(src);
	oslLockImage(dest);
	for(int x=0; x<width; x++)
	{
		for(int y=0; y<height; y++)
		{
			int putX=destX+x;
			int putY=destY+y;
			if(putX<dest->sizeX && putY<dest->sizeY)
			{
				int getX=srcX+x;
				int getY=srcY+y;
				
				int pixeldest=oslGetImagePixel(dest, putX, putY);
				int pixel8888dest=oslConvertColorEx(dest->palette, OSL_PF_8888, dest->pixelFormat, pixeldest);
				
				int dr, dg, db, da; //d: dest
				oslRgbaGet8888(pixel8888dest, dr, dg, db, da);
								
				int pixelsrc=oslGetImagePixel(src, getX, getY);
				int pixel8888src=oslConvertColorEx(src->palette, OSL_PF_8888, src->pixelFormat, pixelsrc);
				int sr, sg, sb, sa; //s: src
				oslRgbaGet8888(pixel8888src, sr, sg, sb, sa);
									
				int pixel8888result=0;
				if(sa+da>0)
				{
					
					int da_left=255-sa; //alpha left
					//int asum=sa+da;
					
					float rr, rg, rb, ra; //r: result
					
					
					ra=sa + ((da_left*da)/255);
					rr=(dr * da_left + sr * sa) / ra;
					rg=(dg * da_left + sg * sa) / ra;
					rb=(db * da_left + sb * sa) / ra;
					if(ra>255)
						ra=255;
					if(rr>255)
						rr=255;
					if(rg>255)
						rg=255;
					if(rb>255)
						rb=255;
					
					pixel8888result=RGBA(int(rr),int(rg),int(rb),int(ra));
					
				}
				
				
				int pixelresult = oslConvertColorEx(dest->palette, dest->pixelFormat, OSL_PF_8888, pixel8888result);
				
				oslSetImagePixel(dest, putX, putY, pixelresult);
			}
		}
	}
	oslUnlockImage(dest);
	oslUnlockImage(src);
}

list<string> *utils::directoryListFiles(string folder)
{
	list<string> *result=new list<string>();
	
	SceIoDirent dir;
	int dfd, ok;
	//char path[]="res/profiles/";
	
	dfd = sceIoDopen(folder.c_str());
	if (dfd>=0)
	{
	   	for (;;)
	   	{
	   		
		    memset(&dir, 0, sizeof dir);
		    ok = sceIoDread(dfd, &dir);
		    if (ok<=0) 
		    	break;
		      
		    //extract info from dir
		    if(dir.d_stat.st_attr & FIO_SO_IFREG) //c'est un fichier
		    {
		    	string s(dir.d_name);
		      	if(s!="." && s!="..")
		      	{
			      	result->push_back(s);
		      	}
			}
	   	}
	   	sceIoDclose(dfd);
	}
	
	return result;
}

list<string> *utils::directoryListFolders(string folder)
{
	list<string> *result=new list<string>();
	
	SceIoDirent dir;
	int dfd, ok;
	//char path[]="res/profiles/";
	
	dfd = sceIoDopen(folder.c_str());
	if (dfd>=0)
	{
	   	for (;;)
	   	{
	   		
		    memset(&dir, 0, sizeof dir);
		    ok = sceIoDread(dfd, &dir);
		    if (ok<=0) 
		    	break;
		      
		    //extract info from dir
		    if(dir.d_stat.st_attr & FIO_SO_IFDIR) //c'est un dossier
		    {
		    	string s(dir.d_name);
		      	if(s!="." && s!="..")
		      	{
			      	result->push_back(s);
		      	}
			}
	   	}
	   	sceIoDclose(dfd);
	}
	
	return result;
}

bool utils::fileExists(string file)
{
	ifstream inp;
	inp.open(file.c_str(), ifstream::in);
	inp.close();
	if(inp.fail())
	{
		inp.clear(ios::failbit);
		return false;
	}
	return true;
}

bool utils::directoryExists(string dir)
{
	int dfd = sceIoDopen(dir.c_str());
	if(dfd<0)
		return false;
	//else
	
	sceIoDclose(dfd);
	return true;
}

string utils::getCurrentDirectory()
{
	char currentdir[256];
	getcwd(currentdir, 256);
	return string(currentdir)+"/";
}

//bool utils::createDirectory(string dir)
//{
//	if(dir.at(dir.length()-1)=='/') //faut enlever le dernier '/'
//	{
//		dir=dir.substr(0, dir.length()-1);
//	}
//	int r=sceIoMkdir(dir.c_str(), 0777);
//	if(r!=0)
//		return false;
//	return true;
//}

bool utils::createDirectory(string dir)
{

	//faut pas créer de répertoire nommé "."
	dir=replaceAll(dir, "./", "");
	
	if(dir.at(dir.length()-1)!='/') //faut mettre le dernier '/'
	{
		dir=dir+"/";
	}
	

	
	unsigned int slashpos=dir.find('/'); //position du prochain '/'
	while(slashpos != string::npos)
	{

		string curdir=dir.substr(0, slashpos);

		if(curdir.length()!=0 && !directoryExists(curdir))
		{
			int r=sceIoMkdir(curdir.c_str(), 0777);
			if(r!=0)
				return false;
		}	
		slashpos=dir.find('/', slashpos+1);
	}
	
	return true;
}


//ex: avec "dans/mon/dossier/unfichier.ext" :
string utils::filepath_getpath(string filepath) //renvoie dans/mon/dossier/"
{
	int pos=filepath.rfind('/');
	return filepath.substr(0, pos+1);
}

string utils::filepath_getfile(string filepath) //renvoie "unfichier.ext"
{
	int pos=filepath.rfind('/');
	return filepath.substr(pos+1, filepath.length()-pos-1);
}

int utils::oslAnalogGetAngle(int x, int y)
{

	if(x==0)
	{
		if(y==0)
		{

			return 0;
		}
		if(y>0)
		{

			return 90;
		}

		return 270;
	}
	
	//x!=0:
	int res=int(57.2957795f*vfpu_atanf(float(-y)/float(x)));

	if(res<0)
		res+=360;
	if(x<0)
		return res+180;
	return res;
}

OSL_IMAGE *utils::getBufferCopy()
{
	OSL_IMAGE *pixelcopy=oslCreateImage(480, 272, OSL_IN_RAM, OSL_PF_8888);
	if(pixelcopy==NULL)
	{
		fatalError("utils::getBufferCopy : impossible de creer une image");
	}
	oslSyncDrawing();
	oslLockImage(OSL_SECONDARY_BUFFER);
	oslLockImage(pixelcopy);
	for(int i=0;i<480;i++)
	{
		for(int j=0;j<272;j++)
		{
			//int pixel=oslConvertColorEx(OSL_DEFAULT_BUFFER ->palette, OSL_PF_8888, OSL_DEFAULT_BUFFER ->pixelFormat, oslGetImagePixel(OSL_DEFAULT_BUFFER , i, j));
			int pixelsrc=oslGetImagePixel(OSL_SECONDARY_BUFFER, i, j);
			int pixel8888src=oslConvertColorEx(OSL_SECONDARY_BUFFER->palette, OSL_PF_8888, OSL_SECONDARY_BUFFER->pixelFormat, pixelsrc);
			int r, g, b, a;
			oslRgbaGet8888(pixel8888src, r, g, b, a);
			int pixel8888result=RGBA(r,g,b,255);
			
			oslSetImagePixel(pixelcopy, i, j, pixel8888result);
		}
	}
	oslUnlockImage(pixelcopy);
	oslUnlockImage(OSL_SECONDARY_BUFFER);
	
	oslSwizzleImage(pixelcopy);
	
	return pixelcopy;
}

bool utils::deleteDirectory(string dir) //efface un répertoire et tous ses fichiers/dossiers
{
	if(endsWith(dir, "/"))
		dir.resize(dir.size()-1);
	//enableDebug();
	debug("deleteDirectory '"+dir+"'");
	if(directoryExists(dir))
	{
		//ON DELETE LES FICHIERS A L'INTERIEUR
		list<string> *files=directoryListFiles(dir);
		list<string>::iterator fit;
		for (fit=files->begin(); fit!=files->end(); fit++)
		{
			string file=dir+"/"+(*fit);
			debug("delete file '"+file+"'");
			if(sceIoRemove(file.c_str())!=0) //failure
			{
				fatalError("cannot delete file '"+file+"'");
			}
		}
		
		//ON DELETE LES DOSSIERS A L'INTERIEUR
		list<string> *dirs=directoryListFolders(dir);
		list<string>::iterator dit;
		for (dit=dirs->begin(); dit!=dirs->end(); dit++)
		{
			string innerdir=dir+"/"+(*dit);
			debug("found dir '"+innerdir+"'");
			deleteDirectory(innerdir);
		}
		
		//ON DELETE LE DOSSIER LUI MEME
		debug("delete dir '"+dir+"'");
		//dir=dir+"/";
		if(sceIoRmdir(dir.c_str())!=0) //failure
		{
			fatalError("cannot delete directory '"+dir+"'");
		}
	}
		
	return true;
}

void utils::fatalError(string message)
{
	PFL::logMessage("utils::fatalError : "+message);
	oslFatalError((char *)message.c_str());
}

void utils::message(string msg)
{
	oslMessageBox(msg.c_str(), "message", oslMake3Buttons(OSL_KEY_CROSS, OSL_MB_OK, OSL_KEY_CIRCLE, OSL_MB_CANCEL, 0, 0));
}

bool utils::endsWith(string str, string with)
{
	string end=str.substr(str.size()-with.size());
	if(end==with)
		return true;
		
	return false;
}


























