#include "code/game/SongBanner.h"
#include <string>

using namespace std;

#define SONGBANNER_X 10
#define SONGBANNER_TRANSL_TIME 210
#define SONGBANNER_SHRINK_TIME 10
#define SONGBANNER_GROW_TIME 10
#define SONGBANNER_WAIT_TIME 100

SongBanner::SongBanner(int y, string fontfolder, string fontname, string fontnameitalic, string line1, string line2, string line3, string line4)
{
	m_y=y;
	
	m_fx_transl_come=new TranslationEffect(480, 0, 0, 0, SONGBANNER_TRANSL_TIME, SONGBANNER_TRANSL_TIME, true, 1); 
//	unsigned short shrinkPercentX, unsigned int growPercentX, unsigned short shrinkPercentY, unsigned int growPercentY, unsigned short growTime, unsigned short shrinkTime, bool startSmall=true, short nbLoops=-1
	m_fx_shrink=new SizeWowEffect(0, 0, 100, 0, SONGBANNER_SHRINK_TIME, SONGBANNER_SHRINK_TIME, false, 1);
	m_fx_grow=new SizeWowEffect(0, 0, 100, 0, SONGBANNER_GROW_TIME, SONGBANNER_GROW_TIME, true, 1);
	m_fx_transl_go=new TranslationEffect(0, 0, -480, 0, SONGBANNER_TRANSL_TIME, SONGBANNER_TRANSL_TIME, true, 1); 
	m_fx_wait=new WaitLocalEffect(SONGBANNER_WAIT_TIME);
	
	//on compte le nombre de textes dispo
	m_nbtxt=1; //line1 obligatoire
	string last=line1; //on repere quelle est la derniere
	if(line2.size()>0)
	{
		m_nbtxt++;
		last=line2;
	}
	if(line3.size()>0)
	{
		m_nbtxt++;
		last=line3;
	}
	if(line4.size()>0)
	{
		m_nbtxt++;
		last=line4;
	}
		
	m_currenttxt=0;
	m_stage=SONGBANNER_STAGE_COMING;
	m_fx_transl_come->start();
	
	m_txt=new OSL_IMAGE *[m_nbtxt];
	
	//BmpFont::initBmpFont();
	BmpFont * fontnormal=new BmpFont(fontfolder, fontname, OSL_PF_4444);
	BmpFont * fontitalic=new BmpFont(fontfolder, fontnameitalic, OSL_PF_4444, true);
	
	BmpText *txt1=new BmpText(line1, fontnormal);
	txt1->compile();
	m_txt[0]=txt1->createSwizzledImageCopy();
	delete txt1;
	setImageLeftCenter(m_txt[0]);
	
	if(line2.size()>0)
	{
		BmpText *txt2=new BmpText(line2, fontnormal);
		txt2->compile();
		m_txt[1]=txt2->createSwizzledImageCopy();
		delete txt2;
		setImageLeftCenter(m_txt[1]);
	}
	if(line3.size()>0)
	{
		BmpText *txt3=new BmpText(line3, fontnormal);
		txt3->compile();
		m_txt[2]=txt3->createSwizzledImageCopy();
		delete txt3;
		setImageLeftCenter(m_txt[2]);
	}
	if(line4.size()>0)
	{
		BmpText *txt4=new BmpText(line4, fontnormal);
		txt4->compile();
		m_txt[3]=txt4->createSwizzledImageCopy();		
		delete txt4;
		setImageLeftCenter(m_txt[3]);
	}
	
	BmpText *txtitalic1=new BmpText(line1, fontitalic);
	txtitalic1->compile();
	m_txt_italic_begin=txtitalic1->createSwizzledImageCopy();
	delete txtitalic1;
	setImageLeftCenter(m_txt_italic_begin);
	
	if(m_nbtxt>1)
	{
		BmpText *txtitalic2=new BmpText(last, fontitalic);
		txtitalic2->compile();
		m_txt_italic_end=txtitalic2->createSwizzledImageCopy();
		delete txtitalic2;
		setImageLeftCenter(m_txt_italic_end);
	}
	else
	{
		m_txt_italic_end=m_txt_italic_begin;
	}
	
	delete fontitalic;
	delete fontnormal;
	
	//BmpFont::freeRes();
	
	m_finished=false;
	
	m_bg=NULL;
	
	m_totalTimePresent=SONGBANNER_TRANSL_TIME*2 + SONGBANNER_WAIT_TIME + (SONGBANNER_SHRINK_TIME + SONGBANNER_GROW_TIME + SONGBANNER_WAIT_TIME)*(m_nbtxt-1);
}

SongBanner::~SongBanner()
{
	delete m_fx_grow;
	delete m_fx_shrink;
	delete m_fx_transl_come;
	delete m_fx_transl_go;
	delete m_fx_wait;
	
	oslDeleteImage(m_txt_italic_begin);
	if(m_nbtxt>1)
	{
		oslDeleteImage(m_txt_italic_end); //sinon il pointe sur le 1er
	}
	
	for(int i=0;i<m_nbtxt;i++)
	{
		oslDeleteImage(m_txt[i]);
	}
	delete[] m_txt;
	
	if(m_bg!=NULL)
		oslDeleteImage(m_bg);
}

void SongBanner::reset()
{
	m_fx_grow->stop();
	m_fx_shrink->stop();
	m_fx_transl_come->stop();
	m_fx_transl_go->stop();
	m_fx_wait->stop();
	
	m_fx_grow->reset();
	m_fx_shrink->reset();
	m_fx_transl_come->reset();
	m_fx_transl_go->reset();
	m_fx_wait->reset();
	
	m_currenttxt=0;
	m_stage=SONGBANNER_STAGE_COMING;
	m_fx_transl_come->start();
	
	m_finished=false;
}

void SongBanner::setBgImage(OSL_IMAGE *bg)
{
	m_bg=bg;
	//setImageLeftCenter(m_bg);
	//m_bg->stretchX=480;
}

void SongBanner::setImageLeftCenter(OSL_IMAGE *img)
{
	img->centerX=0;
	img->centerY=int(img->sizeY/2);
}

void SongBanner::draw()
{
	if(!m_finished)
	{
		if(m_bg!=NULL)
		{
			oslDrawImage(m_bg);
		}
		
		
		if(m_stage==SONGBANNER_STAGE_COMING)
		{
			fx::drawXY(m_txt_italic_begin, SONGBANNER_X, m_y, m_fx_transl_come);
		}
		else if(m_stage==SONGBANNER_STAGE_WAITING)
		{
			fx::drawXY(m_txt[m_currenttxt], SONGBANNER_X, m_y, m_fx_wait);
		}
		else if(m_stage==SONGBANNER_STAGE_SHRINK)
		{
			fx::drawXY(m_txt[m_currenttxt], SONGBANNER_X, m_y, m_fx_shrink);
		}
		else if(m_stage==SONGBANNER_STAGE_GROW)
		{
			fx::drawXY(m_txt[m_currenttxt], SONGBANNER_X, m_y, m_fx_grow);
		}
		else if(m_stage==SONGBANNER_STAGE_GO)
		{
			fx::drawXY(m_txt_italic_end, SONGBANNER_X, m_y, m_fx_transl_go);
		}
	}
}

void SongBanner::update()
{
	if(!m_finished)
	{
		if(m_stage==SONGBANNER_STAGE_COMING)
		{
			m_fx_transl_come->update();
			if(m_fx_transl_come->finished())
			{
				m_stage=SONGBANNER_STAGE_WAITING;
				m_fx_wait->start();
			}
		}
		else if(m_stage==SONGBANNER_STAGE_WAITING)
		{
			m_fx_wait->update();
			if(m_fx_wait->finished())
			{
				//est qu'on est au dernier ou pas?
				if(m_currenttxt==m_nbtxt-1) //c'est le dernier
				{
					m_stage=SONGBANNER_STAGE_GO;
					m_fx_transl_go->start();
				}
				else //y en a d'autres
				{
					m_stage=SONGBANNER_STAGE_SHRINK;
					m_fx_shrink->restart();
				}
			}
		}
		else if(m_stage==SONGBANNER_STAGE_SHRINK)
		{
			m_fx_shrink->update();
			if(m_fx_shrink->finished())
			{
				m_stage=SONGBANNER_STAGE_GROW;
				m_currenttxt++;
				m_fx_grow->restart();
			}
		}
		else if(m_stage==SONGBANNER_STAGE_GROW)
		{
			m_fx_grow->update();
			if(m_fx_grow->finished())
			{
				m_stage=SONGBANNER_STAGE_WAITING;
				m_fx_wait->restart();
			}
		}
		else if(m_stage==SONGBANNER_STAGE_GO)
		{
			m_fx_transl_go->update();
			if(m_fx_transl_go->finished())
			{
				m_finished=true;
			}
		}
	}
}

int SongBanner::getTotalTimePresent()
{
	return m_totalTimePresent;
}



