#include "code/game/levels/abstract/AbstractLevel.h"

#include "code/game/gamedef.h"
#include "code/game/pieces/impl/PieceSimple.h"

/*AbstractLevel::AbstractLevel(int nbLines, int nbCols, int initialDropSpeed, int initialUpSpeed, int difficulty, int nbPlayers)
{
	m_initialDropSpeed=initialDropSpeed;
	m_initialUpSpeed=initialUpSpeed;
	m_nbLines=nbLines;
	m_nbCols=nbCols;
	m_difficulty=difficulty;
	m_nbPlayers=nbPlayers;
}*/

AbstractLevel::AbstractLevel(int initialDropSpeed, int initialUpSpeed, int difficulty, int nbPlayers)
{
	m_initialDropSpeed=initialDropSpeed;
	m_initialUpSpeed=initialUpSpeed;
	m_nbLines=GAMEDEF_NB_LINES;
	m_nbCols=GAMEDEF_NB_COLS;
	m_difficulty=difficulty;
	m_nbPlayers=nbPlayers;
}

int AbstractLevel::getInitialDropSpeed()
{
	return m_initialDropSpeed;
}

int AbstractLevel::getInitialUpSpeed()
{
	return m_initialUpSpeed;
}

void AbstractLevel::update(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	if(m_paused)
	{
//		enableDebug();
//		debug("AbstractLevel::update : update during pause");
//		enableDebug(false);
		update_pause_specific(cc, grid, nextPair, currentPair, fx_trans);
	}
	else
	{
		update_specific(cc, grid, nextPair, currentPair, fx_trans);
	}
}

void AbstractLevel::pause()
{
	m_paused=true;
	pause_specific();
}

void AbstractLevel::start()
{
	m_paused=false;
	start_specific();	
}

bool AbstractLevel::isPaused()
{
	return m_paused;	
}

void AbstractLevel::resume()
{
	m_paused=false;
	resume_specific();
}

int AbstractLevel::getNbLines()
{
	return m_nbLines;
}

int AbstractLevel::getNbCols()
{
	return m_nbCols;
}

int AbstractLevel::getFastDropSpeed()
{
	return GAMEDEF_FAST_DROP_SPEED;
}

int AbstractLevel::getPieceUnconnectedDropSpeed()
{
	return GAMEDEF_PIECE_UNCONNECTED_DROP_SPEED;	
}

int AbstractLevel::getMinimumWaitBeforeNewPair()
{
	return GAMEDEF_MINIMUM_WAIT;
}

int AbstractLevel::getDifficulty()
{
	return m_difficulty;
}

int AbstractLevel::getNbPlayers()
{
	return m_nbPlayers;
}

Pair *AbstractLevel::getNewPair()
{
	int posX=int(m_nbCols/2)-1;
	int posY=-1; //au dessus de tout en haut
	Pair *p=new Pair(new PieceSimple(), new PieceSimple());
	p->goTo(posX, posY);
	
	return p;
}

bool AbstractLevel::isGameOver()
{
	return false;
}
