#include "code/game/levels/impl/8bits/Level8Bits.h"

#include "code/framework/Pos.h"
#include "code/game/gamedef.h"
#include "code/framework/utils.h"
#include <oslib/oslib.h>
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/framework/Properties.h"
#include "code/game/pieces/impl/PieceSimple.h"
#include "code/framework/anim/Anim.h"
#include "code/game/levels/utils/MCOContentSimple.h"
#include "code/framework/particles/complete/animFactories/ControlledAnimationFactory.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomGravity.h"
#include "code/framework/check/loaders.h"

using namespace utils;

#define LEVEL_8BITS_DURATION_EURHAWT 187000.0f
#define LEVEL_8BITS_DURATION_BRUTUS 196800.0f
#define LEVEL_8BITS_DURATION_HEDGEHOG 163000.0f

#define LEVEL_8BITS_COMBO_COLOR_NORMAL_IN RGBA(255,255,255,255)
#define LEVEL_8BITS_COMBO_COLOR_NORMAL_OUT RGB(0,0,0)
#define LEVEL_8BITS_COMBO_COLOR_HOT1 RGBA(230,230,230,255)
#define LEVEL_8BITS_COMBO_COLOR_HOT2 RGBA(255,176,0,255)
#define LEVEL_8BITS_COMBO_COLOR_HOT3 RGBA(0,0,190,255)

#define LEVEL_8BITS_PART_EURHAWT 0
#define LEVEL_8BITS_PART_BRUTUS 1
#define LEVEL_8BITS_PART_HEDGEHOG 2


string Level8Bits::getCode()
{
	return "8bits";
}

Level8Bits::Level8Bits(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{	
	//enableDebug();
	//debug("properties");
	Properties *props=new Properties("res/levels/8bits/8bits.properties");
	
	m_color_pause=RGB(255,0,255);
	
	m_difficulty=difficulty;
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		m_linepattern=new LinePattern("res/levels/8bits/8bits_easy.pattern");
		m_songbanner1=new SongBanner(GAMEDEF_SONGBANNER_Y+3, "res/fonts/bmpFonts/russelsquare/13/", "normal", "italic", props->getStrValue("easy.songbanner.text1"), props->getStrValue("easy.songbanner.text2", ""), props->getStrValue("easy.songbanner.text3", ""), props->getStrValue("easy.songbanner.text4", ""));
		m_songbanner2=NULL;
		m_music1=fwkLoadSoundFileMP3("res/levels/8bits/eurhawt.mp3", OSL_FMT_STREAM);
		m_music2=NULL;
		m_nbCols=props->getIntValue("easy.grid.width");
		m_scenario1=new EventsScenario("res/levels/8bits/eurhawt.scenario");
		m_scenario2=NULL;
		
		m_upspeed=props->getIntValue("easy.upspeed");
		m_dropspeed=props->getIntValue("easy.dropspeed");
		
		m_duration=LEVEL_8BITS_DURATION_EURHAWT;
		m_part=LEVEL_8BITS_PART_EURHAWT;
		
		m_addline=props->getIntValue("easy.lines.start");
		
		m_img_overlay=fwkLoadImageFilePNG("res/levels/8bits/grid12.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_songbanner1=new SongBanner(GAMEDEF_SONGBANNER_Y+3, "res/fonts/bmpFonts/russelsquare/13/", "normal", "italic", props->getStrValue("normal.songbanner.text1"), props->getStrValue("normal.songbanner.text2", ""), props->getStrValue("normal.songbanner.text3", ""), props->getStrValue("normal.songbanner.text4", ""));
		m_linepattern=new LinePattern("res/levels/8bits/8bits_normal.pattern");
		m_songbanner2=NULL;
		m_music1=fwkLoadSoundFileMP3("res/levels/8bits/brutus.mp3", OSL_FMT_STREAM);
		m_music2=NULL;
		m_nbCols=props->getIntValue("normal.grid.width");
		m_scenario1=new EventsScenario("res/levels/8bits/brutus.scenario");
		m_scenario2=NULL;
		
		m_upspeed=props->getIntValue("normal.upspeed");
		m_dropspeed=props->getIntValue("normal.dropspeed");
		
		m_duration=LEVEL_8BITS_DURATION_BRUTUS;
		m_part=LEVEL_8BITS_PART_BRUTUS;
		
		m_addline=props->getIntValue("normal.lines.start");
		
		m_img_overlay=fwkLoadImageFilePNG("res/levels/8bits/grid13.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		m_linepattern=new LinePattern("res/levels/8bits/8bits_hard.pattern");
		m_songbanner2=new SongBanner(GAMEDEF_SONGBANNER_Y+3, "res/fonts/bmpFonts/russelsquare/13/", "normal", "italic", props->getStrValue("hard.songbanner.text1"), props->getStrValue("hard.songbanner.text2", ""), props->getStrValue("hard.songbanner.text3", ""), props->getStrValue("hard.songbanner.text4", ""));
		
		initRandomPart(props);
		
		m_music2=fwkLoadSoundFileMP3("res/levels/8bits/hedgehog.mp3", OSL_FMT_STREAM);
		m_nbCols=props->getIntValue("hard.grid.width");
		m_scenario2=new EventsScenario("res/levels/8bits/hedgehog.scenario");
		
		m_upspeed=props->getIntValue("hard.upspeed");
		m_dropspeed=props->getIntValue("hard.dropspeed");
		m_upspeed2=props->getIntValue("hard.upspeed2");
		m_dropspeed2=props->getIntValue("hard.dropspeed2");
		
		m_addline=props->getIntValue("hard.lines.start");
		m_img_overlay=fwkLoadImageFilePNG("res/levels/8bits/grid14.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_BONUS)
	{
		m_linepattern=new LinePattern("res/levels/8bits/8bits_blixt.pattern");
		m_songbanner2=new SongBanner(GAMEDEF_SONGBANNER_Y+3, "res/fonts/bmpFonts/russelsquare/13/", "normal", "italic", props->getStrValue("hard.songbanner.text1"), props->getStrValue("hard.songbanner.text2", ""), props->getStrValue("hard.songbanner.text3", ""), props->getStrValue("hard.songbanner.text4", ""));
		
		initRandomPart(props);
		
		m_music2=fwkLoadSoundFileMP3("res/levels/8bits/hedgehog.mp3", OSL_FMT_STREAM);
		m_nbCols=props->getIntValue("blixt.grid.width");
		m_scenario2=new EventsScenario("res/levels/8bits/hedgehog.scenario");
		
		m_upspeed=props->getIntValue("blixt.upspeed");
		m_dropspeed=props->getIntValue("blixt.dropspeed");
		m_upspeed2=props->getIntValue("blixt.upspeed2");
		m_dropspeed2=props->getIntValue("blixt.dropspeed2");
		
		m_addline=props->getIntValue("blixt.lines.start");
		m_img_overlay=fwkLoadImageFilePNG("res/levels/8bits/grid14.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	}

	m_comboDrawer=new ComboNumberDrawer(	LEVEL_8BITS_COMBO_COLOR_NORMAL_IN, LEVEL_8BITS_COMBO_COLOR_NORMAL_OUT, 
											LEVEL_8BITS_COMBO_COLOR_HOT1, LEVEL_8BITS_COMBO_COLOR_HOT2, LEVEL_8BITS_COMBO_COLOR_HOT3,
											"res/fonts/bmpFonts/qubix/24/", "qubix", "res/fonts/bmpFonts/null/", "null"
										);
	
	initAnims();
	initParticleGenerators(props);
	initGeneratorsBrutus(props);
	
	m_fx_eurhawt_alpha=new ProgressiveAlphaEffect(	props->getIntValue("fx.eurhawt.alpha.time"), props->getIntValue("fx.eurhawt.alpha.time"), 
													props->getIntValue("fx.eurhawt.alpha.from"), props->getIntValue("fx.eurhawt.alpha.to"), 1);
	
	m_fx_hedgehog_transl=new TranslationEffect(0, -1*props->getIntValue("fx.hedgehog.height"), 0, 0, 
											   props->getIntValue("fx.hedgehog.time"), props->getIntValue("fx.hedgehog.time"),
											   true, 1);
	
	m_fx_particlesRot=new RotationEffect(props->getIntValue("fx.particles.rot.time"), true, props->getIntValue("fx.particles.rot.loops"));
	
	m_fx_boum_shake=new ShakeEffect(props->getIntValue("fx.boum.shake.width"), 0, 
									props->getIntValue("fx.boum.shake.time"), true,
									props->getIntValue("fx.boum.shake.loops"));
	
	oslSetFrameskip(props->getIntValue("frameskip"));
	
	delete props;
	
	
	m_img_songbannerbg=fwkLoadImageFilePNG("res/levels/8bits/sb_bg.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_songbannerbg, 0, 272);
	
	m_fx_songbanner=new SerialLocalEffect(1);
	int sbfxappear=15;
	int sbfxwait=m_songbanner1->getTotalTimePresent();
	int sbfxdisappear=20;
	int sbfxsize=m_img_songbannerbg->sizeY;
	m_fx_songbanner->addEffect(new TranslationEffect(0, 0, 0, -sbfxsize, sbfxappear, sbfxappear, true, 1));
	m_fx_songbanner_wait_ptr=new WaitLocalEffect(sbfxwait);
	m_fx_songbanner_wait_ptr->setPosition(0, -sbfxsize);
	m_fx_songbanner->addEffect(m_fx_songbanner_wait_ptr);
	m_fx_songbanner->addEffect(new TranslationEffect(0, -sbfxsize, 0, 0, sbfxdisappear, sbfxdisappear, true, 1));
	
	
	m_img_pause=fwkLoadImageFilePNG("res/levels/8bits/pause.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageCenterScreen(m_img_pause);	
	
	m_test=false;
}

void Level8Bits::initRandomPart(Properties *props)
{
	int rand=utils::random(0,1);
	if(rand==0) //on a choisi au hasard eurhawt
	{
		m_songbanner1=new SongBanner(GAMEDEF_SONGBANNER_Y+3, "res/fonts/bmpFonts/russelsquare/13/", "normal", "italic", props->getStrValue("easy.songbanner.text1"), props->getStrValue("easy.songbanner.text2", ""), props->getStrValue("easy.songbanner.text3", ""), props->getStrValue("easy.songbanner.text4", ""));
		m_music1=fwkLoadSoundFileMP3("res/levels/8bits/eurhawt.mp3", OSL_FMT_STREAM);
		m_scenario1=new EventsScenario("res/levels/8bits/eurhawt.scenario");
		m_duration=LEVEL_8BITS_DURATION_EURHAWT+LEVEL_8BITS_DURATION_HEDGEHOG;
		m_part=LEVEL_8BITS_PART_EURHAWT;
	}
	else //on a choisi au hasard brutus
	{
		m_songbanner1=new SongBanner(GAMEDEF_SONGBANNER_Y+3, "res/fonts/bmpFonts/russelsquare/13/", "normal", "italic", props->getStrValue("normal.songbanner.text1"), props->getStrValue("normal.songbanner.text2", ""), props->getStrValue("normal.songbanner.text3", ""), props->getStrValue("normal.songbanner.text4", ""));
		m_music1=fwkLoadSoundFileMP3("res/levels/8bits/brutus.mp3", OSL_FMT_STREAM);
		m_scenario1=new EventsScenario("res/levels/8bits/brutus.scenario");
		m_duration=LEVEL_8BITS_DURATION_BRUTUS+LEVEL_8BITS_DURATION_HEDGEHOG;
		m_part=LEVEL_8BITS_PART_BRUTUS;
	}
}

void Level8Bits::reset()
{
	Properties *props=new Properties("res/levels/8bits/8bits.properties");
		
	m_songbanner1->reset();
	if(m_songbanner2!=NULL)
		m_songbanner2->reset();
		
	oslStopSound(m_music1);
	if(m_music2!=NULL)
		oslStopSound(m_music2);
		
	m_scenario1->reset();
	if(m_scenario2!=NULL)
	{
		m_scenario2->pause();
		m_scenario2->reset();
	}
	
	m_linepattern->reset();
	m_comboDrawer->reset();
		
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		m_part=LEVEL_8BITS_PART_EURHAWT;
		m_addline=props->getIntValue("easy.lines.start");
		m_upspeed=props->getIntValue("easy.upspeed");
		m_dropspeed=props->getIntValue("easy.dropspeed");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_part=LEVEL_8BITS_PART_BRUTUS;
		m_addline=props->getIntValue("normal.lines.start");
		m_upspeed=props->getIntValue("normal.upspeed");
		m_dropspeed=props->getIntValue("normal.dropspeed");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		fwkDeleteSound(m_music1);
		delete m_songbanner1;
		delete m_scenario1;		
		initRandomPart(props);
		m_addline=props->getIntValue("hard.lines.start");
		m_upspeed=props->getIntValue("hard.upspeed");
		m_dropspeed=props->getIntValue("hard.dropspeed");
	}
	else //bonus
	{
		fwkDeleteSound(m_music1);
		delete m_songbanner1;
		delete m_scenario1;		
		initRandomPart(props);
		m_addline=props->getIntValue("blixt.lines.start");
		m_upspeed=props->getIntValue("blixt.upspeed");
		m_dropspeed=props->getIntValue("blixt.dropspeed");
	}
	
	setGeneratorsSpeedSlow();	
	
	delete props;
}

void Level8Bits::initGeneratorsBrutus(Properties *props)
{
	m_gen_brutus1=new ParticleGeneratorSegment(	props->getIntValue("pgenbrutus1.pos.x"), props->getIntValue("pgenbrutus1.pos.y"),
											  	90, props->getIntValue("pgenbrutus.scale"),
											  	props->getIntValue("pgenbrutus.throughput.particles"), 
											  	props->getIntValue("pgenbrutus.throughput.time"),
											  	props->getIntValue("pgenbrutus.movetime"),
											  	props->getIntValue("pgenbrutus.dist"),
											  	props->getIntValue("pgenbrutus.lifespan"),
											  	"res/levels/8bits/particles/star.png", OSL_PF_8888
											  );
											  
	m_gen_brutus1->setLocalEffect1SimpleTranslation();
	m_gen_brutus1->setLocalEffect2(new RotationEffect(20));
	
	m_gen_brutus2=new ParticleGeneratorSegment(	props->getIntValue("pgenbrutus2.pos.x"), props->getIntValue("pgenbrutus2.pos.y"),
											  	90, props->getIntValue("pgenbrutus.scale"),
											  	props->getIntValue("pgenbrutus.throughput.particles"), 
											  	props->getIntValue("pgenbrutus.throughput.time"),
											  	props->getIntValue("pgenbrutus.movetime"),
											  	props->getIntValue("pgenbrutus.dist"),
											  	props->getIntValue("pgenbrutus.lifespan"),
											  	"res/levels/8bits/particles/star.png", OSL_PF_8888
											  );
											  
	m_gen_brutus2->setLocalEffect1SimpleTranslation();
	m_gen_brutus2->setLocalEffect2(new RotationEffect(props->getIntValue("pgenbrutus.rotation.time")));
	
	m_fx_gen_brutus_shake=new ShakeEffect(props->getIntValue("pgenbrutus.shake.width"), 0, props->getIntValue("pgenbrutus.shake.time"));
}

void Level8Bits::deleteGeneratorsBrutus()
{
	delete m_gen_brutus1;
	delete m_gen_brutus2;
	delete m_fx_gen_brutus_shake;
}

void Level8Bits::initParticleGenerators(Properties *props)
{
	m_gen_props_speed1=new gen8bitsstruct;
	m_gen_props_speed2=new gen8bitsstruct;
	m_gen_props_speed3=new gen8bitsstruct;
	
	m_gen_props_speed1->particles=props->getIntValue("pgen.speed1.throughput.particles");
	m_gen_props_speed1->gentime=props->getIntValue("pgen.speed1.throughput.time");
	m_gen_props_speed1->movetime=props->getIntValue("pgen.speed1.movetime");
	m_gen_props_speed1->dist=props->getIntValue("pgen.speed1.dist");
	m_gen_props_speed1->lifespan=props->getIntValue("pgen.speed1.lifespan");
	m_gen_props_speed1->gravitymin=props->getFloatValue("pgen.speed1.gravity.min");
	m_gen_props_speed1->gravitymax=props->getFloatValue("pgen.speed1.gravity.max");
	
	m_gen_props_speed2->particles=props->getIntValue("pgen.speed2.throughput.particles");
	m_gen_props_speed2->gentime=props->getIntValue("pgen.speed2.throughput.time");
	m_gen_props_speed2->movetime=props->getIntValue("pgen.speed2.movetime");
	m_gen_props_speed2->dist=props->getIntValue("pgen.speed2.dist");
	m_gen_props_speed2->lifespan=props->getIntValue("pgen.speed2.lifespan");
	m_gen_props_speed2->gravitymin=props->getFloatValue("pgen.speed2.gravity.min");
	m_gen_props_speed2->gravitymax=props->getFloatValue("pgen.speed2.gravity.max");
	
	m_gen_props_speed3->particles=props->getIntValue("pgen.speed3.throughput.particles");
	m_gen_props_speed3->gentime=props->getIntValue("pgen.speed3.throughput.time");
	m_gen_props_speed3->movetime=props->getIntValue("pgen.speed3.movetime");
	m_gen_props_speed3->dist=props->getIntValue("pgen.speed3.dist");
	m_gen_props_speed3->lifespan=props->getIntValue("pgen.speed3.lifespan");
	m_gen_props_speed3->gravitymin=props->getFloatValue("pgen.speed3.gravity.min");
	m_gen_props_speed3->gravitymax=props->getFloatValue("pgen.speed3.gravity.max");
	
	
	m_gen_animfact_left=new ControlledAnimationFactory();
	m_gen_animfact_right=new ControlledAnimationFactory();
	
	m_gen_animBlue=new Anim("res/levels/8bits/particles/blue.png", OSL_PF_8888);
	m_gen_animRed=new Anim("res/levels/8bits/particles/red.png", OSL_PF_8888);
	m_gen_animYellow=new Anim("res/levels/8bits/particles/yellow.png", OSL_PF_8888);
	m_gen_animPurple=new Anim("res/levels/8bits/particles/purple.png", OSL_PF_8888);
	m_gen_animGreen=new Anim("res/levels/8bits/particles/green.png", OSL_PF_8888);
	m_gen_animGrey=new Anim("res/levels/8bits/particles/grey.png", OSL_PF_8888);
	m_gen_animBlue->setReferencePointCenter();
	m_gen_animRed->setReferencePointCenter();
	m_gen_animYellow->setReferencePointCenter();
	m_gen_animPurple->setReferencePointCenter();
	m_gen_animGreen->setReferencePointCenter();
	m_gen_animGrey->setReferencePointCenter();
	
	m_gen_animfact_left->setAnim(m_gen_animBlue);
	m_gen_animfact_right->setAnim(m_gen_animRed);
	
	m_gen_left=new ParticleGeneratorSegment(	0, 136, 0, 272, 
												m_gen_props_speed1->particles, m_gen_props_speed1->gentime, 
												m_gen_props_speed1->movetime, 
												m_gen_props_speed1->dist, 
												m_gen_props_speed1->lifespan, 
												m_gen_animfact_left);
												
	m_gen_right=new ParticleGeneratorSegment(	480, 136, 180, 272, 
												m_gen_props_speed1->particles, m_gen_props_speed1->gentime, 
												m_gen_props_speed1->movetime, 
												m_gen_props_speed1->dist, 
												m_gen_props_speed1->lifespan, 
												m_gen_animfact_right);
	
	m_gen_left->setLocalEffect1SimpleTranslation(45);
	m_gen_right->setLocalEffect1SimpleTranslation(-45);
	m_gen_right_pfxfrg=new PFXFRandomGravity(/*m_gen_props_speed1->gravitymin, m_gen_props_speed1->gravitymax*/);
	m_gen_right_pfxfrg->setAccelYMin(m_gen_props_speed1->gravitymin);
	m_gen_right_pfxfrg->setAccelYMin(m_gen_props_speed1->gravitymax);
	m_gen_left_pfxfrg=new PFXFRandomGravity(/*m_gen_props_speed1->gravitymin, m_gen_props_speed1->gravitymax*/);
	m_gen_left_pfxfrg->setAccelYMin(m_gen_props_speed1->gravitymin);
	m_gen_left_pfxfrg->setAccelYMin(m_gen_props_speed1->gravitymax);
	m_gen_left->setLocalEffect2(m_gen_left_pfxfrg);
	m_gen_right->setLocalEffect2(m_gen_right_pfxfrg);
		
}

void Level8Bits::deleteParticleGenerators()
{
	//delete m_gen_animfact_left;
	//delete m_gen_animfact_right;
	
	delete m_gen_animBlue;
	delete m_gen_animRed;
	delete m_gen_animYellow;
	delete m_gen_animPurple;
	delete m_gen_animGreen;
	delete m_gen_animGrey;
	
	delete m_gen_left;
	delete m_gen_right;
	
	delete m_gen_props_speed1;
	delete m_gen_props_speed2;
	delete m_gen_props_speed3;
}

void Level8Bits::initAnims()
{
	m_animBlue=new Anim(6, "bleu", "res/levels/8bits/pieces/blue/normal/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animRed=new Anim(6, "rouge", "res/levels/8bits/pieces/red/normal/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animYellow=new Anim(6, "jaune", "res/levels/8bits/pieces/yellow/normal/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animPurple=new Anim(6, "mauve", "res/levels/8bits/pieces/purple/normal/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animGreen=new Anim(6, "vert", "res/levels/8bits/pieces/green/normal/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animGrey=new Anim(6, "gris", "res/levels/8bits/pieces/grey/normal/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animBlue->setReferencePointCenter();
	m_animRed->setReferencePointCenter();
	m_animYellow->setReferencePointCenter();
	m_animPurple->setReferencePointCenter();
	m_animGreen->setReferencePointCenter();
	m_animGrey->setReferencePointCenter();
	
	m_animBlueShield=new Anim(6, "bleu_blindage", "res/levels/8bits/pieces/blue/shield/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animRedShield=new Anim(6, "rouge_blindage", "res/levels/8bits/pieces/red/shield/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animYellowShield=new Anim(6, "jaune_blindage", "res/levels/8bits/pieces/yellow/shield/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animPurpleShield=new Anim(6, "mauve_blindage", "res/levels/8bits/pieces/purple/shield/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animGreenShield=new Anim(6, "vert_blindage", "res/levels/8bits/pieces/green/shield/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animGreyShield=new Anim(6, "gris_blindage", "res/levels/8bits/pieces/grey/shield/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animBlueShield->setReferencePointCenter();
	m_animRedShield->setReferencePointCenter();
	m_animYellowShield->setReferencePointCenter();
	m_animPurpleShield->setReferencePointCenter();
	m_animGreenShield->setReferencePointCenter();
	m_animGreyShield->setReferencePointCenter();
}

Level8Bits::~Level8Bits()
{	
	//enableDebug();
	//debug("delete pattern...");
	delete m_linepattern;
	
	
	//debug("delete anims...");
	delete m_animBlue;
	delete m_animRed;
	delete m_animYellow;
	delete m_animPurple;
	delete m_animGreen;
	delete m_animGrey;
	delete m_animBlueShield;
	delete m_animRedShield;
	delete m_animYellowShield;
	delete m_animPurpleShield;
	delete m_animGreenShield;
	delete m_animGreyShield;
	
	//debug("delete songbanners...");
	delete m_songbanner1;
	if(m_songbanner2!=NULL)
		delete m_songbanner2;
		
		
	//debug("delete musics...");
	oslDeleteSound(m_music1);
	if(m_music2!=NULL)
		oslDeleteSound(m_music2);
		
	//debug("delete scenarii...");
	delete m_scenario1;
	if(m_scenario2!=NULL)
		delete m_scenario2;
		
	//debug("delete combodrawer...");
	delete m_comboDrawer;
	
	//debug("delete particle generators...");
	deleteParticleGenerators();
	deleteGeneratorsBrutus();
	
	//debug("deletion ok");
	//enableDebug(false);
	
	oslDeleteImage(m_img_overlay);
	
	delete m_fx_eurhawt_alpha;
	delete m_fx_hedgehog_transl;
	delete m_fx_particlesRot;
	delete m_fx_boum_shake;
	
	oslSetFrameskip(1);
	
	delete m_fx_songbanner;
	oslDeleteImage(m_img_songbannerbg);
	
	oslDeleteImage(m_img_pause);
}


//dessin avec une paire
void Level8Bits::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc, scorer);
	if(!isPaused())
	{
		//currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
		Pos gridPos=currentPair->getGridPos(0);
		Pos pixPos=grid->getContentPosition(gridPos.x, gridPos.y);
		drawPairXY(currentPair, pixPos.x, pixPos.y, grid, fx_trans);
	}
	drawAfter(grid, nextPair, cc, scorer);
}

//dessin avec 2 pieces
void Level8Bits::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc, scorer);
	
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	drawPieceXY(pieceConnected, pixPosConnected.x, pixPosConnected.y);
	drawPieceXY(pieceUnconnected, pixPosUnconnected.x, pixPosUnconnected.y, fx_trans);
	
	drawAfter(grid, nextPair, cc, scorer);
}

//dessin avec rien
void Level8Bits::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc, scorer);
	drawAfter(grid, nextPair, cc, scorer);
}

void Level8Bits::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	m_gen_left->drawXY(m_gen_left->getX(), m_gen_left->getY(), m_fx_hedgehog_transl->isRunning()?m_fx_hedgehog_transl:NULL, m_fx_particlesRot->isRunning()?m_fx_particlesRot:NULL);
	m_gen_right->drawXY(m_gen_right->getX(), m_gen_right->getY(), m_fx_hedgehog_transl->isRunning()?m_fx_hedgehog_transl:NULL, m_fx_particlesRot->isRunning()?m_fx_particlesRot:NULL);
		
	if(isPaused())
	{
		
	}
	else
	{
		fx::draw(m_img_overlay, 0, 0, m_fx_boum_shake->isRunning()?m_fx_boum_shake:NULL);
		
		if(m_fx_eurhawt_alpha->isRunning())
			m_fx_eurhawt_alpha->activate();
		
		//dessin du contenu de la grille
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco->getPiece()!=NULL)
				{
					Pos pos=grid->getContentPosition(i, j);
					AbstractPiece *p=mco->getPiece();
					
					if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_SHIELDED)
					{
						if(mco->getMCOContent()!=NULL)
						{
							mco->getMCOContent()->drawXY(pos.x,pos.y, m_fx_boum_shake->isRunning()?m_fx_boum_shake:NULL);
						}
					}
					else
					{
						p->drawXY(pos.x,pos.y);
					}
				}
			}	
		}
		
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	
}

void Level8Bits::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	debug("draw after");
	
	if(isPaused())
	{
		oslSetAlpha(OSL_FX_TINT, m_color_pause);
		oslDrawImage(m_img_pause);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	else
	{
		//nextPair->drawXY(60, 30, grid, APPLY_FX_ON_NONE);
		//drawPairXY(nextPair, 414, 229, grid);	
		
		//dessin des jauges
		//15 carrés de haut
		int nbSquares;
		if(cc->getTime()==0)
			nbSquares=0;
		else
			nbSquares = int(14*cc->getTime()/cc->getTimeTotalAvailable())+1;
		//	nbSquares=( int((cc->getTime()*272)/cc->getTimeTotalAvailable()) % 15)+1;
		
		int y=255;
		for(int i=0;i<nbSquares;i++)
		{
			fx::drawXY(particleFromColor(nextPair->getPiece(0)->getColor())->getImage(), 16, y, m_fx_boum_shake->isRunning()?m_fx_boum_shake:NULL);
			fx::drawXY(particleFromColor(nextPair->getPiece(1)->getColor())->getImage(), 464, y, m_fx_boum_shake->isRunning()?m_fx_boum_shake:NULL);
			y-=17;
		}
		
		//oslDrawFillRect(80, 272-int((cc->getTime()*272)/cc->getTimeTotalAvailable()), 95, 272, LEVEL_8BITS_COMBO_COLOR_NORMAL_OUT);
		
	
		m_gen_brutus1->draw();
		m_gen_brutus2->draw();
		
		m_comboDrawer->drawXY(158, 180); 
	}
	
	if(m_fx_songbanner->isRunning())
	{
		fx::draw(m_img_songbannerbg, m_fx_songbanner);
		
		m_songbanner1->draw();
		if(m_songbanner2!=NULL)
			m_songbanner2->draw();
	}
	
		
	
	
	//fx::drawXY(m_gen_animBlue->getImage(), 10, 136, m_fx_hedgehog_transl->isRunning()?m_fx_hedgehog_transl:NULL);
	
}

void Level8Bits::start_specific()
{
	oslPlaySound(m_music1, 0);
	m_scenario1->restart();
	m_fx_songbanner->restart();
	m_gen_brutus1->generationOff();
	m_gen_brutus1->start();
	m_gen_brutus2->generationOff();
	m_gen_brutus2->start();
	m_fx_gen_brutus_shake->start();
	m_gen_left->generationOn();
	m_gen_right->generationOn();
	m_gen_left->start();
	m_gen_right->start();
}

//appelé uniquement si le level n'est pas en pause.
void Level8Bits::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	debug("update");
	//cas special 8bits: on utilise des anims séparées
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			AbstractPiece *p=mco->getPiece();
			if(p!=NULL)
			{
				//on cree si une piece n'en a pas
				if(mco->getMCOContent()==NULL)
				{
					Anim *anim=animFromColor(p->getColor(), p->getType()==PIECE_TYPE_NORMAL?false:true);
					mco->setMCOContent(new MCOContentSimple(new Anim(*anim)));
				}				
				//on démarre si la piece explose
				if(mco->isColorExploding() || mco->isShapeExploding() || mco->isSpecialExploding())
					mco->getMCOContent()->start();
			}
			else//mco->getPiece()==NULL
			{
				if(mco->getMCOContent()!=NULL)
				{
					mco->deleteMCOContent();
				}
			}
			
			if(mco->getMCOContent()!=NULL) //peu importe le cas, on update
			{
				mco->getMCOContent()->update();
			}
		}
	}
	
	
	if(m_part==LEVEL_8BITS_PART_HEDGEHOG)
	{
		m_songbanner2->update();
		m_scenario2->update();
	}
	else
	{
		m_songbanner1->update();
		m_scenario1->update();
	}
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	
	
	State * state=new State();
	
	if(m_fx_hedgehog_transl->isRunning())
	{
		m_gen_left->savestate(state);
		m_fx_hedgehog_transl->applyOnPositionable(m_gen_left);
		m_gen_left->update();
		m_gen_left->restorestate(state);
		m_gen_right->savestate(state);
		m_fx_hedgehog_transl->applyOnPositionable(m_gen_right);
		m_gen_right->update();
		m_gen_right->restorestate(state);
	}
	else
	{
		m_gen_left->update();
		m_gen_right->update();
	}
	
	m_gen_brutus1->savestate(state);
	m_fx_gen_brutus_shake->applyOnPositionable(m_gen_brutus1);
	m_gen_brutus1->update();
	m_gen_brutus1->restorestate(state);
	m_gen_brutus2->savestate(state);
	m_fx_gen_brutus_shake->applyOnPositionable(m_gen_brutus2);
	m_gen_brutus2->update();
	m_gen_brutus2->restorestate(state);
	delete state;
	
	m_fx_gen_brutus_shake->update();
	m_fx_eurhawt_alpha->update();
	m_fx_hedgehog_transl->update();
	m_fx_particlesRot->update();
	m_fx_boum_shake->update();
	m_fx_songbanner->update();
	
	//faut changer de zik
	if( (m_difficulty==GAMEDEF_DIFFICULTY_HARD || m_difficulty==GAMEDEF_DIFFICULTY_BONUS) && 
			((m_part==LEVEL_8BITS_PART_EURHAWT && m_scenario1->getTime()>=LEVEL_8BITS_DURATION_EURHAWT)
				||
			(m_part==LEVEL_8BITS_PART_BRUTUS && m_scenario1->getTime()>=LEVEL_8BITS_DURATION_BRUTUS)))
	{
		m_part=LEVEL_8BITS_PART_HEDGEHOG;
		oslPlaySound(m_music2, 5);
		m_scenario2->restart();
		m_upspeed=m_upspeed2;
		m_dropspeed=m_dropspeed2;
		m_fx_songbanner_wait_ptr->setTime(m_songbanner2->getTotalTimePresent());
		m_fx_songbanner->restart();
	}
	
	//evenements du scenario
	if(hasEvent(1)) //vitesse niveau 1 (lent)
	{	
		setGeneratorsSpeedSlow();
	}
	else if(hasEvent(2)) //vitesse niveau 2 (normal)
	{	
		setGeneratorsSpeedNormal();
	}
	else if(hasEvent(3)) //vitesse niveau 3 (rapide)
	{	
		setGeneratorsSpeedFast();
	}
	
	//BRUTUS
	if(hasEvent(4)) //debut son final avant boum
	{
		m_gen_left->generationOff();
		m_gen_right->generationOff();
		m_upspeed=-1;
	}
	if(hasEvent(5)) //boum final
	{
		m_addline++;
		m_fx_boum_shake->restart();
	}
	if(hasEvent(6)) //debut trrlrrlrlrlrrrllrllrlrlrlrlrlrllr
	{
		m_gen_brutus1->generationOn();
		m_gen_brutus2->generationOn();
	}
	if(hasEvent(7)) //fin trrlrrlrlrlrrrllrllrlrlrlrlrlrllr
	{
		m_gen_brutus1->generationOff();
		m_gen_brutus2->generationOff();
	}
	
	//EURHAWT
	if(hasEvent(8)) //debut son final avant boum
	{
		m_gen_left->generationOff();
		m_gen_right->generationOff();
		m_upspeed=-1;
	}
	if(hasEvent(9)) //boum final
	{
		m_addline++;
		m_fx_boum_shake->restart();
	}
	if(hasEvent(10)) //tsoing
	{
		m_fx_eurhawt_alpha->restart();
		m_fx_particlesRot->restart();
	}
	
	//HEDGEHOG
	if(hasEvent(11)) //biouuu
	{
		m_fx_eurhawt_alpha->restart();
		m_fx_particlesRot->restart();
	}
	if(hasEvent(12)) //tsui (oui)
	{
		m_fx_hedgehog_transl->restart();
	}
	
}

void Level8Bits::setGeneratorsSpeedSlow()
{
	m_gen_left->generationOn();
	m_gen_right->generationOn();
	
	m_gen_left->setThroughput(m_gen_props_speed1->particles, m_gen_props_speed1->gentime);
	m_gen_left->setPTranslTime(m_gen_props_speed1->movetime);
	m_gen_left->setPDistance(m_gen_props_speed1->dist);
	m_gen_left->setPLifespan(m_gen_props_speed1->lifespan);
	
	m_gen_right->setThroughput(m_gen_props_speed1->particles, m_gen_props_speed1->gentime);
	m_gen_right->setPTranslTime(m_gen_props_speed1->movetime);
	m_gen_right->setPDistance(m_gen_props_speed1->dist);
	m_gen_right->setPLifespan(m_gen_props_speed1->lifespan);
	
	m_gen_right_pfxfrg->setAccelYMin(m_gen_props_speed1->gravitymin);
	m_gen_right_pfxfrg->setAccelYMax(m_gen_props_speed1->gravitymax);
	m_gen_left_pfxfrg->setAccelYMin(m_gen_props_speed1->gravitymin);
	m_gen_left_pfxfrg->setAccelYMax(m_gen_props_speed1->gravitymax);
}

void Level8Bits::setGeneratorsSpeedNormal()
{
	m_gen_left->generationOn();
	m_gen_right->generationOn();
	
	m_gen_left->setThroughput(m_gen_props_speed2->particles, m_gen_props_speed2->gentime);
	m_gen_left->setPTranslTime(m_gen_props_speed2->movetime);
	m_gen_left->setPDistance(m_gen_props_speed2->dist);
	m_gen_left->setPLifespan(m_gen_props_speed2->lifespan);
	
	m_gen_right->setThroughput(m_gen_props_speed2->particles, m_gen_props_speed2->gentime);
	m_gen_right->setPTranslTime(m_gen_props_speed2->movetime);
	m_gen_right->setPDistance(m_gen_props_speed2->dist);
	m_gen_right->setPLifespan(m_gen_props_speed2->lifespan);
	
	m_gen_right_pfxfrg->setAccelYMin(m_gen_props_speed2->gravitymin);
	m_gen_right_pfxfrg->setAccelYMax(m_gen_props_speed2->gravitymax);
	m_gen_left_pfxfrg->setAccelYMin(m_gen_props_speed2->gravitymin);
	m_gen_left_pfxfrg->setAccelYMax(m_gen_props_speed2->gravitymax);
}

void Level8Bits::setGeneratorsSpeedFast()
{
	m_gen_left->generationOn();
	m_gen_right->generationOn();
	
	m_gen_left->setThroughput(m_gen_props_speed3->particles, m_gen_props_speed3->gentime);
	m_gen_left->setPTranslTime(m_gen_props_speed3->movetime);
	m_gen_left->setPDistance(m_gen_props_speed3->dist);
	m_gen_left->setPLifespan(m_gen_props_speed3->lifespan);
	
	m_gen_right->setThroughput(m_gen_props_speed3->particles, m_gen_props_speed3->gentime);
	m_gen_right->setPTranslTime(m_gen_props_speed3->movetime);
	m_gen_right->setPDistance(m_gen_props_speed3->dist);
	m_gen_right->setPLifespan(m_gen_props_speed3->lifespan);
	
	m_gen_right_pfxfrg->setAccelYMin(m_gen_props_speed3->gravitymin);
	m_gen_right_pfxfrg->setAccelYMax(m_gen_props_speed3->gravitymax);
	m_gen_left_pfxfrg->setAccelYMin(m_gen_props_speed3->gravitymin);
	m_gen_left_pfxfrg->setAccelYMax(m_gen_props_speed3->gravitymax);
}


bool Level8Bits::hasEvent(int event)
{
	if(m_scenario1->hasEvent(event))
		return true;
	
	if(m_scenario2==NULL)
		return false;
	
	if(m_scenario2->hasEvent(event))
		return true;
	
	return false;
}

void Level8Bits::pause_specific() 
{
	if(m_part==LEVEL_8BITS_PART_HEDGEHOG)
	{
		oslPauseSound(m_music2, 1);
		m_scenario2->pause();
	}
	else
	{
		oslPauseSound(m_music1, 1);
		m_scenario1->pause();
	}
	
	//couleur de l'image de pause
	
}

void Level8Bits::resume_specific()
{
	if(m_part==LEVEL_8BITS_PART_HEDGEHOG)
	{
		oslPauseSound(m_music2, 0);
		m_scenario2->start();
	}
	else
	{
		oslPauseSound(m_music1, 0);
		m_scenario1->start();
	}
}
		
list<AbstractPiece *> *Level8Bits::getNextLine()
{
	list<AbstractPiece *> *nextLine=m_linepattern->getNextLine();
	list<AbstractPiece *>::iterator it;
	for(it=nextLine->begin();it!=nextLine->end();it++)
	{
		AbstractPiece *piece=*it;
		if(piece!=NULL && piece->isRandom())
		{
			switch(piece->getColor())
			{
				case PIECE_COLOR_BLUE:
				{
					piece->setShape(PIECE_SHAPE_CIRCLE);
					break;	
				}
				case PIECE_COLOR_RED:
				{
					piece->setShape(PIECE_SHAPE_SQUARE);
					break;	
				}
				case PIECE_COLOR_YELLOW:
				{
					piece->setShape(PIECE_SHAPE_DIAMOND);
					break;	
				}
				case PIECE_COLOR_PURPLE:
				{
					piece->setShape(PIECE_SHAPE_STAR);
					break;	
				}
				case PIECE_COLOR_GREEN:
				{
					piece->setShape(PIECE_SHAPE_CROSS);
					break;	
				}
				case PIECE_COLOR_ORANGE:
				{
					piece->setShape(PIECE_SHAPE_TRIANGLE);
					break;	
				}
			}
		}
	}
	
	return nextLine;
}

bool Level8Bits::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int Level8Bits::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return m_dropspeed;
}

int Level8Bits::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return m_upspeed;
}

void Level8Bits::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_TRIANGLE)
	{
		//m_fx_eurhawt_alpha->restart();
	}
	else if(key==OSL_KEY_SQUARE)
	{
		//m_fx_hedgehog_transl->restart();
		m_test=true;
	}
}

bool Level8Bits::finished()
{
	//if(m_test)
	//	return true;
	
	if	(
			(m_difficulty==GAMEDEF_DIFFICULTY_EASY && m_scenario1->getTime()>=LEVEL_8BITS_DURATION_EURHAWT)
			||
			(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL && m_scenario1->getTime()>=LEVEL_8BITS_DURATION_BRUTUS)
			||
			( (m_difficulty==GAMEDEF_DIFFICULTY_HARD || m_difficulty==GAMEDEF_DIFFICULTY_BONUS) && m_scenario2->getTime()>=LEVEL_8BITS_DURATION_HEDGEHOG)
		)
	{
		return true;
	}
	return false;
}

float Level8Bits::getDuration()
{
	return m_duration;
}

Pair *Level8Bits::getNewPair()
{	
	Pair *p=new Pair(randomPiece(), randomPiece());
	int posX=int(m_nbCols/2)-1;
	int posY=0;
	
	p->goTo(posX, posY);
	
	m_gen_animfact_left->setAnim(particleFromColor(p->getPiece(0)->getColor()));
	m_gen_animfact_right->setAnim(particleFromColor(p->getPiece(1)->getColor()));
	
	//changement de couleur de l'image de pause
	m_color_pause=RGB(255,0,255);
	if(p->getPiece(0)->getColor()==PIECE_COLOR_PURPLE || p->getPiece(1)->getColor()==PIECE_COLOR_PURPLE)
	{
		m_color_pause=RGB(255, 250, 36);
		if(p->getPiece(0)->getColor()==PIECE_COLOR_YELLOW || p->getPiece(1)->getColor()==PIECE_COLOR_YELLOW)
			m_color_pause=RGB(0, 210, 0);
	}
		
	return p;
}


AbstractPiece *Level8Bits::randomPiece()
{
	int color=random(0,5);
	PieceSimple *p=NULL;
	switch(color)
	{
		case 0:
		{
			p=new PieceSimple(PIECE_SHAPE_CIRCLE, PIECE_COLOR_BLUE);
			break;
		}
		case 1:
		{
			p=new PieceSimple(PIECE_SHAPE_SQUARE, PIECE_COLOR_RED);
			break;
		}
		case 2:
		{
			p=new PieceSimple(PIECE_SHAPE_DIAMOND, PIECE_COLOR_YELLOW);	
			break;
		}
		case 3:
		{
			p=new PieceSimple(PIECE_SHAPE_STAR, PIECE_COLOR_PURPLE);	
			break;
		}
		case 4:
		{
			p=new PieceSimple(PIECE_SHAPE_CROSS, PIECE_COLOR_GREEN);
			break;
		}
		case 5:
		{
			p=new PieceSimple(PIECE_SHAPE_TRIANGLE, PIECE_COLOR_ORANGE);
			break;
		}
		default:
		{
			oslFatalError(string("random generated "+toString(color)).c_str());
		}
	}
	
	return p;
}

void Level8Bits::drawPairXY(Pair *pair, int x, int y, GameGrid *grid, AbstractLocalEffect *fx_transl, AbstractLocalEffect *fx1)
{
	//suivant l'orientation, la 2eme piece est a un autre endroit
	int x2=x, y2=y;
	switch(pair->getOrientation())
	{
		case ORIENTATION_EAST:
			x2=x+grid->getCellWidth();
			break;	
		
		case ORIENTATION_NORTH:
			y2=y-grid->getCellHeight();
			break;
			
		case ORIENTATION_SOUTH:
			y2=y+grid->getCellHeight();
			break;
			
		case ORIENTATION_WEST:
			x2=x-grid->getCellWidth();
			break;
	}
	
	fx::drawXY(animFromColor(pair->getPiece(1)->getColor())->getImage(), x2, y2, fx_transl, fx1);
	fx::drawXY(animFromColor(pair->getPiece(0)->getColor())->getImage(), x, y, fx_transl, fx1);
}

void Level8Bits::drawPieceXY(AbstractPiece *piece, int x, int y, AbstractLocalEffect *fx_transl)
{
	fx::drawXY(animFromColor(piece->getColor())->getImage(), x, y, fx_transl);
}

Anim *Level8Bits::particleFromColor(int color)
{
	switch(color)
	{
		case PIECE_COLOR_BLUE:
		{
			return m_gen_animBlue;
			break;
		}
		case PIECE_COLOR_RED:
		{
			return m_gen_animRed;
			break;
		}
		case PIECE_COLOR_YELLOW:
		{
			return m_gen_animYellow;
			break;
		}
		case PIECE_COLOR_PURPLE:
		{
			return m_gen_animPurple;
			break;
		}
		case PIECE_COLOR_GREEN:
		{
			return m_gen_animGreen;
			break;
		}
		case PIECE_COLOR_ORANGE:
		{
			return m_gen_animGrey;
			break;
		}
	}
	
	return NULL;
}

Anim *Level8Bits::animFromColor(int color, bool shielded)
{
	if(!shielded) //pieces normales
	{
		switch(color)
		{
			case PIECE_COLOR_BLUE:
			{
				return m_animBlue;
				break;
			}
			case PIECE_COLOR_RED:
			{
				return m_animRed;
				break;
			}
			case PIECE_COLOR_YELLOW:
			{
				return m_animYellow;
				break;
			}
			case PIECE_COLOR_PURPLE:
			{
				return m_animPurple;
				break;
			}
			case PIECE_COLOR_GREEN:
			{
				return m_animGreen;
				break;
			}
			case PIECE_COLOR_ORANGE:
			{
				return m_animGrey;
				break;
			}
		}
	}
	else //shielded
	{
		switch(color)
		{
			case PIECE_COLOR_BLUE:
			{
				return m_animBlueShield;
				break;
			}
			case PIECE_COLOR_RED:
			{
				return m_animRedShield;
				break;
			}
			case PIECE_COLOR_YELLOW:
			{
				return m_animYellowShield;
				break;
			}
			case PIECE_COLOR_PURPLE:
			{
				return m_animPurpleShield;
				break;
			}
			case PIECE_COLOR_GREEN:
			{
				return m_animGreenShield;
				break;
			}
			case PIECE_COLOR_ORANGE:
			{
				return m_animGreyShield;
				break;
			}
		}
	}
	
	return NULL;
}




