#include "code/game/levels/impl/cathedral/Meteor.h"
#include "code/game/gamedef.h"
#include "code/framework/State.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/utils.h"

using namespace utils;

//#define PARTGEN_POS_Y GAMEDEF_PIECE_HEIGHT/2

Meteor::Meteor(DJMCOContentLayered *mco, AbstractParticleGenerator *partgen, int partgen_shift_y)
{
	m_partgen_shift_y=partgen_shift_y;
	m_mco=mco;
	m_partgen=partgen;
	m_partgen->start();
	m_partgen->generationOn();
	m_partgen->setX(0);
	m_partgen->setY(m_partgen_shift_y);
	m_repeater=new Repeater(int(GAMEDEF_EXPLOSION_TIME), 1);
	
	m_state=new State();
}

Meteor::~Meteor()
{
	delete m_mco;
	delete m_partgen;
	delete m_repeater;
	delete m_state;
}


bool Meteor::finished()
{
	return m_repeater->finished();
}

AbstractParticleGenerator *Meteor::getGenerator()
{
	return m_partgen;
}


//drawable
void Meteor::draw(AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	m_mco->draw(fx1, fx2, fx3, fx4, fx5);
	
	fx::saveState(m_partgen);
	if(fx::applyFxOnPositionable(m_partgen, fx1, fx2, fx3, fx4, fx5)!=NULL)
		m_partgen->draw();
	fx::restoreState(m_partgen);
}

void Meteor::drawXY(int x, int y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	m_mco->drawXY(x, y, fx1, fx2, fx3, fx4, fx5);
	m_partgen->drawXY(x, y);
}


//playable
void Meteor::update()
{
	m_mco->update();
	m_partgen->update();
	m_repeater->update();
}

void Meteor::update(AbstractLocalEffect *fx_trans)
{
	m_mco->update();
	m_repeater->update();
	
//	if(fx_trans!=NULL)
//		enableDebug();
//	debug("Meteor::update : fx_trans != NULL");
	fx::saveState(m_partgen);
	fx::applyFxOnPositionable(m_partgen, fx_trans);
	m_partgen->update();
	fx::restoreState(m_partgen);
//	enableDebug(false);
}

void Meteor::start()
{
	m_mco->start();
	m_partgen->generationOff();
	m_repeater->start();
}

void Meteor::stop()
{
	m_mco->stop();
	m_partgen->stop();
	m_repeater->stop();
}

void Meteor::reset()
{
	m_mco->reset();
	m_partgen->reset();
	m_partgen->start();
	m_partgen->generationOn();
	m_repeater->reset();
}

void Meteor::restart()
{
	m_mco->restart();
	m_partgen->restart();
	m_repeater->restart();
}

bool Meteor::isRunning()
{
	return m_mco->isRunning();
}


//positionable
void Meteor::setX(int x)
{
	m_x=x;
	m_partgen->setX(x);
	m_mco->setX(x);
}

void Meteor::setY(int y)
{
	m_y=y;	
	m_partgen->setY(y+m_partgen_shift_y);
	m_mco->setY(y);
}

int Meteor::getX()
{
	return m_x;
}

int Meteor::getY()
{
	return m_y;
}


//statesaveable
State *Meteor::savestate(State * state)
{
	state->m_x=m_x;
	state->m_y=m_y;
	
	return state;
}

void Meteor::restorestate(State * state)
{
	m_x=state->m_x;
	m_y=state->m_y;
}







