#include "code/game/levels/impl/frozen/PDR4CLevelFrozen.h"
#include "code/framework/Pos.h"
#include "code/game/gamedef.h"
#include "code/framework/utils.h"
#include <oslib/oslib.h>
#include "code/framework/fx/EffectsLib.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include "code/framework/Properties.h"
#include "code/game/pieces/impl/PieceSimple.h"
#include "code/framework/anim/Anim.h"
#include "code/game/levels/utils/MCOContentSimple.h"
#include "code/game/levels/utils/MCOContentCombo.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomAngleTranslation.h"
#include "code/framework/particles/complete/animFactories/RandomAnimationFactory.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomColor.h"
#include "code/framework/check/loaders.h"

using namespace utils;

#define PDR4C_LEVEL_FROZEN_COLOR_WHITE RGB(255,255,255)
#define PDR4C_LEVEL_FROZEN_COLOR_DARKBLUE RGB(20,0,50)
#define PDR4C_LEVEL_FROZEN_COLOR_BLUE RGB(0,128,255)
#define PDR4C_LEVEL_FROZEN_COLOR_LIGHTBLUE RGB(128,255,255)
#define PDR4C_LEVEL_FROZEN_DURATION 166340.0f

string PDR4CLevelFrozen::getCode()
{
	return "pdr4cfrozen";
}

PDR4CLevelFrozen::PDR4CLevelFrozen(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{		
	Properties *props=new Properties("res/pdr4c/levels/frozen/frozen.properties");
		
	m_img_overlay=fwkLoadImageFilePNG("res/pdr4c/levels/frozen/overlay.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	m_img_overlay->x=56;
	m_img_overlay->y=9;
	
	
	m_songbanner=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/15/nosmooth/", "normal", "italic", props->getStrValue("songbanner.text1"), props->getStrValue("songbanner.text2", ""), props->getStrValue("songbanner.text3", ""), props->getStrValue("songbanner.text4", ""));
	
	m_comboDrawer=new PDR4CComboNumberDrawer(PDR4C_LEVEL_FROZEN_COLOR_LIGHTBLUE, PDR4C_LEVEL_FROZEN_COLOR_DARKBLUE, PDR4C_LEVEL_FROZEN_COLOR_WHITE, PDR4C_LEVEL_FROZEN_COLOR_BLUE, PDR4C_LEVEL_FROZEN_COLOR_LIGHTBLUE);
	
	m_scenario=new EventsScenario("res/pdr4c/levels/frozen/frozen.scenario");
	
	m_music=fwkLoadSoundFileMP3("res/pdr4c/levels/frozen/frozen.mp3", OSL_FMT_STREAM);
	
	
	m_addline=0;
	
	m_difficulty=difficulty;
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		debug("difficulty is easy");
		m_nbColors=props->getIntValue("easy.colors");
		m_dropspeed_slow=props->getIntValue("easy.dropspeed.slow");
		m_dropspeed_fast=props->getIntValue("easy.dropspeed.fast");
		m_linepattern=new LinePattern("res/pdr4c/levels/frozen/frozen_easy.pattern");
	}
	if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		debug("difficulty is normal");
		m_nbColors=props->getIntValue("normal.colors");
		m_dropspeed_slow=props->getIntValue("normal.dropspeed.slow");
		m_dropspeed_fast=props->getIntValue("normal.dropspeed.fast");
		m_linepattern=new LinePattern("res/pdr4c/levels/frozen/frozen_normal.pattern");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		debug("difficulty is hard");
		m_nbColors=props->getIntValue("hard.colors");
		m_dropspeed_slow=props->getIntValue("hard.dropspeed.slow");
		m_dropspeed_fast=props->getIntValue("hard.dropspeed.fast");
		m_linepattern=new LinePattern("res/pdr4c/levels/frozen/frozen_hard.pattern");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_VERYHARD)
	{
		debug("difficulty is vhard");
		m_nbColors=props->getIntValue("vhard.colors");
		m_dropspeed_slow=props->getIntValue("vhard.dropspeed.slow");
		m_dropspeed_fast=props->getIntValue("vhard.dropspeed.fast");
		m_linepattern=new LinePattern("res/pdr4c/levels/frozen/frozen_vhard.pattern");
	}
	m_dropspeed=m_dropspeed_slow;

	m_animBlue=new Anim(10, "ball", "res/pdr4c/levels/frozen/pieces/ball/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animRed=new Anim(10, "cloud", "res/pdr4c/levels/frozen/pieces/cloud/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animYellow=new Anim(10, "diamond", "res/pdr4c/levels/frozen/pieces/diamond/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animPurple=new Anim(10, "verglas", "res/pdr4c/levels/frozen/pieces/verglas/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animGreen=new Anim(10, "star", "res/pdr4c/levels/frozen/pieces/star/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animOrange=new Anim(10, "flake", "res/pdr4c/levels/frozen/pieces/snowflake/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animBlue->setReferencePointCenter();
	m_animRed->setReferencePointCenter();
	m_animYellow->setReferencePointCenter();
	m_animPurple->setReferencePointCenter();
	m_animGreen->setReferencePointCenter();
	m_animOrange->setReferencePointCenter();
	
	
	
	m_animComboExplode=new Anim(10, "combo", "res/pdr4c/levels/frozen/pieces/combo/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animComboExplode->setReferencePointCenter();
	m_animComboChange=new Anim(5, "combo_change_", "res/pdr4c/levels/frozen/pieces/combo/change/", 10, OSL_PF_8888, 1);
	m_animComboChange->setReferencePointCenter();
	
	
	m_imgCombo=new OSL_IMAGE *[22]; //tableau de pointeurs
	for(int i=0;i<=20;i++)
	{
		m_imgCombo[i]=fwkLoadImageFilePNG("res/pdr4c/levels/frozen/pieces/combo/numbers/combo_num_"+toString(i)+".png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		oslImageSetRotCenter(m_imgCombo[i]);
	}
	m_imgCombo[21]=fwkLoadImageFilePNG("res/pdr4c/levels/frozen/pieces/combo/numbers/combo_num_more.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_imgCombo[21]);
	
	m_fx_shake=new ShakeEffect(props->getIntValue("fx.shake.width"), 0, props->getIntValue("fx.shake.time"));
	
	m_thunderCounter=0;
	m_thunderTime=props->getIntValue("thunder.time");
	m_stormCounter=0;
	m_stormTime=props->getIntValue("storm.time");
	m_stormOn=false;
	
	//particules neige devant
	Anim *flake=new Anim(2, "flake", "res/pdr4c/levels/frozen/snowflakes/flake5/", 30, OSL_PF_8888);
	m_gen_front_throughputLowParticles=props->getIntValue("generator.front.throughput.low.particles");
	m_gen_front_throughputLowFrames=props->getIntValue("generator.front.throughput.low.frames");
	m_gen_front_throughputHighParticles=props->getIntValue("generator.front.throughput.high.particles");
	m_gen_front_throughputHighFrames=props->getIntValue("generator.front.throughput.high.frames");
	m_gen_front_lowTime=props->getIntValue("generator.front.low.time");
	m_gen_front_highTime=props->getIntValue("generator.front.high.time");
	m_gen_front_lowLifespan=props->getIntValue("generator.front.low.lifespan");
	m_gen_front_highLifespan=props->getIntValue("generator.front.high.lifespan");
			
	m_gen_front = new ParticleGeneratorSegment(	props->getIntValue("generator.front.x"), 
												props->getIntValue("generator.front.y"), 
												props->getIntValue("generator.front.direction"), 
												props->getIntValue("generator.front.scale"), 
												m_gen_front_throughputLowParticles, 
												m_gen_front_throughputLowFrames, 
												m_gen_front_lowTime, 
												props->getIntValue("generator.front.distance"), 
												m_gen_front_lowLifespan, 
												flake);
	PFXFRandomAngleTranslation *localfx1=new PFXFRandomAngleTranslation(
						props->getIntValue("generator.front.angle.min"),
						props->getIntValue("generator.front.angle.max"));
	m_gen_front->setLocalEffect1(localfx1);
	
	//particules neige derrière
	Anim *flake1=new Anim(2, "flake", "res/pdr4c/levels/frozen/snowflakes/flake1/", 30, OSL_PF_8888);
	Anim *flake2=new Anim(2, "flake", "res/pdr4c/levels/frozen/snowflakes/flake2/", 30, OSL_PF_8888);
	Anim *flake3=new Anim(2, "flake", "res/pdr4c/levels/frozen/snowflakes/flake3/", 30, OSL_PF_8888);
	Anim *flake4=new Anim(2, "flake", "res/pdr4c/levels/frozen/snowflakes/flake4/", 30, OSL_PF_8888);
	RandomAnimationFactory *flakes=new RandomAnimationFactory(flake1);
	flakes->addAnimation(flake2);
	flakes->addAnimation(flake3);
	flakes->addAnimation(flake4);
	
	m_gen_back_throughputLowParticles=props->getIntValue("generator.back.throughput.low.particles");
	m_gen_back_throughputLowFrames=props->getIntValue("generator.back.throughput.low.frames");
	m_gen_back_throughputHighParticles=props->getIntValue("generator.back.throughput.high.particles");
	m_gen_back_throughputHighFrames=props->getIntValue("generator.back.throughput.high.frames");
	m_gen_back_lowTime=props->getIntValue("generator.back.low.time");
	m_gen_back_highTime=props->getIntValue("generator.back.high.time");
	m_gen_back_lowLifespan=props->getIntValue("generator.back.low.lifespan");
	m_gen_back_highLifespan=props->getIntValue("generator.back.high.lifespan");
			
	m_gen_back = new ParticleGeneratorSegment(	props->getIntValue("generator.back.x"), 
												props->getIntValue("generator.back.y"), 
												props->getIntValue("generator.back.direction"), 
												props->getIntValue("generator.back.scale"), 
												m_gen_back_throughputLowParticles, 
												m_gen_back_throughputLowFrames, 
												m_gen_back_lowTime, 
												props->getIntValue("generator.back.distance"), 
												m_gen_back_lowLifespan, 
												flakes);
	/*PFXFRandomAngleTranslation */localfx1=new PFXFRandomAngleTranslation(
						props->getIntValue("generator.back.angle.min"),
						props->getIntValue("generator.back.angle.max"));
	m_gen_back->setLocalEffect1(localfx1);
	PFXFRandomColor *globalfx1=new PFXFRandomColor(PDR4C_LEVEL_FROZEN_COLOR_WHITE);
	globalfx1->addColor(PDR4C_LEVEL_FROZEN_COLOR_LIGHTBLUE);
	globalfx1->addColor(PDR4C_LEVEL_FROZEN_COLOR_BLUE);
	m_gen_back->setGlobalEffect1(globalfx1);
	
	
	//la jauge de combo
	m_gauge_colors[0]=PDR4C_LEVEL_FROZEN_COLOR_WHITE;
	m_gauge_colors[1]=PDR4C_LEVEL_FROZEN_COLOR_BLUE;
	m_gauge_colors[2]=PDR4C_LEVEL_FROZEN_COLOR_LIGHTBLUE;
	m_gauge_current_color_num=0;
	m_gauge_delay=props->getIntValue("gauge.blink.delay");
	m_gauge_counter=m_gauge_delay;
	
	
	m_img_pause=fwkLoadImageFilePNG("res/pdr4c/levels/frozen/frozen2.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_img_pause);
	m_fx_pause_shake=new ShakeEffect(0, 20, 50);
	m_fx_pause_shake->start();
	
	m_gen_front->generationOff();
	m_gen_back->generationOff();
	
	delete props;
}

void PDR4CLevelFrozen::reset()
{
	//Properties *props=new Properties("res/pdr4c/levels/frozen/frozen.properties");
	
	m_songbanner->reset();
	m_comboDrawer->reset();
	m_scenario->reset();
	oslStopSound(m_music);
	m_linepattern->reset();
	m_dropspeed=m_dropspeed_slow;
	m_fx_shake->stop();
	m_fx_shake->reset();
	m_gen_front->generationOff();
	m_gen_back->generationOff();
	m_addline=0;
	//delete props;
}

PDR4CLevelFrozen::~PDR4CLevelFrozen()
{
	oslDeleteImage(m_img_overlay);
	oslDeleteImage(m_img_pause);
	delete m_fx_pause_shake;
	
	delete m_linepattern;
	delete m_songbanner;
	delete m_comboDrawer;
	delete m_scenario;
	
	oslDeleteSound(m_music);
	
	delete m_animBlue;
	delete m_animRed;
	delete m_animYellow;
	delete m_animPurple;
	delete m_animGreen;
	delete m_animOrange;
	
	delete m_animComboExplode;
	delete m_animComboChange;
	
	for(int i=0;i<=21;i++)
	{
		oslDeleteImage(m_imgCombo[i]);
	}
	
	delete[] m_imgCombo;
	
	delete m_fx_shake;
	
	//enableDebug();
	//debug("delete m_gen_front..");
	delete m_gen_front;
	//debug("delete m_gen_back");
	delete m_gen_back;
	//debug("end delete PDR4CLevelFrozen");
	//enableDebug(false);
}


//dessin avec une paire
void PDR4CLevelFrozen::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	if(!isPaused())
	{
		//currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
		Pos gridPos=currentPair->getGridPos(0);
		Pos pixPos=grid->getContentPosition(gridPos.x, gridPos.y);
		drawPairXY(currentPair, pixPos.x, pixPos.y, grid, fx_trans);
	}
	drawAfter(grid, nextPair, cc);
}

//dessin avec 2 pieces
void PDR4CLevelFrozen::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	drawPieceXY(pieceConnected, pixPosConnected.x, pixPosConnected.y);
	drawPieceXY(pieceUnconnected, pixPosUnconnected.x, pixPosUnconnected.y, fx_trans);
	
	drawAfter(grid, nextPair, cc);
}

//dessin avec rien
void PDR4CLevelFrozen::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
}

void PDR4CLevelFrozen::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	if(m_thunderCounter>0)
	{
		oslClearScreen(PDR4C_LEVEL_FROZEN_COLOR_LIGHTBLUE);
		m_thunderCounter--;
	}
	else
	{
		oslClearScreen(PDR4C_LEVEL_FROZEN_COLOR_DARKBLUE);
	}
	
	if(m_difficulty!=GAMEDEF_DIFFICULTY_EASY)
		m_gen_back->draw();
	
	//barre de combo
	oslDrawFillRect(384, 262-int((cc->getTime()*252)/cc->getTimeTotalAvailable()), 398, 262, m_gauge_colors[m_gauge_current_color_num]);
	
	oslDrawImage(m_img_overlay);
			
	if(isPaused())
	{
		fx::drawXY(m_img_pause, 243,136,m_fx_pause_shake);
	}
	else
	{
		//dessin du contenu de la grille
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco->getPiece()!=NULL)
				{
					Pos pos=grid->getContentPosition(i, j);
					AbstractPiece *p=mco->getPiece();
					
					if(p->getType()==PIECE_TYPE_NORMAL || p->getType()==PIECE_TYPE_COMBO)
					{
						if(mco->getMCOContent()!=NULL)
						{
							mco->getMCOContent()->drawXY(pos.x,pos.y, m_fx_shake->isRunning()?m_fx_shake:NULL);
						}
					}
					else
					{
						p->drawXY(pos.x,pos.y);
					}
				}
			}	
		}
	}
	
}

void PDR4CLevelFrozen::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	if(!isPaused())
		drawPairXY(nextPair, 67, 30, grid);	
		
	m_gen_front->draw();
		
	m_songbanner->draw();
		
	m_comboDrawer->drawXY(240, 180);
	
}

void PDR4CLevelFrozen::start_specific()
{
	oslPlaySound(m_music, 0);
	m_scenario->restart();
}

//appelé uniquement si le level n'est pas en pause.
void PDR4CLevelFrozen::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	//cas special pour la pdrom: on utilise des anims séparées
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			AbstractPiece *p=mco->getPiece();
			if(p!=NULL)
			{
				if(p->getType()==PIECE_TYPE_NORMAL)
				{
					//on cree si une piece n'en a pas
					if(mco->getMCOContent()==NULL)
					{
						Anim *anim=animFromColor(p->getColor());
						mco->setMCOContent(new MCOContentSimple(new Anim(*anim)));
					}				
				}
				else if(p->getType()==PIECE_TYPE_COMBO)
				{
					//remplir si c'est une piece combo
					if(mco->getMCOContent()==NULL)
					{
						int nbCombos=p->specialEvent(PIECE_EVENT_COMBO_GET);
						Anim *shine=new Anim(*m_animComboChange);
						Anim *explode=new Anim(*m_animComboExplode);
						mco->setMCOContent(new MCOContentCombo(explode, shine, m_imgCombo, nbCombos));
					}
					
					//on met a jour le nombre de combos
					int nbCombosPiece=p->specialEvent(PIECE_EVENT_COMBO_GET);
					int nbCombosMCOContent=mco->getMCOContent()->specialEvent(MCOCONTENT_EVENT_COMBO_GETNB);
					if(nbCombosPiece!=nbCombosMCOContent)
					{
						mco->getMCOContent()->specialEvent(MCOCONTENT_EVENT_COMBO_DECREMENT);
					}
				}
				//on démarre si la piece explose
				if(mco->isColorExploding() || mco->isShapeExploding() || mco->isSpecialExploding())
					mco->getMCOContent()->start();
			}
			else//mco->getPiece()==NULL
			{
				if(mco->getMCOContent()!=NULL)
				{
					mco->deleteMCOContent();
				}
			}
			
			if(mco->getMCOContent()!=NULL) //peu importe le cas, on update
			{
				mco->getMCOContent()->update();
			}
		}
	}
	
	//la jauge
	m_gauge_counter--;
	if(m_gauge_counter<=0)
	{
		m_gauge_current_color_num++;
		if(m_gauge_current_color_num==3)
			m_gauge_current_color_num=0;
		m_gauge_counter=m_gauge_delay;
	}
	
	m_songbanner->update();
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	m_scenario->update();
	m_fx_shake->update();
	m_gen_front->update();
	m_gen_back->update();
	
	
	if(m_stormOn)
	{
		m_stormCounter--;
		if(m_stormCounter<=0)
		{
			stopStorm();
		}
	}
	
	//evenements du scenario
	if(m_scenario->hasEvent(1)) //fiout' (cymbales aspirées)
	{
		m_addline++;
		if(!m_gen_front->isGenerationOn())
		{
			m_gen_front->start();
			m_gen_front->generationOn();
		}
	}
	if(m_scenario->hasEvent(2)) //debut criuuuuuuuuuuuuuuuu
	{
		m_fx_shake->restart();
		if(m_difficulty==GAMEDEF_DIFFICULTY_HARD || m_difficulty==GAMEDEF_DIFFICULTY_VERYHARD)
		{
			m_addline+=2;
		}
		
		if(!m_gen_back->isGenerationOn())
		{
			m_gen_back->start();
			m_gen_back->generationOn();
		}
		
		startStorm();
	}
	if(m_scenario->hasEvent(3)) //fin criuuuuuuuuuuuuuuuu
	{
		m_fx_shake->stop();
	}
	if(m_scenario->hasEvent(4)) //debut ligne basses
	{
		m_dropspeed=m_dropspeed_fast;
	}
	if(m_scenario->hasEvent(5)) //fin ligne basses
	{
		m_dropspeed=m_dropspeed_slow;
	}
	if(m_scenario->hasEvent(6)) //tin tin
	{
		m_thunderCounter+=m_thunderTime;
		m_addline++;
		startStorm();
	}
}

void PDR4CLevelFrozen::update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
//	enableDebug();
//	debug("PDR4CLevelFrozen::update_pause_specific, yeah ok!");
//	enableDebug(false);
	m_fx_pause_shake->update();
}

void PDR4CLevelFrozen::pause_specific() 
{
	oslPauseSound(m_music, 1);
	m_scenario->pause();
}

void PDR4CLevelFrozen::resume_specific()
{
	oslPauseSound(m_music, 0);
	m_scenario->start();
}

list<AbstractPiece *> *PDR4CLevelFrozen::getNextLine()
{
	return m_linepattern->getNextLine();
}

bool PDR4CLevelFrozen::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int PDR4CLevelFrozen::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return m_dropspeed;
}

int PDR4CLevelFrozen::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return -1;
}

void PDR4CLevelFrozen::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_TRIANGLE)
	{
				
	}
	else if(key==OSL_KEY_SQUARE)
	{
		
	}
}

bool PDR4CLevelFrozen::finished()
{
	if(m_scenario->getTime()>=PDR4C_LEVEL_FROZEN_DURATION)
	{
		return true;
	}
	return false;
}

float PDR4CLevelFrozen::getDuration()
{
	return PDR4C_LEVEL_FROZEN_DURATION;
}

Pair *PDR4CLevelFrozen::getNewPair()
{	
	Pair *p=new Pair(randomPiece(), randomPiece());
	int posX=int(m_nbCols/2)-1;
	int posY=-1; 
	p->goTo(posX, posY);
	
	return p;
}



AbstractPiece *PDR4CLevelFrozen::randomPiece()
{
	//suivant la difficulté, il y a + ou - de couleurs à l'écran
	int color=random(0,m_nbColors-1);
	PieceSimple *p=NULL;
	switch(color)
	{
		case 0:
		{
			p=new PieceSimple(PIECE_SHAPE_CIRCLE, PIECE_COLOR_BLUE);
			break;
		}
		case 1:
		{
			p=new PieceSimple(PIECE_SHAPE_SQUARE, PIECE_COLOR_RED);
			break;
		}
		case 2:
		{
			p=new PieceSimple(PIECE_SHAPE_DIAMOND, PIECE_COLOR_YELLOW);	
			break;
		}
		case 3:
		{
			p=new PieceSimple(PIECE_SHAPE_STAR, PIECE_COLOR_PURPLE);	
			break;
		}
		case 4:
		{
			p=new PieceSimple(PIECE_SHAPE_CROSS, PIECE_COLOR_GREEN);
			break;
		}
		case 5:
		{
			p=new PieceSimple(PIECE_SHAPE_TRIANGLE, PIECE_COLOR_ORANGE);
			break;
		}
	}
	
	
	
	return p;
}

void PDR4CLevelFrozen::drawPairXY(Pair *pair, int x, int y, GameGrid *grid, AbstractLocalEffect *fx_transl, AbstractLocalEffect *fx1)
{
	//suivant l'orientation, la 2eme piece est a un autre endroit
	int x2=x, y2=y;
	switch(pair->getOrientation())
	{
		case ORIENTATION_EAST:
			x2=x+grid->getCellWidth();
			break;	
		
		case ORIENTATION_NORTH:
			y2=y-grid->getCellHeight();
			break;
			
		case ORIENTATION_SOUTH:
			y2=y+grid->getCellHeight();
			break;
			
		case ORIENTATION_WEST:
			x2=x-grid->getCellWidth();
			break;
	}
		
	fx::drawXY(animFromColor(pair->getPiece(1)->getColor())->getImage(), x2, y2, fx_transl, fx1);
	fx::drawXY(animFromColor(pair->getPiece(0)->getColor())->getImage(), x, y, fx_transl, fx1);
}

void PDR4CLevelFrozen::drawPieceXY(AbstractPiece *piece, int x, int y, AbstractLocalEffect *fx_transl)
{
	fx::drawXY(animFromColor(piece->getColor())->getImage(), x, y, fx_transl);
}

Anim *PDR4CLevelFrozen::animFromColor(int color)
{
	switch(color)
	{
		case PIECE_COLOR_BLUE:
		{
			return m_animBlue;
			break;
		}
		case PIECE_COLOR_RED:
		{
			return m_animRed;
			break;
		}
		case PIECE_COLOR_YELLOW:
		{
			return m_animYellow;
			break;
		}
		case PIECE_COLOR_PURPLE:
		{
			return m_animPurple;
			break;
		}
		case PIECE_COLOR_GREEN:
		{
			return m_animGreen;
			break;
		}
		case PIECE_COLOR_ORANGE:
		{
			return m_animOrange;
			break;
		}
	}
	
	return NULL;
}

void PDR4CLevelFrozen::startStorm()
{
	m_stormOn=true;
	
	m_gen_front->setThroughput(m_gen_front_throughputHighParticles, m_gen_front_throughputHighFrames);
	m_gen_front->setPLifespan(m_gen_front_highLifespan);
	m_gen_front->setPTranslTime(m_gen_front_highTime);
	
	m_gen_back->setThroughput(m_gen_back_throughputHighParticles, m_gen_back_throughputHighFrames);
	m_gen_back->setPLifespan(m_gen_back_highLifespan);
	m_gen_back->setPTranslTime(m_gen_back_highTime);
	
	m_stormCounter=m_stormTime;
}

void PDR4CLevelFrozen::stopStorm()
{
	m_stormOn=false;
	
	m_gen_front->setThroughput(m_gen_front_throughputLowParticles, m_gen_front_throughputLowFrames);
	m_gen_front->setPLifespan(m_gen_front_lowLifespan);
	m_gen_front->setPTranslTime(m_gen_front_lowTime);
	
	m_gen_back->setThroughput(m_gen_back_throughputLowParticles, m_gen_back_throughputLowFrames);
	m_gen_back->setPLifespan(m_gen_back_lowLifespan);
	m_gen_back->setPTranslTime(m_gen_back_lowTime);
}







