#ifndef PDR4CLEVELFROZEN_H_
#define PDR4CLEVELFROZEN_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/pdr4c/PDR4CComboNumberDrawer.h"
#include "code/game/EventsScenario.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/anim/Anim.h"
#include "code/framework/particles/complete/impl/ParticleGeneratorSegment.h"


using namespace std;

class PDR4CLevelFrozen : public AbstractLevel
{
	private:
			
		OSL_IMAGE *m_img_overlay;
		LinePattern *m_linepattern;
			
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); 
		void resume_specific();
				
		SongBanner *m_songbanner;
		
		PDR4CComboNumberDrawer * m_comboDrawer;
		
		EventsScenario *m_scenario;
		OSL_SOUND* m_music;
		
		int m_dropspeed; //current
		int m_dropspeed_slow;
		int m_dropspeed_fast;
		
		int m_addline; //ajoute 1 qd on veut qu'une ligne monte
								
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
				
		int m_difficulty;
		int m_nbColors; //nombre de couleurs pour les pieces
		
		//animations qui remplacent les pieces normales
		Anim *m_animBlue,*m_animRed,*m_animYellow,*m_animPurple,*m_animGreen,*m_animOrange;
		//animations et images qui remplacent les pieces combo
		Anim *m_animComboExplode, *m_animComboChange;
		OSL_IMAGE **m_imgCombo; //tableau de pointeurs
		
		ShakeEffect *m_fx_shake;
		
		//thunder: flash dans le fond (2 rapprochés)
		int m_thunderCounter;
		int m_thunderTime;
		//storm: neige + forte, déclenchée par thunder et vibrations
		int m_stormCounter;
		int m_stormTime;
		bool m_stormOn;
		
		//particules pour la neige
		ParticleGeneratorSegment *m_gen_front;
		int m_gen_front_throughputLowParticles;
		int m_gen_front_throughputLowFrames;
		int m_gen_front_throughputHighParticles;
		int m_gen_front_throughputHighFrames;
		int m_gen_front_lowTime;
		int m_gen_front_highTime;
		int m_gen_front_lowLifespan;
		int m_gen_front_highLifespan;
		
		ParticleGeneratorSegment *m_gen_back;
		int m_gen_back_throughputLowParticles;
		int m_gen_back_throughputLowFrames;
		int m_gen_back_throughputHighParticles;
		int m_gen_back_throughputHighFrames;
		int m_gen_back_lowTime;
		int m_gen_back_highTime;
		int m_gen_back_lowLifespan;
		int m_gen_back_highLifespan;
		
		//pour faire clignoter la jauge de combo
		int m_gauge_colors[3];
		int m_gauge_current_color_num;
		int m_gauge_counter;
		int m_gauge_delay;
		
		OSL_IMAGE * m_img_pause;
		ShakeEffect *m_fx_pause_shake;
		
		AbstractPiece *randomPiece();
		void drawPairXY(Pair *pair, int x, int y, GameGrid *grid, AbstractLocalEffect *fx_transl=NULL, AbstractLocalEffect *fx1=NULL);
		void drawPieceXY(AbstractPiece *piece, int x, int y, AbstractLocalEffect *fx_transl=NULL);
		Anim *animFromColor(int color);
		void startStorm();
		void stopStorm();
		
		
	public:
		PDR4CLevelFrozen(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~PDR4CLevelFrozen();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		
		bool finished();
		
		list<AbstractPiece *> *getNextLine();
		
		//surchargé pour avoir les bonnes pieces
		Pair *getNewPair();
		
		float getDuration();
		
		void reset();
};


#endif /*PDR4CLEVELFROZEN_H_*/
