#ifndef LEVELPROJECTSEXY_H_
#define LEVELPROJECTSEXY_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/ComboNumberDrawer.h"
#include "code/game/EventsScenario.h"
#include "code/framework/anim/Anim.h"
#include "code/framework/Properties.h"
#include "code/game/levels/impl/projectsexy/SexyMCOContentParticles.h"
#include "code/game/levels/utils/JokerDrawer.h"

using namespace utils;

#define LEVEL_SEXY_DRAWTYPE_PAIR 0
#define LEVEL_SEXY_DRAWTYPE_PIECES 1
#define LEVEL_SEXY_DRAWTYPE_NOTHING 2

class LevelProjectSexy : public AbstractLevel
{
	private:
		
		ComboNumberDrawer * m_comboDrawer;
		EventsScenario *m_scenario;
		LinePattern *m_linepattern;
		Delayer *m_delayer;
		bool m_endExplosion;
		
		OSL_SOUND *m_music;
		OSL_IMAGE *m_img_grid; //overlay
		OSL_IMAGE *m_img_kti;
		OSL_IMAGE *m_img_pause;
		
		//songbanner
		SongBanner *m_songbanner;
		//pieces combos
		Anim *m_anim_comboturn;
		int m_ctspeed_slow;
		int m_ctspeed_med;
		int m_ctspeed_fast;
		int m_ctalpha;
		ProgressiveTintEffect *m_fx_combo;
		int m_ctintspeed_slow;
		int m_ctintspeed_medium;
		int m_ctintspeed_fast;
		
		//clouds
		OSL_IMAGE **m_img_clouds;
		SerialLocalEffect **m_fx_clouds; //mouvements lents
		int m_nbClouds;
		
		//pfiou dancer
		OSL_IMAGE *m_img_dancer;
		ProgressiveAlphaEffect *m_fx_dancer;
		ParticleGeneratorPoint *m_partgen_dancer;
		
		//animations qui remplacent les pieces normales
		Anim *m_animBlue,*m_animRed,*m_animYellow,*m_animPurple,*m_animGreen,*m_animOrange;
		//animations et images qui remplacent les pieces combo
		Anim *m_animComboExplode, *m_animComboChange;
		OSL_IMAGE **m_imgCombo; //tableau de pointeurs
		
		//caracteristiques du stage
		int m_addline;
		int m_difficulty;
		int m_dropspeed; //current
		int m_dropspeed_slow;
		int m_dropspeed_medium;
		int m_dropspeed_fast;
		int m_upspeed; //current
		int m_upspeed_slow;
		int m_upspeed_medium;
		int m_upspeed_fast;
		int m_nbColors;
		
		//flash
		int* m_flash_clouds_colorsfrom; //tableau
		int* m_flash_clouds_colorsto; //tableau
		ProgressiveTintEffect *m_fx_flash_clouds;
		int* m_flash_kti_colorsfrom; //tableau
		int* m_flash_kti_colorsto; //tableau
		ProgressiveTintEffect *m_fx_flash_kti;
		int m_flash_currentcolor;
		int m_flash_nbcolors;
		
		//piece particules
		int m_blocParticles_colorFrom;
		int m_blocParticles_colorTo;
		int m_blocParticles_distance;
		int m_blocParticles_time;
		int m_blocParticles_frames;
		int m_blocParticles_particles;
		
		//les generateurs pour next paire, current paire et pieces connected et unconnected
		SexyMCOContentParticles *m_smcocp_next1;
		SexyMCOContentParticles *m_smcocp_next2;
		SexyMCOContentParticles *m_smcocp_current1;
		SexyMCOContentParticles *m_smcocp_current2;
		SexyMCOContentParticles *m_smcocp_connected;
		SexyMCOContentParticles *m_smcocp_unconnected;
		int m_lastdraw;
		
		//tremblement de la grille (uniquement grille? nuages? certaines pieces?)
		SerialLocalEffect *m_fx_shake;
		
		//gestion du joker
		int m_jgauge_size;
		int m_jgauge_add;
		int m_jgauge_frem;
		int m_jgauge_current;
		JokerDrawer *m_jokerDrawer;
		
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); 
		void resume_specific();
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
				
		//void drawPairXY(Pair *pair, int x, int y, GameGrid *grid, AbstractLocalEffect *fx_transl=NULL, AbstractLocalEffect *fx1=NULL);
		//void drawPieceXY(AbstractPiece *piece, int x, int y, AbstractLocalEffect *fx_transl=NULL);
		
		void initAnims();
		Anim *animFromColor(int color);
		AbstractPiece *randomPiece();
		void initFlash(Properties *props);
		void restartFlash();
		SexyMCOContentParticles *newParticlesMCO(Anim *anim);
		void initSmcocp(Pair *currentPair, Pair *nextPair);
		Pos getCurrentPairPixPos(int piecenum, GameGrid *grid, Pair *currentPair); //0: 1ere piece, 1: 2eme piece
		
		void drawPieceUnconnected(AbstractPiece *piece, int x, int y, SerialLocalEffect *fx_transl=NULL);
		void drawPieceConnected(AbstractPiece *piece, int x, int y);
		void drawNextPair(Pair *pair);
		void drawCurrentPair(Pair *pair, int x, int y, GameGrid *grid, SerialLocalEffect *fx_transl=NULL, AbstractLocalEffect *fx1=NULL);
		
		void initClouds(Properties *props);
		void initShake(Properties *props);
		void initDancer(Properties *props);
		void initComboTurn(Properties *props);
		
		void changeToJoker(Pair *pair);
		
		void drawOverlay(ComboCounter *cc);
		
		//string m_debugstr;
		//bool m_debugbool;
		
		void startLastExplosion();
		
	public:
		LevelProjectSexy(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~LevelProjectSexy();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		
		bool finished();
		
		list<AbstractPiece *> *getNextLine();
		
		//surchargé pour avoir les bonnes pieces
		Pair *getNewPair();
		
		float getDuration();
		
		void reset();
};

#endif /*LEVELPROJECTSEXY_H_*/
