#include "code/game/levels/impl/test/LevelTest.h"
#include "code/framework/Pos.h"
#include "code/game/gamedef.h"
#include "code/game/pieces/impl/PieceJoker.h"
#include "code/framework/check/loaders.h"

#define LEVEL_TEST_EASY_DROPSPEED 60
#define LEVEL_TEST_EASY_UPSPEED 360
#define LEVEL_TEST_EASY_NBCOLS 10
#define LEVEL_TEST_EASY_NBLINES 10

#define LEVEL_TEST_NORMAL_DROPSPEED 25
#define LEVEL_TEST_NORMAL_UPSPEED 160
#define LEVEL_TEST_NORMAL_NBCOLS 12
#define LEVEL_TEST_NORMAL_NBLINES 12

#define LEVEL_TEST_HARD_DROPSPEED 30
#define LEVEL_TEST_HARD_UPSPEED 200
#define LEVEL_TEST_HARD_NBCOLS 16
#define LEVEL_TEST_HARD_NBLINES 12

string LevelTest::getCode()
{
	return "test";
}

LevelTest::LevelTest(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{
	//enableDebug();
	debug("construction level test");
	//m_dropSpeed= //getInitialDropSpeed();
	//m_upSpeed= //getInitialUpSpeed();	
	
	if(difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		m_gridImage=fwkLoadImageFilePNG("res/levels/test/grid_easy.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
		m_nbLines=LEVEL_TEST_EASY_NBLINES;
		m_nbCols=LEVEL_TEST_EASY_NBCOLS;
		m_linepattern=new LinePattern("res/levels/test/test_easy.pattern");
		m_dropSpeed=LEVEL_TEST_EASY_DROPSPEED;
		m_upSpeed=LEVEL_TEST_EASY_UPSPEED;
		m_delayer=new Delayer(10, 2);
	}
	else if(difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_gridImage=fwkLoadImageFilePNG("res/levels/test/grid_normal.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
		m_nbCols=LEVEL_TEST_NORMAL_NBCOLS; //GAMEDEF_NB_LINES;
		m_nbLines=LEVEL_TEST_NORMAL_NBLINES; //GAMEDEF_NB_COLS;
		m_linepattern=new LinePattern("res/levels/test/test_normal.pattern");
		m_dropSpeed=LEVEL_TEST_NORMAL_DROPSPEED;
		m_upSpeed=LEVEL_TEST_NORMAL_UPSPEED;
		m_delayer=new Delayer(10, 4);
	}
	else //hard
	{
		m_gridImage=fwkLoadImageFilePNG("res/levels/test/grid_hard.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
		m_nbCols=LEVEL_TEST_HARD_NBCOLS;
		m_nbLines=LEVEL_TEST_HARD_NBLINES;
		m_linepattern=new LinePattern("res/levels/test/test_hard.pattern");
		m_dropSpeed=LEVEL_TEST_HARD_DROPSPEED;
		m_upSpeed=LEVEL_TEST_HARD_UPSPEED;
		m_delayer=new Delayer(10, 4);
	}
	
	m_bgImage=fwkLoadImageFilePNG("res/levels/test/bgtest.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	m_bgImage->x=0;
	m_bgImage->y=0;
	
	m_addline=0;
	
	m_linecounter=0;
	
	m_songbanner=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", "I'M LOOKING FOR GRAPHIC ARTISTS!!", "help me make this look like something!!", "PM me on QJ.net I'm daaa57150", "GFX ARTISTS NEEDED!!");
	m_comboDrawer=new ComboNumberDrawer();
	
	m_putJoker=false;
	m_addspecialexplosion=false;
	
	//enableDebug();
	debug("construction level test ok");
}

void LevelTest::reset()
{
	m_linepattern->reset();
	m_addline=0;
	m_linecounter=0;
	m_songbanner->reset();
	m_comboDrawer->reset();
	m_putJoker=false;
	m_addspecialexplosion=false;
}

LevelTest::~LevelTest()
{
	oslDeleteImage(m_gridImage);
	oslDeleteImage(m_bgImage);
	
	delete m_linepattern;
	delete m_delayer;
	delete m_songbanner;
	delete m_comboDrawer;
}

//dessin avec une paire
void LevelTest::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	//enableDebug();
	debug("LevelTest::draw avec paire");
	drawBefore(grid, nextPair, cc);
	//debug("draw current pair...");
	currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
	
	drawAfter(grid, nextPair, cc);
	debug("draw ok!");
}

//dessin avec 2 pieces
void LevelTest::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	debug("LevelTest::draw avec paire");
	drawBefore(grid, nextPair, cc);
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	pieceConnected->drawXY(pixPosConnected.x, pixPosConnected.y);
	pieceUnconnected->drawXY(pixPosUnconnected.x, pixPosUnconnected.y, fx_trans);
	
	drawAfter(grid, nextPair, cc);
	debug("draw ok!");
}

//dessin avec rien
void LevelTest::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	debug("LevelTest::draw avec paire");
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
	debug("draw ok!");
}

void LevelTest::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	oslDrawImage(m_bgImage);
	//	grid->draw(); //TODO: ajouter les effets 
	oslDrawImageXY(m_gridImage, grid->getPosX(), grid->getPosY());
	
	//debug("dessin du contenu de la grille...");
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			if(mco->getPiece()!=NULL)
			{
				Pos pos=grid->getContentPosition(i, j);
				mco->getPiece()->drawXY(pos.x,pos.y);
			}
		}	
	}
}

void LevelTest::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	nextPair->drawXY(400, 80, grid, APPLY_FX_ON_NONE);
	oslDrawFillRect(400, 270-int((cc->getTime()*260)/cc->getTimeTotalAvailable()), 415, 270, RGB(255,0,0));
	m_songbanner->draw();
	m_comboDrawer->drawXY(240, 180);
}

void LevelTest::start_specific()
{
	
}

//appelé uniquement si le level n'est pas en pause.
void LevelTest::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	m_delayer->update();
	if(m_delayer->get())
	{
		m_addline++;
	}
	m_songbanner->update();
	
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	
	if(m_putJoker)
	{
		if(nextPair==NULL)
			fatalError("LevelTest::update_specific nextPair is NULL!");
		
		m_putJoker=false;
		nextPair->setPiece0(new PieceJoker(nextPair->getPiece(0)->getShape(), nextPair->getPiece(0)->getColor()));
		//enableDebug();
		//debug("joker ok!");
		//enableDebug(false);
	}
	
}

void LevelTest::pause_specific() //pause, pour par exemple utiliser spliff politics
{
	
}

void LevelTest::resume_specific()
{
	
}

	
		
/*LinePattern *LevelTest::getLevelPieces() //TODO: renvoyer un tableau de pieces a nettoyer
{
	return m_linepattern;
}*/

list<AbstractPiece *> *LevelTest::getNextLine()
{
	debug("get next line!");
	if(m_linepattern!=NULL)
	{
		m_linecounter++;
		return m_linepattern->getNextLine();
	}
	
	return NULL;
}

bool LevelTest::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	debug("get add line event");
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int LevelTest::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return m_dropSpeed;
}

int LevelTest::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return m_upSpeed;
}

void LevelTest::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_SQUARE)
	{
		//m_addline++;
		//m_putJoker=true;
		oslMirrorImageH(m_bgImage);
	}
	if(key==OSL_KEY_TRIANGLE)
	{
		m_addspecialexplosion=true;
	}
}

bool LevelTest::finished()
{
	if(m_linecounter>=m_linepattern->getNbLinesAvailable())
	{
		return true;
	}
	return false;
}

float LevelTest::getDuration()
{
	return -1;
}

int LevelTest::getNbSpecialExplosions()
{
	int ret=0;
	if(m_addspecialexplosion)
	{
		ret++;
		m_addspecialexplosion=false;
	}
	return ret;
}
























