#include "code/game/levels/impl/tetris/LevelTetris.h"
#include "code/framework/Pos.h"
#include "code/game/gamedef.h"
#include "code/framework/utils.h"
#include <oslib/oslib.h>
#include "code/framework/fx/EffectsLib.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
//#include "code/game/pieces/impl/PieceJoker.h"
#include "code/framework/Properties.h"
#include "code/game/pieces/impl/PieceSimple.h"
#include "code/framework/anim/Anim.h"
#include "code/game/levels/utils/MCOContentSimple.h"
#include "code/framework/check/loaders.h"

using namespace utils;

#define LEVEL_TETRIS_DURATION_PART1 273650.0f
#define LEVEL_TETRIS_DURATION_PART2 403010.0f

#define LEVEL_TETRIS_COLOR_GAUGE RGBA(166,91,91,255)
#define LEVEL_TETRIS_COLOR_COMBO RGBA(41,38,36,200)
#define LEVEL_TETRIS_COLOR_COMBO_HOT1 RGBA(166,91,91,255)
#define LEVEL_TETRIS_COLOR_COMBO_HOT2 RGBA(41,38,36,255)
#define LEVEL_TETRIS_COLOR_COMBO_HOT3 RGBA(208,190,178,255)
#define LEVEL_TETRIS_COLOR_SONGBANNER RGBA(41,38,36,255)
#define LEVEL_TETRIS_COLOR_OSD RGBA(41,38,36,255)

string LevelTetris::getCode()
{
	return "tetris";
}

LevelTetris::LevelTetris(int difficulty, int nbPlayers):AbstractLevel(0, -1, difficulty, nbPlayers)
{	
	
	Properties *props=new Properties("res/levels/tetris/tetris.properties");
			
	m_songbannerA=new SongBanner(GAMEDEF_SONGBANNER_Y+3, "res/fonts/bmpFonts/arcadepix/14/", "arcadepix", "arcadepix", props->getStrValue("tetrisA.songbanner.text1"), props->getStrValue("tetrisA.songbanner.text2", ""), props->getStrValue("tetrisA.songbanner.text3", ""), props->getStrValue("tetrisA.songbanner.text4", ""));
	m_songbannerB=new SongBanner(GAMEDEF_SONGBANNER_Y+3, "res/fonts/bmpFonts/arcadepix/14/", "arcadepix", "arcadepix", props->getStrValue("tetrisB.songbanner.text1"), props->getStrValue("tetrisB.songbanner.text2", ""), props->getStrValue("tetrisB.songbanner.text3", ""), props->getStrValue("tetrisB.songbanner.text4", ""));
	m_songbannerPause=new SongBanner(GAMEDEF_SONGBANNER_Y+3, "res/fonts/bmpFonts/arcadepix/14/", "arcadepix", "arcadepix", props->getStrValue("pause.songbanner.text1"), props->getStrValue("pause.songbanner.text2", ""), props->getStrValue("pause.songbanner.text3", ""), props->getStrValue("pause.songbanner.text4", ""));
	
	m_comboDrawer=new ComboNumberDrawer(	LEVEL_TETRIS_COLOR_GAUGE, LEVEL_TETRIS_COLOR_COMBO, 
											LEVEL_TETRIS_COLOR_COMBO_HOT1, LEVEL_TETRIS_COLOR_COMBO_HOT2, LEVEL_TETRIS_COLOR_COMBO_HOT3,
											"res/fonts/bmpFonts/arcadepix/16/inout/", "in", "res/fonts/bmpFonts/arcadepix/16/inout/", "out"
										);
	
	m_scenario=new EventsScenario("res/levels/tetris/tetrisA.scenario");
	
	m_musicA=fwkLoadSoundFileMP3("res/levels/tetris/tetrisA.mp3", OSL_FMT_STREAM);
	m_musicB=fwkLoadSoundFileMP3("res/levels/tetris/tetrisB.mp3", OSL_FMT_STREAM);
	m_musicPause=fwkLoadSoundFileMP3("res/levels/tetris/tetrisPause.mp3", OSL_FMT_STREAM);
	oslSetSoundLoop(m_musicPause, true);
	
	
	m_addline=0;
	m_currentLevel=0;
	m_nbPiecesTotal=0;
	m_part=0;
	m_upspeed=-1;
	
	
	m_difficulty=difficulty;
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		m_nbColors=props->getIntValue("easy.colors");
		m_dropspeed_start=props->getIntValue("easy.dropspeed.start");
		m_levelup_nbBlocks=props->getIntValue("easy.levelup.nbBlocks");
		m_levelup_acceleration=props->getIntValue("easy.levelup.acceleration");
		m_linepattern=new LinePattern("res/levels/tetris/tetris_easy.pattern");
	}
	if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_nbColors=props->getIntValue("normal.colors");;
		m_dropspeed_start=props->getIntValue("normal.dropspeed.start");
		m_levelup_nbBlocks=props->getIntValue("normal.levelup.nbBlocks");
		m_levelup_acceleration=props->getIntValue("normal.levelup.acceleration");
		m_linepattern=new LinePattern("res/levels/tetris/tetris_normal.pattern");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		m_nbColors=props->getIntValue("hard.colors");;
		m_dropspeed_start=props->getIntValue("hard.dropspeed.start");
		m_levelup_nbBlocks=props->getIntValue("hard.levelup.nbBlocks");
		m_levelup_acceleration=props->getIntValue("hard.levelup.acceleration");
		m_linepattern=new LinePattern("res/levels/tetris/tetris_hard.pattern");
		m_dropspeed_themeB=props->getIntValue("hard.dropspeed.themeB");
		m_upspeed_themeB=props->getIntValue("hard.upspeed.themeB");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_VERYHARD)
	{
		//debug("difficulty is hard");
		m_nbColors=props->getIntValue("vhard.colors");;
		m_dropspeed_start=props->getIntValue("vhard.dropspeed.start");
		m_levelup_nbBlocks=props->getIntValue("vhard.levelup.nbBlocks");
		m_levelup_acceleration=props->getIntValue("vhard.levelup.acceleration");
		m_linepattern=new LinePattern("res/levels/tetris/tetris_vhard.pattern");
		m_dropspeed_themeB=props->getIntValue("vhard.dropspeed.themeB");
		m_upspeed_themeB=props->getIntValue("vhard.upspeed.themeB");
	}
	m_dropspeed=m_dropspeed_start;
	
	m_nbLines=13;
	m_nbCols=10;
	
	m_animBlue=new Anim(10, "blue", "res/levels/tetris/pieces/blue/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animRed=new Anim(21, "red", "res/levels/tetris/pieces/red/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animYellow=new Anim(10, "yellow", "res/levels/tetris/pieces/yellow/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animPurple=new Anim(10, "purple", "res/levels/tetris/pieces/purple/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animGreen=new Anim(10, "green", "res/levels/tetris/pieces/green/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animOrange=new Anim(10, "orange", "res/levels/tetris/pieces/orange/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_animBlue->setReferencePointCenter();
	m_animRed->setReferencePointCenter();
	m_animYellow->setReferencePointCenter();
	m_animPurple->setReferencePointCenter();
	m_animGreen->setReferencePointCenter();
	m_animOrange->setReferencePointCenter();
		
	m_fx_up3=new SerialLocalEffect(1);
	
	int up=props->getIntValue("fx.up3.time.up");
	int wait=props->getIntValue("fx.up3.time.wait");
	int y=props->getIntValue("fx.up3.pos.y");
	int down=props->getIntValue("fx.up3.time.down");
	
	WaitLocalEffect *waitfx=new WaitLocalEffect(wait);
	waitfx->setPosition(0, y);
	
	m_fx_up3->addEffect(new TranslationEffect(0, 0, 0, y, up, up, true, 1)); //up
	m_fx_up3->addEffect(waitfx); //wait
	
	m_fx_up3->addEffect(new TranslationEffect(0, 0, 0, y, up, up, true, 1)); //up
	m_fx_up3->addEffect(waitfx->clone()); //wait
	
	m_fx_up3->addEffect(new TranslationEffect(0, 0, 0, y, up, up, true, 1)); //up
	m_fx_up3->addEffect(waitfx->clone()); //wait
	
	m_fx_up3->addEffect(new TranslationEffect(0, y, 0, 0, down, down, true, 1)); //descend
	
	int gaugefxtime=props->getIntValue("fx.gauge.alpha.time");
	int gaugefxfrom=props->getIntValue("fx.gauge.alpha.from");
	int gaugefxto=props->getIntValue("fx.gauge.alpha.to");
	m_fx_gauge=new ProgressiveAlphaEffect(gaugefxtime,gaugefxtime,gaugefxfrom,gaugefxto);
	m_fx_gauge->start();
	
	
	m_nextline=m_linepattern->getNextLine();
	
	m_img_bg=fwkLoadImageFilePNG("res/levels/tetris/bg.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	
	m_fx_blink=new BlinkEffect(props->getIntValue("fx.blink.on"), props->getIntValue("fx.blink.off"), false, props->getIntValue("fx.blink.nb"));
	
	m_snd_tetris=fwkLoadSoundFileWAV("res/levels/tetris/tetris44hz16bmono.wav", OSL_FMT_NONE);
	m_snd_levelup=fwkLoadSoundFileWAV("res/levels/tetris/level44hz16bmono.wav", OSL_FMT_NONE);
	m_snd_levelup->volumeLeft=0x800;
	m_snd_levelup->volumeRight=0x800;
	m_delayer_levelup=NULL;
	m_levelup_delay=props->getIntValue("snd.levelup.delay");
	m_snd_pause=fwkLoadSoundFileWAV("res/levels/tetris/tetrispause44hz16bmono.wav", OSL_FMT_NONE);
	
	
	m_test=false;
	
	m_img_pause=fwkLoadImageFilePNG("res/levels/tetris/pause.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	
	m_img_songbannerbg=fwkLoadImageFilePNG("res/levels/tetris/songbannerbg.png", OSL_IN_RAM | OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_songbannerbg, 0, 272);
	
	m_fx_songbanner=new SerialLocalEffect(1);
	int sbfxappear=15;
	int sbfxwait=m_songbannerA->getTotalTimePresent();
	int sbfxdisappear=20;
	int sbfxsize=m_img_songbannerbg->sizeY;
	m_fx_songbanner->addEffect(new TranslationEffect(0, 0, 0, -sbfxsize, sbfxappear, sbfxappear, true, 1));
	WaitLocalEffect *fxwait=new WaitLocalEffect(sbfxwait);
	fxwait->setPosition(0, -sbfxsize);
	m_fx_songbanner->addEffect(fxwait);
	m_fx_songbanner->addEffect(new TranslationEffect(0, -sbfxsize, 0, 0, sbfxdisappear, sbfxdisappear, true, 1));
	
	
	m_fx_songbanner_pause=m_fx_songbanner->clone();
	
	
	m_score=0;
	m_combos=0;
	m_font=new BmpFont("res/fonts/bmpFonts/arcadepix/14/", "arcadepix", OSL_PF_8888);
	m_text_score=new BmpText("0", m_font);
	m_text_combos=new BmpText("0", m_font);
	m_text_level=new BmpText("0", m_font);
	
	delete props;
}

void LevelTetris::reset()
{
	m_songbannerA->reset();
	m_songbannerB->reset();
	m_comboDrawer->reset();
	m_scenario->reset();
	
	oslStopSound(m_musicA);
	oslStopSound(m_musicB);
	
	m_addline=0;
	m_currentLevel=0;
	m_nbPiecesTotal=0;
	m_part=0;
	m_upspeed=-1;
	
	m_linepattern->reset();
	m_nextline=m_linepattern->getNextLine();
	
	m_dropspeed=m_dropspeed_start;
	
	m_fx_up3->stop();
	m_fx_up3->reset();
	m_fx_blink->stop();
	m_fx_blink->reset();
	m_fx_songbanner->stop();
	m_fx_songbanner->reset();
	m_fx_songbanner_pause->stop();
	m_fx_songbanner_pause->reset();
	
	m_score=0;
	m_combos=0;
}

LevelTetris::~LevelTetris()
{	
	delete m_linepattern;
	
	delete m_songbannerA;
	delete m_songbannerB;	
	delete m_songbannerPause;
	
	delete m_comboDrawer;
	
	delete m_scenario;
	
	delete m_fx_gauge;
	
	oslDeleteSound(m_musicA);
	oslDeleteSound(m_musicB);
	oslDeleteSound(m_musicPause);
	
	//debug("delete anims...");
	delete m_animBlue;
	delete m_animRed;
	delete m_animYellow;
	delete m_animPurple;
	delete m_animGreen;
	delete m_animOrange;
	
	delete m_fx_up3;
	
	oslDeleteImage(m_img_bg);
	
	delete m_fx_blink;
	
	fwkDeleteSound(m_snd_tetris);
	fwkDeleteSound(m_snd_levelup);
	fwkDeleteSound(m_snd_pause);
	
	if(m_delayer_levelup!=NULL)
		delete m_delayer_levelup;
		
	oslDeleteImage(m_img_pause);
	oslDeleteImage(m_img_songbannerbg);
	
	delete m_fx_songbanner;
	delete m_fx_songbanner_pause;
	
	delete m_text_score;
	delete m_text_combos;
	delete m_text_level;
	delete m_font;
	
}


//dessin avec une paire
void LevelTetris::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc, scorer);
	if(!isPaused())
	{
		//currentPair->draw(grid, fx_trans, APPLY_FX_ON_NONE);
		Pos gridPos=currentPair->getGridPos(0);
		Pos pixPos=grid->getContentPosition(gridPos.x, gridPos.y);
		drawPairXY(currentPair, pixPos.x, pixPos.y, grid, fx_trans, m_fx_songbanner->isRunning()?m_fx_songbanner:NULL);
	}
	drawAfter(grid, nextPair, cc, scorer);
}

//dessin avec 2 pieces
void LevelTetris::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc, scorer);
	
	Pos pixPosConnected=grid->getContentPosition(pieceConnectedPos.x, pieceConnectedPos.y);
	Pos pixPosUnconnected=grid->getContentPosition(pieceUnconnectedPos.x, pieceUnconnectedPos.y);
	
	//pieceConnected->drawXY(pixPosConnected->x, pixPosConnected->y);
	//pieceUnconnected->drawXY(pixPosUnconnected->x, pixPosUnconnected->y, fx_trans);
	drawPieceXY(pieceConnected, pixPosConnected.x, pixPosConnected.y);
	drawPieceXY(pieceUnconnected, pixPosUnconnected.x, pixPosUnconnected.y, fx_trans);
	
	drawAfter(grid, nextPair, cc, scorer);
}

//dessin avec rien
void LevelTetris::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc, scorer);
	drawAfter(grid, nextPair, cc, scorer);
}

void LevelTetris::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	oslDrawImageXY(m_img_bg, 0, 0);
	
	//oslDrawImageXY(m_gridImage, grid->getPosX(), grid->getPosY());
		
	if(isPaused())
	{
		//oslDrawString(220, 130, "pause");
		oslDrawImageXY(m_img_pause, 135, 0);
	}
	else if(!m_fx_blink->isRunning())
	{
		//dessin du contenu de la grille
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco->getPiece()!=NULL)
				{
					Pos pos=grid->getContentPosition(i, j);
					AbstractPiece *p=mco->getPiece();
					
					if(p->getType()==PIECE_TYPE_NORMAL)
					{
						if(mco->getMCOContent()!=NULL)
						{
							mco->getMCOContent()->drawXY(pos.x,pos.y, m_fx_up3->isRunning()?m_fx_up3:NULL, m_fx_songbanner->isRunning()?m_fx_songbanner:NULL);
						}
					}
					else
					{
						p->drawXY(pos.x,pos.y, m_fx_up3->isRunning()?m_fx_up3:NULL, m_fx_songbanner->isRunning()?m_fx_songbanner:NULL);
					}
				}
			}	
		}
		
		//dessin de la ligne sous la grille qd fx_up3 est en route!
		if(m_fx_up3->isRunning())
		{
			Pos pos=grid->getContentPosition(0, m_nbLines-1);
			int x=pos.x;
			int y=pos.y+GAMEDEF_PIECE_HEIGHT;
			
			list<AbstractPiece *>::iterator it;
			for (it=m_nextline->begin(); it!=m_nextline->end();it++)
			{
				AbstractPiece *p=(*it);
				if(p!=NULL) //ne devrait pas etre null...
				{
					//p->drawXY(x, y, m_fx_up3);
					drawPieceXY(p, x, y);
				}
				x+=GAMEDEF_PIECE_WIDTH;
			}
		}
	}
	else //m_fx_blink->isRunning()
	{
		//enableDebug();
		//debug("dessin avec blink running");
		int i,j;
		//on parcours par ligne
		for(j=grid->getNbLines()-1;j>=0;j--)
		{
			bool empty=false;
			//parcours de la ligne pour vérifier si une case est vide
			for(i=0;i<grid->getNbCols();i++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco==NULL)
					fatalError("mco est null!!");
				if(mco->isEmpty())
				{
					empty=true;
				}
			}
			
			
			for(i=0;i<grid->getNbCols();i++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				if(mco==NULL)
					fatalError("mco est null!!");
				Pos pos=grid->getContentPosition(i, j);
				//on dessine la ligne, avec l'effet blink si elle est pleine, sinon normalement
				//mco->getMCOContent()->drawXY(pos->x,pos->y, empty?m_fx_blink:NULL); 
				AbstractPiece *p=mco->getPiece();
				if(p!=NULL)
				{
					drawPieceXY(p, pos.x, pos.y, empty?NULL:m_fx_blink);
				}
			}
		}
		//debug("fin dessin avec blink running");
	}	
	
	
}

void LevelTetris::update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	m_songbannerPause->update();
	m_fx_songbanner_pause->update();
}

void LevelTetris::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{	
	
	drawPairXY(nextPair, 414, 229, grid);	
	
	m_fx_gauge->activate();
	oslDrawFillRect(80, 272-int((cc->getTime()*272)/cc->getTimeTotalAvailable()), 95, 272, LEVEL_TETRIS_COLOR_GAUGE);
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	if(m_fx_songbanner->isRunning() && !isPaused())
	{
		fx::draw(m_img_songbannerbg, m_fx_songbanner);
		oslSetAlpha(OSL_FX_TINT, LEVEL_TETRIS_COLOR_SONGBANNER);
		if(m_part==0)
			m_songbannerA->draw();
		else
			m_songbannerB->draw();
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	
	if(m_fx_songbanner_pause->isRunning() && isPaused())
	{
		fx::draw(m_img_songbannerbg, m_fx_songbanner_pause);
		oslSetAlpha(OSL_FX_TINT, LEVEL_TETRIS_COLOR_SONGBANNER);
			m_songbannerPause->draw();
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
		
	if(!isPaused())
		m_comboDrawer->drawXY(170, 180);
	//oslDrawString(10, 190, (toString(m_scenario->getTime())).c_str());
	//TODO: dessiner un coeur pour les levels >10
	//oslDrawString(400, 100, ("level "+toString(m_currentLevel)).c_str());
	
	
	oslSetAlpha(OSL_FX_TINT, LEVEL_TETRIS_COLOR_OSD);
	//les textes osd (score...)
	if(m_score!=scorer->getScoreCurrent())
	{
		m_score=scorer->getScoreCurrent();
		m_text_score->setText(toString(m_score));
	}
	
	m_text_level->setText(toString(m_currentLevel));
	
	if(m_combos!=scorer->getMaxcombosCurrent())
	{
		m_combos=scorer->getMaxcombosCurrent();
		m_text_combos->setText(toString(m_combos));
	}
	
	m_text_score->drawXY(460-m_text_score->getWidth(),47);
	m_text_level->drawXY(450-m_text_level->getWidth(),105);
	m_text_combos->drawXY(450-m_text_combos->getWidth(),158);
	
	oslSetAlpha(OSL_FX_DEFAULT, 0);
}

void LevelTetris::start_specific()
{
	oslPlaySound(m_musicA, 0);
	m_scenario->restart();
	oslPlaySound(m_musicPause, 5);
	oslPauseSound(m_musicPause, 1);
	m_fx_songbanner->start();
	m_fx_songbanner_pause->start();
}

//appelé uniquement si le level n'est pas en pause.
void LevelTetris::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	//cas special tetris: on utilise des anims séparées
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			AbstractPiece *p=mco->getPiece();
			if(p!=NULL)
			{
				
				//on cree si une piece n'en a pas
				if(mco->getMCOContent()==NULL)
				{
					Anim *anim=animFromColor(p->getColor());
					mco->setMCOContent(new MCOContentSimple(new Anim(*anim)));
					
				}				
				//on démarre si la piece explose
				if(mco->isColorExploding() || mco->isShapeExploding() || mco->isSpecialExploding())
					mco->getMCOContent()->start();
			}
			else//mco->getPiece()==NULL
			{
				if(mco->getMCOContent()!=NULL)
				{
					mco->deleteMCOContent();
				}
			}
			
			if(mco->getMCOContent()!=NULL) //peu importe le cas, on update
			{
				mco->getMCOContent()->update();
			}
		}
	}
	
	
	if(m_part==0)
		m_songbannerA->update();
	else
		m_songbannerB->update();
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	m_scenario->update();
	m_fx_up3->update();
	m_fx_blink->update();
	m_fx_gauge->update();
	m_fx_songbanner->update();
	
	if(m_delayer_levelup!=NULL)
		m_delayer_levelup->update();
	
	//faut changer de zik
	if(m_test || (m_part==0 && m_scenario->getTime()>=LEVEL_TETRIS_DURATION_PART1 && (m_difficulty==GAMEDEF_DIFFICULTY_HARD || m_difficulty==GAMEDEF_DIFFICULTY_VERYHARD)))
	{
		if(!m_fx_blink->finished() && !m_fx_blink->isRunning() && m_difficulty==GAMEDEF_DIFFICULTY_HARD)
		{
			m_fx_blink->restart();
			m_dropspeed=-1; //on arrete la descente
			oslPlaySound(m_snd_tetris, 0);
		}
		else if(m_difficulty==GAMEDEF_DIFFICULTY_VERYHARD || (m_fx_blink->finished() && !m_fx_blink->isRunning()))
		{
			m_test=false;
			m_part=1;
			//oslStopSound(m_musicA);
			oslPlaySound(m_musicB, 0);
			m_dropspeed=m_dropspeed_themeB;
			m_currentLevel=19;
			m_fx_songbanner->restart();
			
			//faut enlever les pieces des lignes pleines	
			if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
			{		
				for(int j=grid->getNbLines()-1;j>=0;j--)
				{
					bool empty=false;
					//parcours de la ligne pour vérifier si une case est vide
					for(int i=0;i<grid->getNbCols();i++)
					{
						MatrixContentObject *mco=grid->getAt(i,j);
						if(mco->getPiece()==NULL)
						{
							empty=true;
						}
					}
					
					if(!empty) //ligne pleine, on enleve les pieces
					{
						for(int i=0;i<grid->getNbCols();i++)
						{
							grid->immediatePieceDestruction(i,j);
						}
					}
				}
			}
		}
	}
	
	//evenements du scenario
	if(m_scenario->hasEvent(1)) //symbales seches (+1 ligne)
	{
		m_addline++;
		m_fx_up3->stop();
	}
	if(m_scenario->hasEvent(2))
	{
		m_fx_up3->restart();
	}
	if(m_scenario->hasEvent(3)) //symbales seches fin de groupe, + fortes (ligne +2?)
	{
		/*if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
			m_addline++;
		else
			m_addline+=2;*/
		m_addline++;
		m_fx_up3->stop();
	}
	
	if(m_delayer_levelup!=NULL && m_delayer_levelup->get())
	{
		oslPlaySound(m_snd_levelup, 3);
	}
	
	
	if(m_fx_songbanner->isRunning())
	{
		nextPair->goTo(int(m_nbCols/2)-1, 1);
	}
	else
	{
		nextPair->goTo(int(m_nbCols/2)-1, 0);
	}
	
	
}

void LevelTetris::pause_specific() 
{
	if(m_part==0)
	{
		oslPauseSound(m_musicA, 1);
	}
	else
	{
		oslPauseSound(m_musicB, 1);
	}
	m_scenario->pause();
	oslPauseSound(m_musicPause, 0);
	oslPlaySound(m_snd_pause, 3);
}

void LevelTetris::resume_specific()
{
	if(m_part==0)
	{
		oslPauseSound(m_musicA, 0);
	}
	else
	{
		oslPauseSound(m_musicB, 0);
	}
	m_scenario->start();
	oslPauseSound(m_musicPause, 1);
}
		

list<AbstractPiece *> *LevelTetris::getNextLine()
{
	m_nbPiecesTotal+=10; //une ligne fait 10 de large
	recalculateSpeeds();
	list<AbstractPiece *> *ret=m_nextline;
	m_nextline=m_linepattern->getNextLine();
	return ret;
}

bool LevelTetris::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int LevelTetris::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return m_dropspeed;
}

int LevelTetris::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	if(m_part==0)
		return m_upspeed;
	//else
	return m_upspeed_themeB;
}

void LevelTetris::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(key==OSL_KEY_TRIANGLE)
	{
				
	}
	else if(key==OSL_KEY_SQUARE)
	{
		//m_test=true;
		//m_fx_songbanner->restart();
	}
}

bool LevelTetris::finished()
{
	if	(
			(m_difficulty==GAMEDEF_DIFFICULTY_EASY && m_scenario->getTime()>=LEVEL_TETRIS_DURATION_PART1)
			||
			(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL && m_scenario->getTime()>=LEVEL_TETRIS_DURATION_PART1)
			||
			(m_difficulty==GAMEDEF_DIFFICULTY_HARD && m_scenario->getTime()>=LEVEL_TETRIS_DURATION_PART2)
			||
			(m_difficulty==GAMEDEF_DIFFICULTY_VERYHARD && m_scenario->getTime()>=LEVEL_TETRIS_DURATION_PART2)
		)
	{
		return true;
	}
	return false;
}

float LevelTetris::getDuration()
{
	if	(m_difficulty==GAMEDEF_DIFFICULTY_EASY || m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
		return LEVEL_TETRIS_DURATION_PART1;
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD || m_difficulty==GAMEDEF_DIFFICULTY_VERYHARD)
		return LEVEL_TETRIS_DURATION_PART2;
	
	return -1;
}

Pair *LevelTetris::getNewPair()
{
	m_nbPiecesTotal+=2;
	recalculateSpeeds();
	
	Pair *p=new Pair(randomPiece(), randomPiece());
	int posX=int(m_nbCols/2)-1;
	int posY;
	if(m_fx_songbanner->isRunning())
	{
		posY=1;
	}
	else
	{
		posY=0;
	}
	
	
	
	p->goTo(posX, posY);
	
	return p;
}

//recalcule les dropspeed, level
void LevelTetris::recalculateSpeeds()
{
	if(m_part==0) //1ere partie du level
	{
		int oldlevel=m_currentLevel;
		m_currentLevel=int(m_nbPiecesTotal/m_levelup_nbBlocks);
		if(m_currentLevel>19)
			m_currentLevel=19;
		m_dropspeed=m_dropspeed_start-(m_currentLevel*m_levelup_acceleration);
		
		//on limite la vitesse de descente quand meme!
		if(m_dropspeed<m_dropspeed_themeB)
			m_dropspeed=m_dropspeed_themeB;
		
		if(oldlevel!=m_currentLevel)
		{
			//oslPlaySound(m_snd_levelup, 3);
			if(m_delayer_levelup!=NULL)
				delete m_delayer_levelup;
			m_delayer_levelup=new Delayer(m_levelup_delay, 1);
		}
	}
	//else on touche pas ca va a fond de toute facon
}

AbstractPiece *LevelTetris::randomPiece()
{
	//suivant la difficulté, il y a + ou - de couleurs à l'écran
	int color=random(0,m_nbColors-1);
	PieceSimple *p=NULL;
	switch(color)
	{
		case 0:
		{
			p=new PieceSimple(PIECE_SHAPE_CIRCLE, PIECE_COLOR_BLUE);
			break;
		}
		case 1:
		{
			p=new PieceSimple(PIECE_SHAPE_SQUARE, PIECE_COLOR_RED);
			break;
		}
		case 2:
		{
			p=new PieceSimple(PIECE_SHAPE_DIAMOND, PIECE_COLOR_YELLOW);	
			break;
		}
		case 3:
		{
			p=new PieceSimple(PIECE_SHAPE_STAR, PIECE_COLOR_PURPLE);	
			break;
		}
		case 4:
		{
			p=new PieceSimple(PIECE_SHAPE_CROSS, PIECE_COLOR_GREEN);
			break;
		}
		case 5:
		{
			p=new PieceSimple(PIECE_SHAPE_TRIANGLE, PIECE_COLOR_ORANGE);
			break;
		}
		default:
		{
			fatalError(string("random generated "+toString(color)).c_str());
		}
	}
	
	
	
	return p;
}

void LevelTetris::drawPairXY(Pair *pair, int x, int y, GameGrid *grid, AbstractLocalEffect *fx_transl, AbstractLocalEffect *fx1)
{
	//suivant l'orientation, la 2eme piece est a un autre endroit
	int x2=x, y2=y;
	switch(pair->getOrientation())
	{
		case ORIENTATION_EAST:
			x2=x+grid->getCellWidth();
			break;	
		
		case ORIENTATION_NORTH:
			y2=y-grid->getCellHeight();
			break;
			
		case ORIENTATION_SOUTH:
			y2=y+grid->getCellHeight();
			break;
			
		case ORIENTATION_WEST:
			x2=x-grid->getCellWidth();
			break;
	}
	
	//pair->getPiece(1)->drawXY(x2, y2, fx_transl);
	//pair->getPiece(0)->drawXY(x, y, fx_transl);
	
	fx::drawXY(animFromColor(pair->getPiece(1)->getColor())->getImage(), x2, y2, fx_transl, fx1);
	fx::drawXY(animFromColor(pair->getPiece(0)->getColor())->getImage(), x, y, fx_transl, fx1);
}

void LevelTetris::drawPieceXY(AbstractPiece *piece, int x, int y, AbstractLocalEffect *fx_transl)
{
	fx::drawXY(animFromColor(piece->getColor())->getImage(), x, y, fx_transl, m_fx_up3->isRunning()?m_fx_up3:NULL, m_fx_songbanner->isRunning()?m_fx_songbanner:NULL);
}

Anim *LevelTetris::animFromColor(int color)
{
	switch(color)
	{
		case PIECE_COLOR_BLUE:
		{
			return m_animBlue;
			break;
		}
		case PIECE_COLOR_RED:
		{
			return m_animRed;
			break;
		}
		case PIECE_COLOR_YELLOW:
		{
			return m_animYellow;
			break;
		}
		case PIECE_COLOR_PURPLE:
		{
			return m_animPurple;
			break;
		}
		case PIECE_COLOR_GREEN:
		{
			return m_animGreen;
			break;
		}
		case PIECE_COLOR_ORANGE:
		{
			return m_animOrange;
			break;
		}
	}
	
	return NULL;
}



