#ifndef LEVELTURNANDBURN_H_
#define LEVELTURNANDBURN_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/framework/time/Delayer.h"
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/ComboNumberDrawer.h"
#include "code/game/EventsScenario.h"
#include "code/framework/particles/complete/impl/ParticleGeneratorPoint.h"
#include "code/framework/fx/EffectsLib.h"


using namespace std;

class LevelTurnAndBurn : public AbstractLevel
{
	private:
			
		OSL_IMAGE *m_gridImage;
		LinePattern *m_linepattern;
			
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); 
		void resume_specific();
				
		SongBanner *m_songbanner;
		
		ComboNumberDrawer * m_comboDrawer;
		
		EventsScenario *m_scenario;
		OSL_SOUND* m_music;
		
		int m_dropspeed;
		
		int m_addline; //ajoute 1 qd on veut qu'une ligne monte
		
		int m_round;
		int m_difficulty;
		
		int m_dropspeedFast;
		int m_dropspeedNormal;
		int m_dropspeedSlow;
		int m_burnDelay;
		
		Delayer *m_delayer;
		
		//2effets de rotation ta-taaa
		RotationEffect *m_fx_rotation1;
		SerialLocalEffect *m_fx_rotation2;
		
		bool m_burning;
		
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		int getRandomColor(); //rouge, orange ou jaune
		
		ParticleGeneratorPoint *m_generator_model;
		
	public:
		LevelTurnAndBurn(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~LevelTurnAndBurn();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		//LinePattern *getLevelPieces(); 
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		
		bool finished();
		
		list<AbstractPiece *> *getNextLine();
		
		float getDuration();
};

#endif /*LEVELTURNANDBURN_H_*/
