#include "code/game/levels/impl/underwater/LevelUnderwater.h"
#include "code/framework/Properties.h"
#include <string>
#include "code/game/gamedef.h"
#include "code/game/levels/utils/DJMCOContentSimple.h"
#include "code/game/grid/AbstractMCOContent.h"
#include "code/game/Pair.h"
#include "code/framework/utils.h"
#include "code/game/levels/utils/MCOContentSimple.h"
#include "code/framework/particles/complete/animFactories/RandomAnimationFactory.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomShake.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomGravity.h"
#include "code/framework/particles/complete/fxFactories/PFXFRandomTranslation.h"
#include "code/game/levels/utils/MCOContentCombo.h"
#include "code/framework/particles/complete/fx/ParticlesProgressiveTintEffect.h"
#include "code/framework/check/loaders.h"
#include "code/framework/check/PrettyFileLogger.h"
#include "code/game/levels/impl/underwater/CondemnedParticleGeneratorPoint.h"
#include "code/game/levels/utils/DJMCOContentLayered.h"
#include "code/game/levels/utils/MCOContentFlagged.h"


using namespace utils;
using namespace std;


#define LEVEL_UNDERWATER_PART_PIRATES 1
#define LEVEL_UNDERWATER_PART_SMOKER 2

#define LEVEL_UNDERWATER_DURATION_PIRATES 165740 
#define LEVEL_UNDERWATER_DURATION_SMOKER 140000

#define LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_BUBBLE 0
#define LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_SQUARE 2
#define LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_TRIANGLE 1


string LevelUnderwater::getCode()
{
	return "underwater";
}

LevelUnderwater::LevelUnderwater(int difficulty, int nbPlayers)
:AbstractLevel(0, -1, difficulty, nbPlayers)
{
	PFL::logConstructionStart("LevelUnderwater");	
	debug("level construction...");
	Properties *props=new Properties("res/levels/underwater/underwater.properties");
	
	m_music_pirates=fwkLoadSoundFileMP3("res/levels/underwater/space_pirates.mp3", OSL_FMT_STREAM);
	m_music_smoker=fwkLoadSoundFileMP3("res/levels/underwater/black_smoker.mp3", OSL_FMT_STREAM);
	
	string nbcols;
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		nbcols=props->getStrValue("easy.nbcols");
		
		m_linepattern=new LinePattern("res/levels/underwater/underwater_easy.pattern", 1);
		m_dropspeed_normal=props->getIntValue("easy.dropspeed");
		m_water_upspeed=props->getIntValue("easy.water.upspeed");
		m_waterlevel=props->getIntValue("easy.waterlevel.start"); //niveau actuel, si == 12 c'est perdu
		m_addline=props->getIntValue("easy.lines.start");
		m_nbCols=props->getIntValue("easy.nbcols");
		m_water_upspeed_smoker=props->getIntValue("easy.water.upspeed.smoker");
		
		m_water_move_busy_pirates=props->getBoolValue("easy.water.move.whenbusy.pirates");
		m_water_move_busy_smoker=props->getBoolValue("easy.water.move.whenbusy.smoker");
		
		m_water_min=props->getIntValue("easy.water.min");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		nbcols=props->getStrValue("normal.nbcols");
		
		m_linepattern=new LinePattern("res/levels/underwater/underwater_normal.pattern", 1);
		m_dropspeed_normal=props->getIntValue("normal.dropspeed");
		m_water_upspeed=props->getIntValue("normal.water.upspeed");
		//water level
		m_waterlevel=props->getIntValue("normal.waterlevel.start"); //niveau actuel, si == 12 c'est perdu
		m_addline=props->getIntValue("normal.lines.start");
		m_nbCols=props->getIntValue("normal.nbcols");
		m_water_upspeed_smoker=props->getIntValue("normal.water.upspeed.smoker");
		
		m_water_move_busy_pirates=props->getBoolValue("normal.water.move.whenbusy.pirates");
		m_water_move_busy_smoker=props->getBoolValue("normal.water.move.whenbusy.smoker");
		
		m_water_min=props->getIntValue("normal.water.min");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		nbcols=props->getStrValue("hard.nbcols");
		
		m_linepattern=new LinePattern("res/levels/underwater/underwater_hard.pattern", 1);	
		m_dropspeed_normal=props->getIntValue("hard.dropspeed");	
		m_water_upspeed=props->getIntValue("hard.water.upspeed");
		//water level
		m_waterlevel=props->getIntValue("hard.waterlevel.start"); //niveau actuel, si == 12 c'est perdu
		m_addline=props->getIntValue("hard.lines.start");
		m_nbCols=props->getIntValue("hard.nbcols");
		m_water_upspeed_smoker=props->getIntValue("hard.water.upspeed.smoker");
		
		m_water_move_busy_pirates=props->getBoolValue("hard.water.move.whenbusy.pirates");
		m_water_move_busy_smoker=props->getBoolValue("hard.water.move.whenbusy.smoker");
		
		m_water_min=props->getIntValue("hard.water.min");
	}
	else
	{
		fatalError("no such difficulty in level underwater");
	}
	
	m_anim_grid_under=new Anim(
		props->getIntValue("grid.underwater."+nbcols+".nbimages"), 
		"grid_under_", "res/levels/underwater/grid/"+nbcols+"/", 
		props->getIntValue("grid.underwater."+nbcols+".time"), 
		OSL_PF_8888
	);
	m_anim_grid_under->setX(props->getIntValue("grid.underwater."+nbcols+".pos.x"));
	m_anim_grid_under->setY(props->getIntValue("grid.underwater."+nbcols+".pos.y"));
	m_img_grid_over=fwkLoadImageFilePNG("res/levels/underwater/grid/"+nbcols+"/grid_over.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_grid_over, props->getIntValue("grid."+nbcols+".pos.x"), props->getIntValue("grid."+nbcols+".pos.y"));
		
	m_anim_grid_under->start();
	
	m_dropspeed=-1;
	
	sceDisplayWaitVblankStart();
	
	m_songbanner_pirates=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", 
								props->getStrValue("songbanner.pirates.text1"), props->getStrValue("songbanner.pirates.text2", ""), 
								props->getStrValue("songbanner.pirates.text3", ""), props->getStrValue("songbanner.pirates.text4", ""));
	m_songbanner_smoker=new SongBanner(GAMEDEF_SONGBANNER_Y, "res/fonts/bmpFonts/russelsquare/14/", "normal", "italic", 
								props->getStrValue("songbanner.smoker.text1"), props->getStrValue("songbanner.smoker.text2", ""), 
								props->getStrValue("songbanner.smoker.text3", ""), props->getStrValue("songbanner.smoker.text4", ""));
								
	m_comboDrawer=new ComboNumberDrawer(
		props->getColorValue("combo.color.in.normal"), 
		props->getColorValue("combo.color.out"), 
		props->getColorValue("combo.color.in.hot1"), 
		props->getColorValue("combo.color.in.hot2"), 
		props->getColorValue("combo.color.in.hot3"),
		"res/fonts/bmpFonts/aachen/16/inout/", "in", 
		"res/fonts/bmpFonts/aachen/16/inout/", "out"
	);
	m_scenario_pirates=new EventsScenario("res/levels/underwater/pirates.scenario");
	m_scenario_smoker=new EventsScenario("res/levels/underwater/smoker.scenario");
	
	m_part=LEVEL_UNDERWATER_PART_PIRATES;
	//m_addline=0;
	
	m_upspeed=-1;
	
	
	int waterfxtime=props->getIntValue("water.fx.translation.speed");
	m_fx_water_up=new TranslationEffect(0, 0, 0, -21, waterfxtime, waterfxtime, true, 1);
	m_fx_water_down=new TranslationEffect(0, 0, 0, 21, waterfxtime, waterfxtime, true, 1);
	m_counter=0;
	
	//m_img_waves=fwkLoadImageFilePNG("res/levels/underwater/waves.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	
	m_snd_water_up=fwkLoadSoundFileWAV("res/levels/underwater/water_up.wav", OSL_FMT_NONE);
	m_snd_water_down=fwkLoadSoundFileWAV("res/levels/underwater/water_down.wav", OSL_FMT_NONE);
	//m_snd_combo=fwkLoadSoundFileBGM("res/levels/underwater/combo2.bgm", OSL_FMT_NONE);
	m_snd_combo=fwkLoadSoundFileWAV("res/levels/underwater/combo2.wav", OSL_FMT_NONE);
	//m_snd_explode=fwkLoadSoundFileWAV("res/levels/underwater/explode2.wav", OSL_FMT_NONE);
	//m_snd_explode=fwkLoadSoundFileBGM("res/levels/underwater/explode.bgm", OSL_FMT_NONE);
	m_snd_explode=fwkLoadSoundFileWAV("res/levels/underwater/explode2.wav", OSL_FMT_NONE);
	oslSetSoundVolume(m_snd_water_up, props->getIntValue("sound.water.up.volume"));
	oslSetSoundVolume(m_snd_water_down, props->getIntValue("sound.water.down.volume"));
	oslSetSoundVolume(m_snd_combo, props->getIntValue("sound.combo.volume"));
	oslSetSoundVolume(m_snd_explode, props->getIntValue("sound.explode.volume"));
	
	m_fxmgr_waves=new LocalEffectManager();
	int wavesize=props->getIntValue("fx.waves.size");
	int wavetimenextline=props->getIntValue("fx.waves.time.nextline");
	int wavetimeshake=props->getIntValue("fx.waves.time.shake");
	
	int wavewaitfirst=0;
	for(int i=0;i<m_nbCols;i++)
	{
		SerialLocalEffect *fx=new SerialLocalEffect(1);
		
		if(wavewaitfirst!=0)
		{
			fx->addEffect(new WaitLocalEffect(wavewaitfirst));
		}
		wavewaitfirst+=wavetimenextline;
		
		fx->addEffect(new ShakeEffect(wavesize, 0, wavetimeshake));
		
		m_fxmgr_waves->addEffect(fx);
	}
	
	//filtre sous l'eau
	m_img_underwater_filter=oslCreateImage(480,272,OSL_IN_RAM,OSL_PF_8888);
	oslClearImage(m_img_underwater_filter, RGBA(255,255,255,255));
	oslSwizzleImage(m_img_underwater_filter);
	int filtertime=props->getIntValue("fx.water.filter.time");
	m_fx_underwater_filter=new ProgressiveTintEffect(
		filtertime, filtertime, 
		props->getColorValue("fx.water.filter.color.from"),
		props->getColorValue("fx.water.filter.color.to"),
		1
	);
	
	m_piece_special_explosion_time=props->getFloatValue("piece.special.explosion.time");
	
	sceDisplayWaitVblankStart();
	initPieceAnims(props);
	sceDisplayWaitVblankStart();
	//m_color_pieces_underwater=props->getColorValue("pieces.underwater.color");
	m_fx_color_pieces_underwater=new ProgressiveTintEffect(
		filtertime, filtertime, 
		props->getColorValue("pieces.underwater.color.from"), 
		props->getColorValue("pieces.underwater.color.to"), 1
	);
	
	//les generateurs de bulles
	m_partgen_bubbles=new list<CondemnedParticleGeneratorPoint *>();
	initPartgenCombos(props);
	
	for(int i=0;i<2;i++)
	{
		RandomAnimationFactory *animf=new RandomAnimationFactory();
		for(int j=1; j<=props->getIntValue("generator.frontback.nbkinds");j++)
		{
			animf->addAnimation(new Anim("res/levels/underwater/bubbles/front-back/bubble"+toString(j)+".png"));
		}
		m_partgen_bubbles_frontback[i]=new ParticleGeneratorSegment(
			240, 290, 90, 480,
			props->getIntValue("generator.frontback.throughput.particles"),
			props->getIntValue("generator.frontback.throughput.frames"),
			1,
			1,
			props->getIntValue("generator.frontback.lifespan"),
			animf, 
			-1
		);
		m_partgen_bubbles_frontback[i]->setLocalEffect1SimpleTranslation();
		
		PFXFRandomShake *randomshake=new PFXFRandomShake();
		randomshake->setWidthMin(props->getIntValue("generator.frontback.fx.shake.width.min"));
		randomshake->setWidthMax(props->getIntValue("generator.frontback.fx.shake.width.max"));
		randomshake->setTimeMin(props->getIntValue("generator.frontback.fx.shake.time.min"));
		randomshake->setTimeMax(props->getIntValue("generator.frontback.fx.shake.time.max"));
		m_partgen_bubbles_frontback[i]->setLocalEffect2(randomshake);
		
		PFXFRandomGravity *randomgrav=new PFXFRandomGravity();
		randomgrav->setSpeedYMin(props->getFloatValue("generator.frontback.fx.gravity.speed.min"));
		randomgrav->setSpeedYMax(props->getFloatValue("generator.frontback.fx.gravity.speed.max"));
		randomgrav->setAccelYMin(props->getFloatValue("generator.frontback.fx.gravity.acceleration.min"));
		randomgrav->setAccelYMax(props->getFloatValue("generator.frontback.fx.gravity.acceleration.max"));
		m_partgen_bubbles_frontback[i]->setLocalEffect3(randomgrav);
		
		
		m_partgen_bubbles_frontback[i]->start();
	}
	
	m_nb_specialexplosions=0;
	
	sceDisplayWaitVblankStart();
	initGaugeGenerators(props);
	sceDisplayWaitVblankStart();
	
	m_triangle_user=false;
	m_square_user=false;
	
	m_color_sky_top=props->getColorValue("sky.color.top");
	m_color_sky_bottom=props->getColorValue("sky.color.bottom");
	
	m_watery=calculateWaterPixPosY();
	
	m_anim_underwater_bg=new Anim(
							props->getIntValue("bg.underwater.nbImages"), 
							"underwaterbg_", 
							"res/levels/underwater/underwaterbg/", 
							props->getFloatValue("bg.underwater.delay")
						); 
	m_anim_underwater_bg->start();
	
	sceDisplayWaitVblankStart();
	m_animals=new AnimalManager(props);
	sceDisplayWaitVblankStart();
	
	m_anim_waves=new Anim(
		props->getIntValue("anim.waves.nbImages"), 
		"under", "res/levels/underwater/waves/", 
		props->getFloatValue("anim.waves.delay")
	);
	m_anim_waves->setY(props->getIntValue("anim.waves.y"));
	m_anim_waves->start();
		
	//next pair
	m_pos_nextpair=Pos(props->getIntValue("pair.next."+nbcols+".x"), props->getIntValue("pair.next.y.waterrelative"));
	m_img_buoy=fwkLoadImageFilePNG("res/levels/underwater/buoy.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetPosition(m_img_buoy, props->getIntValue("pair.next.buoy."+nbcols+".x"), props->getIntValue("pair.next.buoy.y.waterrelative"));
	m_fx_nextpair_shake=new ShakeEffect(
		props->getIntValue("pair.next.fx.shake.width"), 
		props->getIntValue("pair.next.fx.shake.height"), 
		props->getIntValue("pair.next.fx.shake.time")
	);
	m_fx_nextpair_shake->start();
	
	m_nextpair_size_coeff=props->getFloatValue("pair.next.size.coeff");
	int growpercent=int(m_nextpair_size_coeff*100.0f)-100;
	//enableDebug();
	//debug(toString(m_nextpair_size_coeff));
	//debug(toString(growpercent));
	//enableDebug(false);
	m_fx_nextpair_resize=new SizeWowEffect(
		0, growpercent,
		0, growpercent, 
		1, 1, false
	);
	
	//pause
	int pause_height=props->getIntValue("fx.pause.height");
	int pause_time_min=props->getIntValue("fx.pause.time.min");
	int pause_time_max=props->getIntValue("fx.pause.time.max");
	int pause_img_y=props->getIntValue("img.pause.y.waterrelative");
	m_fxmgr_pause=new LocalEffectManager();
	for(int i=0;i<5;i++)
	{
		m_fxmgr_pause->addEffect(new ShakeEffect(
			0, pause_height, random(pause_time_min, pause_time_max)
		));
		
		m_img_pause[i]=fwkLoadImageFilePNG("res/levels/underwater/pause/pause_"+toString(i+1)+".png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		oslImageSetPosition(m_img_pause[i], props->getIntValue("img.pause."+toString(i+1)+".x"), pause_img_y);
	}
	m_fxmgr_pause->start();
	
	
	delete props;
	
	PFL::logConstructionEnd("LevelUnderwater");
	sceDisplayWaitVblankStart();
}

void LevelUnderwater::reset()
{
	Properties *props=new Properties("res/levels/underwater/underwater.properties");
	if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
	{
		m_waterlevel=props->getIntValue("easy.waterlevel.start");
		m_addline=props->getIntValue("easy.lines.start");
		m_water_upspeed=props->getIntValue("easy.water.upspeed");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
	{
		m_waterlevel=props->getIntValue("normal.waterlevel.start");
		m_addline=props->getIntValue("normal.lines.start");
		m_water_upspeed=props->getIntValue("normal.water.upspeed");
	}
	else if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
	{
		m_waterlevel=props->getIntValue("hard.waterlevel.start");
		m_addline=props->getIntValue("hard.lines.start");
		m_water_upspeed=props->getIntValue("hard.water.upspeed");
	}
	else
		fatalError("no such difficulty in level underwater");
	
	oslStopSound(m_music_pirates);
	oslStopSound(m_music_smoker);
	
	m_linepattern->reset();
	m_dropspeed=-1;
	
	m_fx_water_up->stop();
	m_fx_water_down->stop();
	m_fx_water_up->reset();
	m_fx_water_down->reset();
	
	m_songbanner_pirates->reset();
	m_songbanner_smoker	->reset();
	m_comboDrawer->reset();
	m_scenario_pirates->pause();
	m_scenario_pirates->reset();
	m_scenario_smoker->pause();
	m_scenario_smoker->reset();
	
	m_part=LEVEL_UNDERWATER_PART_PIRATES;
	
	m_upspeed=-1;
	m_fx_underwater_filter->stop();
	m_fx_underwater_filter->reset();
	
	m_fx_color_pieces_underwater->stop();
	m_fx_color_pieces_underwater->reset();
	
	m_triangle_user=false;
	m_square_user=false;
	
	m_watery=calculateWaterPixPosY();
	
	m_counter=0;
	
	//faut condamner les generateurs
	list<CondemnedParticleGeneratorPoint *>::iterator generators_it;
  	for(generators_it=m_partgen_bubbles->begin(); generators_it != m_partgen_bubbles->end(); generators_it++)
  	{
  		CondemnedParticleGeneratorPoint *generator=*generators_it;
  		generator->setCondemned(true);
  		generator->generationOff();
  	}
  	
  	delete props;
}

void LevelUnderwater::initPartgenCombos(Properties *props)
{
	RandomAnimationFactory *animf;
	PFXFRandomShake *randomshake;
	PFXFRandomGravity *randomgrav;
	
	/* CONSTANT */
	animf=new RandomAnimationFactory();
	for(int i=1; i<=props->getIntValue("piece.combo.particles.constant.animfactory.nbkinds");i++)
	{
		Anim *anim=new Anim("res/levels/underwater/bubbles/combo-constant/bubble"+toString(i)+".png");
		anim->setReferencePointCenter();
		animf->addAnimation(anim);
	}
	
	m_partgen_combo_constant=new CondemnedParticleGeneratorPoint(
		0, 0, 90, 1, 
		props->getIntValue("piece.combo.particles.constant.throughput.particles"),
		props->getIntValue("piece.combo.particles.constant.throughput.frames"),
		props->getIntValue("piece.combo.particles.constant.movetime"),
		props->getIntValue("piece.combo.particles.constant.distance"),
		props->getIntValue("piece.combo.particles.constant.lifespan"),
		animf, 
		-1
	);
	
	m_partgen_combo_constant->setLocalEffect1SimpleTranslation();
	randomshake=new PFXFRandomShake();
	randomshake->setWidthMin(props->getIntValue("piece.combo.particles.constant.fx.shake.width.min"));
	randomshake->setWidthMax(props->getIntValue("piece.combo.particles.constant.fx.shake.width.max"));
	randomshake->setTimeMin(props->getIntValue("piece.combo.particles.constant.fx.shake.time.min"));
	randomshake->setTimeMax(props->getIntValue("piece.combo.particles.constant.fx.shake.time.max"));
	m_partgen_combo_constant->setLocalEffect2(randomshake);
	randomgrav=new PFXFRandomGravity();
	randomgrav->setSpeedYMin(props->getFloatValue("piece.combo.particles.constant.fx.gravity.speed.min"));
	randomgrav->setSpeedYMax(props->getFloatValue("piece.combo.particles.constant.fx.gravity.speed.max"));
	randomgrav->setAccelYMin(props->getFloatValue("piece.combo.particles.constant.fx.gravity.acceleration.min"));
	randomgrav->setAccelYMax(props->getFloatValue("piece.combo.particles.constant.fx.gravity.acceleration.max"));
	m_partgen_combo_constant->setLocalEffect3(randomgrav);
	
	
	/* COMBO */
	animf=new RandomAnimationFactory();
	for(int i=1; i<=props->getIntValue("piece.combo.particles.combos.animfactory.nbkinds");i++)
	{
		Anim *anim=new Anim("res/levels/underwater/bubbles/combo-decrement/bubble"+toString(i)+".png");
		anim->setReferencePointCenter();
		animf->addAnimation(anim);
	}
	
	m_partgen_combo_decrement=new CondemnedParticleGeneratorPoint(
		0, 0, 0, 360, 
		props->getIntValue("piece.combo.particles.combos.throughput.particles"),
		props->getIntValue("piece.combo.particles.combos.throughput.frames"),
		props->getIntValue("piece.combo.particles.combos.movetime"),
		props->getIntValue("piece.combo.particles.combos.distance"),
		props->getIntValue("piece.combo.particles.combos.lifespan"),
		animf, 
		props->getIntValue("piece.combo.particles.combos.nbloops")
	);
	m_partgen_combo_decrement->setLocalEffect1SimpleTranslation();
	randomshake=new PFXFRandomShake();
	randomshake->setWidthMin(props->getIntValue("piece.combo.particles.combos.fx.shake.width.min"));
	randomshake->setWidthMax(props->getIntValue("piece.combo.particles.combos.fx.shake.width.max"));
	randomshake->setTimeMin(props->getIntValue("piece.combo.particles.combos.fx.shake.time.min"));
	randomshake->setTimeMax(props->getIntValue("piece.combo.particles.combos.fx.shake.time.max"));
	m_partgen_combo_decrement->setLocalEffect2(randomshake);
	randomgrav=new PFXFRandomGravity();
	randomgrav->setSpeedYMin(props->getFloatValue("piece.combo.particles.combos.fx.gravity.speed.min"));
	randomgrav->setSpeedYMax(props->getFloatValue("piece.combo.particles.combos.fx.gravity.speed.max"));
	randomgrav->setAccelYMin(props->getFloatValue("piece.combo.particles.combos.fx.gravity.acceleration.min"));
	randomgrav->setAccelYMax(props->getFloatValue("piece.combo.particles.combos.fx.gravity.acceleration.max"));
	m_partgen_combo_decrement->setLocalEffect3(randomgrav);
	
	/* EXPLODE */
	animf=new RandomAnimationFactory();
	for(int i=1; i<=props->getIntValue("piece.combo.particles.explode.animfactory.nbkinds");i++)
	{
		Anim *anim=new Anim("res/levels/underwater/bubbles/combo-explode/bubble"+toString(i)+".png");
		anim->setReferencePointCenter();
		animf->addAnimation(anim);
	}
	
	m_partgen_combo_explode=new CondemnedParticleGeneratorPoint(
		0, 0, 0, 360, 
		props->getIntValue("piece.combo.particles.explode.throughput.particles"),
		props->getIntValue("piece.combo.particles.explode.throughput.frames"),
		props->getIntValue("piece.combo.particles.explode.movetime"),
		0,
		props->getIntValue("piece.combo.particles.explode.lifespan"),
		animf, 
		props->getIntValue("piece.combo.particles.explode.nbloops")
	);
	//m_partgen_combo_explode->setLocalEffect1SimpleTranslation();
	m_partgen_combo_explode->setLocalEffect1(new PFXFRandomTranslation(
		props->getIntValue("piece.combo.particles.explode.distance.min"), 
		props->getIntValue("piece.combo.particles.explode.distance.max")
	));
	
	
	randomshake=new PFXFRandomShake();
	randomshake->setWidthMin(props->getIntValue("piece.combo.particles.explode.fx.shake.width.min"));
	randomshake->setWidthMax(props->getIntValue("piece.combo.particles.explode.fx.shake.width.max"));
	randomshake->setTimeMin(props->getIntValue("piece.combo.particles.explode.fx.shake.time.min"));
	randomshake->setTimeMax(props->getIntValue("piece.combo.particles.explode.fx.shake.time.max"));
	m_partgen_combo_explode->setLocalEffect2(randomshake);
	randomgrav=new PFXFRandomGravity();
	randomgrav->setSpeedYMin(props->getFloatValue("piece.combo.particles.explode.fx.gravity.speed.min"));
	randomgrav->setSpeedYMax(props->getFloatValue("piece.combo.particles.explode.fx.gravity.speed.max"));
	randomgrav->setAccelYMin(props->getFloatValue("piece.combo.particles.explode.fx.gravity.acceleration.min"));
	randomgrav->setAccelYMax(props->getFloatValue("piece.combo.particles.explode.fx.gravity.acceleration.max"));
	m_partgen_combo_explode->setLocalEffect3(randomgrav);
}


void LevelUnderwater::initGaugeGenerators(Properties *props)
{
	//les generateurs d'eau / de bulles représentant la jauge de combo
	
	RandomAnimationFactory *animf;
	PFXFRandomGravity *randomgrav;
	
	/* WATER */
	animf=new RandomAnimationFactory();
	for(int i=1; i<=props->getIntValue("gauge.particles.water.nbkinds");i++)
	{
		Anim *anim=new Anim("res/levels/underwater/gauge/water"+toString(i)+".png");
		anim->setReferencePointCenter();
		animf->addAnimation(anim);
	}
	int transl_time=props->getIntValue("gauge.particles.water.movetime");
	m_gen_gauge_left_water=new ParticleGeneratorPoint(
		0, 0, 0, props->getIntValue("gauge.particles.water.scale"), 
		props->getIntValue("gauge.particles.water.throughput.particles"), 
		props->getIntValue("gauge.particles.water.throughput.frames"), 
		transl_time, 
		0, 
		-1, 
		animf
	);
	m_gen_gauge_left_water->setLocalEffect1(new PFXFRandomTranslation(
		props->getIntValue("gauge.particles.water.distance.min"), 
		props->getIntValue("gauge.particles.water.distance.max")
	));
	randomgrav=new PFXFRandomGravity();
	randomgrav->setSpeedYMin(props->getFloatValue("gauge.particles.water.fx.gravity.speed.min"));
	randomgrav->setSpeedYMax(props->getFloatValue("gauge.particles.water.fx.gravity.speed.max"));
	randomgrav->setAccelYMin(props->getFloatValue("gauge.particles.water.fx.gravity.acceleration.min"));
	randomgrav->setAccelYMax(props->getFloatValue("gauge.particles.water.fx.gravity.acceleration.max"));
	m_gen_gauge_left_water->setLocalEffect2(randomgrav);
	
	SerialGlobalEffect *tint_fx=new SerialGlobalEffect(1);
	ParticlesProgressiveTintEffect *m_fx_alpha1=new ParticlesProgressiveTintEffect((int)((transl_time*27)/100), 0, RGBA(0,255,255,0), RGBA(0,186,240,255), 1);
	ParticlesProgressiveTintEffect *m_fx_alpha2=new ParticlesProgressiveTintEffect((int)((transl_time*18)/100), 0, RGBA(0,186,240,255), RGBA(0,140,230,255), 1);
	ParticlesProgressiveTintEffect *m_fx_alpha3=new ParticlesProgressiveTintEffect((int)((transl_time*55)/100), 0, RGBA(0,140,230,255), RGBA(0,0,200,0), 1);
	tint_fx->addEffect(m_fx_alpha1); 
	tint_fx->addEffect(m_fx_alpha2);
	tint_fx->addEffect(m_fx_alpha3);
	m_gen_gauge_left_water->setGlobalEffect1(tint_fx);
	
	//le generateur de droite c'est le meme mais orienté a gauche:
	m_gen_gauge_right_water=m_gen_gauge_left_water->clone();
	m_gen_gauge_right_water->setOrientation(180);
	
	m_gen_gauge_left_water->setX(-10);
	m_gen_gauge_left_water->setY(20);
	m_gen_gauge_right_water->setX(490);
	m_gen_gauge_right_water->setY(20);
	
		
	
	/* BUBBLES */
	
	animf=new RandomAnimationFactory();
	for(int i=1; i<=props->getIntValue("gauge.particles.bubbles.nbkinds");i++)
	{
		Anim *anim=new Anim("res/levels/underwater/gauge/bubble"+toString(i)+".png");
		anim->setReferencePointCenter();
		animf->addAnimation(anim);
	}
	//int transl_time=props->getIntValue("gauge.particles.water.movetime");
	m_gen_gauge_left_bubbles=new ParticleGeneratorPoint(
		0, 0, 0, props->getIntValue("gauge.particles.bubbles.scale"), 
		props->getIntValue("gauge.particles.bubbles.throughput.particles"), 
		props->getIntValue("gauge.particles.bubbles.throughput.frames"), 
		props->getIntValue("gauge.particles.bubbles.movetime"), 
		0,//props->getIntValue("gauge.particles.bubbles.distance"), 
		-1, 
		animf
	);
	m_gen_gauge_left_bubbles->setLocalEffect1(new PFXFRandomTranslation(
		props->getIntValue("gauge.particles.bubbles.distance.min"), 
		props->getIntValue("gauge.particles.bubbles.distance.max")
	));
	
	PFXFRandomShake *randomshake=new PFXFRandomShake();
	randomshake->setWidthMin(props->getIntValue("gauge.particles.bubbles.fx.shake.width.min"));
	randomshake->setWidthMax(props->getIntValue("gauge.particles.bubbles.fx.shake.width.max"));
	randomshake->setTimeMin(props->getIntValue("gauge.particles.bubbles.fx.shake.time.min"));
	randomshake->setTimeMax(props->getIntValue("gauge.particles.bubbles.fx.shake.time.max"));
	m_gen_gauge_left_bubbles->setLocalEffect2(randomshake);
	
	m_gen_gauge_left_bubbles->setOrientation(props->getIntValue("gauge.particles.bubbles.orientation.up"));
	
	//le generateur de droite c'est le meme mais orienté a gauche:
	m_gen_gauge_right_bubbles=m_gen_gauge_left_bubbles->clone();
	m_gen_gauge_right_bubbles->setOrientation(180-props->getIntValue("gauge.particles.bubbles.orientation.up"));
	
	randomgrav=new PFXFRandomGravity();
	randomgrav->setSpeedYMin(props->getFloatValue("gauge.particles.bubbles.fx.gravity.speed.min"));
	randomgrav->setSpeedYMax(props->getFloatValue("gauge.particles.bubbles.fx.gravity.speed.max"));
	randomgrav->setAccelYMin(props->getFloatValue("gauge.particles.bubbles.fx.gravity.acceleration.min"));
	randomgrav->setAccelYMax(props->getFloatValue("gauge.particles.bubbles.fx.gravity.acceleration.max"));
	randomgrav->setSpeedXMin(0.0f);
	randomgrav->setSpeedXMax(0.3f);
	m_gen_gauge_left_bubbles->setLocalEffect3(randomgrav);
	randomgrav=randomgrav->clone();
	randomgrav->setSpeedXMax(-0.3f);
	m_gen_gauge_right_bubbles->setLocalEffect3(randomgrav);
	
	
	m_gen_gauge_left_bubbles->setX(-10);
	m_gen_gauge_left_bubbles->setY(250);
	m_gen_gauge_right_bubbles->setX(490);
	m_gen_gauge_right_bubbles->setY(250);
	
	
	
	//m_gauge_side=false; //on n'affiche pas les 2 cotés en même temps mais l'un apres l'autre
//	if(utils::random(0,1)==0)
//		m_gauge_side=false;
//	else
//		m_gauge_side=true;
	m_gen_gauge_left_water->generationOff();
	m_gen_gauge_left_bubbles->generationOff();
	m_gen_gauge_right_water->generationOff();
	m_gen_gauge_right_bubbles->generationOff();
	m_gen_gauge_left_water->start();
	m_gen_gauge_left_bubbles->start();
	m_gen_gauge_right_water->start();
	m_gen_gauge_right_bubbles->start();
}

void LevelUnderwater::initPieceAnims(Properties *levelprops)
{
	Properties *piecesprops=new Properties("res/pieces/pieces.properties");
	
	m_anim_circle=new Anim(piecesprops->getIntValue("piece.circle.nbframes"), "circle", "res/pieces/circle/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_cross=new Anim(piecesprops->getIntValue("piece.cross.nbframes"), "cross", "res/pieces/cross/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_diamond=new Anim(piecesprops->getIntValue("piece.diamond.nbframes"), "diamond", "res/pieces/diamond/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_square=new Anim(piecesprops->getIntValue("piece.square.nbframes"), "square", "res/pieces/square/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_star=new Anim(piecesprops->getIntValue("piece.star.nbframes"), "star", "res/pieces/star/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_triangle=new Anim(piecesprops->getIntValue("piece.triangle.nbframes"), "triangle", "res/pieces/triangle/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	
	m_anim_circle->setReferencePointCenter();
	m_anim_cross->setReferencePointCenter();
	m_anim_diamond->setReferencePointCenter();
	m_anim_square->setReferencePointCenter();
	m_anim_star->setReferencePointCenter();
	m_anim_triangle->setReferencePointCenter();
	
	//combos
	m_anim_combo_explode=new Anim(piecesprops->getIntValue("piece.combo.nbframes"), "combo", "res/pieces/combo/", GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1);
	m_anim_combo_explode->setReferencePointCenter();
	m_anim_combo_shine=new Anim(piecesprops->getIntValue("piece.combo.shine.nbframes"), "combo_change_", "res/pieces/combo/change/", piecesprops->getIntValue("piece.combo.shine.time"), OSL_PF_8888, 1);
	m_anim_combo_shine->setReferencePointCenter();
	
	m_img_combo_numbers=new OSL_IMAGE *[22]; //tableau de pointeurs
	for(int i=0;i<=20;i++)
	{
		m_img_combo_numbers[i]=fwkLoadImageFilePNG("res/pieces/combo/numbers/combo_num_"+toString(i)+".png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		oslImageSetRotCenter(m_img_combo_numbers[i]);
	}
	m_img_combo_numbers[21]=fwkLoadImageFilePNG("res/pieces/combo/numbers/combo_num_more.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetRotCenter(m_img_combo_numbers[21]);
	
	delete piecesprops;
	
	
	
	//pieces speciales
	//bubble
	m_anim_special_bubble_explode=new Anim(
		levelprops->getIntValue("piece.special.bubble.nbimages"), 
		"bubble_explode_", "res/levels/underwater/piece_special/", 
		m_piece_special_explosion_time, OSL_PF_8888, 1
	);
	m_anim_special_bubble_explode->setReferencePointCenter();
	
	//square
	m_anim_special_square_explode=new Anim(
		levelprops->getIntValue("piece.special.square.explode.nbimages"), 
		"square_explode_", "res/levels/underwater/piece_special/", 
		GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1
	);
	m_anim_special_square_explode->setReferencePointCenter();
	
	int square_const_nbimages=levelprops->getIntValue("piece.special.square.constant.nbimages");
	m_anim_special_square_constant=new Anim(
		square_const_nbimages, 
		"square_constant_", "res/levels/underwater/piece_special/", 
		levelprops->getIntValue("piece.special.square.constant.time")
	);
	for(int i=0; i<levelprops->getIntValue("piece.special.square.constant.repeatlastimage"); i++)
	{
		m_anim_special_square_constant->addImage(
			"res/levels/underwater/piece_special/square_constant_"+toString(square_const_nbimages)+".png", 
			OSL_PF_8888
		);
	}
	m_anim_special_square_constant->setReferencePointCenter();
	
	//triangle
	m_anim_special_triangle_explode=new Anim(
		levelprops->getIntValue("piece.special.triangle.explode.nbimages"), 
		"triangle_explode_", "res/levels/underwater/piece_special/", 
		GAMEDEF_EXPLOSION_TIME, OSL_PF_8888, 1
	);
	m_anim_special_triangle_explode->setReferencePointCenter();
	
	int triangle_const_nbimages=levelprops->getIntValue("piece.special.triangle.constant.nbimages");
	m_anim_special_triangle_constant=new Anim(
		triangle_const_nbimages, 
		"triangle_constant_", "res/levels/underwater/piece_special/", 
		levelprops->getIntValue("piece.special.triangle.constant.time")
	);
	for(int i=0; i<levelprops->getIntValue("piece.special.triangle.constant.repeatlastimage"); i++)
	{
		m_anim_special_triangle_constant->addImage(
			"res/levels/underwater/piece_special/triangle_constant_"+toString(triangle_const_nbimages)+".png", 
			OSL_PF_8888
		);
	}
	m_anim_special_triangle_constant->setReferencePointCenter();
	
	
}

LevelUnderwater::~LevelUnderwater()
{
	PFL::logDestructionStart("LevelUnderwater");
	oslDeleteImage(m_img_grid_over);
	delete m_anim_grid_under;
	
	delete m_linepattern;
	delete m_songbanner_pirates;
	delete m_songbanner_smoker;
	delete m_comboDrawer;
	delete m_scenario_pirates;
	delete m_scenario_smoker;
	
	oslDeleteSound(m_music_pirates);
	oslDeleteSound(m_music_smoker);
	
	
	delete m_fx_water_up;
	delete m_fx_water_down;
	
	//oslDeleteImage(m_img_waves);
	
	oslDeleteSound(m_snd_water_up);
	oslDeleteSound(m_snd_water_down);
	oslDeleteSound(m_snd_combo);
	oslDeleteSound(m_snd_explode);
	
	m_fxmgr_waves->deleteAll();
	delete m_fxmgr_waves;
	
	oslDeleteImage(m_img_underwater_filter);
	delete m_fx_underwater_filter;
	
	delete m_anim_circle;
	delete m_anim_cross;
	delete m_anim_diamond;
	delete m_anim_square;
	delete m_anim_star;
	delete m_anim_triangle;
	delete m_anim_combo_explode;
	delete m_anim_combo_shine;
	for(int i=0;i<=21;i++)
	{
		oslDeleteImage(m_img_combo_numbers[i]);
	}
	delete[] m_img_combo_numbers;
	delete m_anim_special_bubble_explode;
	delete m_anim_special_square_explode;
	delete m_anim_special_square_constant;
	delete m_anim_special_triangle_explode;
	delete m_anim_special_triangle_constant;
	
	//les generateurs de bulles
	list<CondemnedParticleGeneratorPoint *>::iterator it;
  	for(it=m_partgen_bubbles->begin() ; it != m_partgen_bubbles->end(); it++ )
    {
    	CondemnedParticleGeneratorPoint *gen=*it;
    	delete gen;
    }
	delete m_partgen_bubbles;
	
	delete m_partgen_combo_constant;
	delete m_partgen_combo_decrement;
	delete m_partgen_combo_explode;
	
	delete m_partgen_bubbles_frontback[0];
	delete m_partgen_bubbles_frontback[1];
	
	delete m_gen_gauge_left_water;
	delete m_gen_gauge_left_bubbles;
	delete m_gen_gauge_right_water;
	delete m_gen_gauge_right_bubbles;
	
	delete m_fx_color_pieces_underwater;
	
	delete m_anim_underwater_bg;
	
	delete m_animals;
	
	delete m_anim_waves;
	
	//next pair
	oslDeleteImage(m_img_buoy);
	delete m_fx_nextpair_shake;
	delete m_fx_nextpair_resize;
	
	//pause
	m_fxmgr_pause->deleteAll();
	delete m_fxmgr_pause;
	for(int i=0;i<5;i++)
	{
		oslDeleteImage(m_img_pause[i]);
	}
		
	PFL::logDestructionEnd("LevelUnderwater");
}

//dessin avec une paire
void LevelUnderwater::draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	//Pos *pixPos0=grid->getContentPosition(currentPair->getGridPos(0).x, currentPair->getGridPos(0).y);
	//currentPair->drawXY(pixPos0->x, pixPos0->y, grid, fx_trans, APPLY_FX_ON_NONE);
	
//	int x1=currentPair->getGridPos().x, y1=currentPair->getGridPos().y;
//	int x2=x1, y2=y1;
//	switch(currentPair->getOrientation())
//	{
//		case ORIENTATION_EAST:
//		{
//			x2=x2+GAMEDEF_PIECE_WIDTH;
//			break;
//		}
//		case ORIENTATION_WEST:
//		{
//			x2=x2-GAMEDEF_PIECE_WIDTH;
//			break;
//		}
//		case ORIENTATION_NORTH:
//		{
//			y2=y2-GAMEDEF_PIECE_HEIGHT;
//			break;
//		}
//		case ORIENTATION_EAST:
//		{
//			y2=y2+GAMEDEF_PIECE_HEIGHT;
//			break;
//		}
//	}
	if(!isPaused())
	{
		Pos piece0Pos=currentPair->getGridPos(0);
		Pos piece1Pos=currentPair->getGridPos(1);
		
		drawPiece(currentPair->getPiece(0), piece0Pos.x, piece0Pos.y, grid, fx_trans);
		drawPiece(currentPair->getPiece(1), piece1Pos.x, piece1Pos.y, grid, fx_trans);
	}
	drawAfter(grid, nextPair, cc);
}

//dessin avec 2 pieces
void LevelUnderwater::draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	
	if(!isPaused())
	{
		drawPiece(pieceConnected, pieceConnectedPos.x, pieceConnectedPos.y, grid);
		drawPiece(pieceUnconnected, pieceUnconnectedPos.x, pieceUnconnectedPos.y, grid);
	}
	drawAfter(grid, nextPair, cc);
}

//dessin avec rien
void LevelUnderwater::draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer)
{
	drawBefore(grid, nextPair, cc);
	drawAfter(grid, nextPair, cc);
}

void LevelUnderwater::drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	//le ciel
	oslDrawGradientRect(0, 0, 480, m_watery, m_color_sky_top, m_color_sky_top, m_color_sky_bottom, m_color_sky_bottom);
	
	//le fond sous l'eau
	m_anim_underwater_bg->drawXY(0, m_watery>0?m_watery:0);
	
		
	//les vagues
	fx::drawXY(
		m_anim_waves->getImage(), 
		0, m_watery+m_anim_waves->getY()
	);
	
	m_partgen_bubbles_frontback[0]->draw();
	
	//les poissons
	m_animals->draw();
		
	
	if(!isPaused())
	{
		//dessin des ordres du joueur + montée/descente de l'eau
		int orders_x=0;
		if(m_difficulty==GAMEDEF_DIFFICULTY_HARD)
			orders_x=379; //taille: 19
		else if(m_difficulty==GAMEDEF_DIFFICULTY_NORMAL)
			orders_x=370; //taille: 19
		else if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
			orders_x=360;
	
		//l'eau monte
		if(m_fx_water_up->isRunning())
		{
			if(m_square_user) //ordre du joueur
			{
				oslDrawFillRect(orders_x, 37, orders_x+19, 55, RGB(255, 128, 255));
			}
			else //c'est pas un ordre, ca monte tout seul
			{
				oslDrawFillRect(orders_x, 37, orders_x+19, 40, RGB(255, 0, 0));
			}
		}	
		
		//l'eau descend
		if(m_fx_water_down->isRunning())
		{
			if(m_triangle_user) //ordre du joueur
			{
				oslDrawFillRect(orders_x, 65, orders_x+19, 82, RGB(0, 187, 0));
			}
			else //c'est pas un ordre, ca descend tout seul (n'arrive jamais, mais bon au cas ou c'est pret)
			{
				oslDrawFillRect(orders_x, 80, orders_x+19, 82, RGB(255, 0, 0));
			}
		}
		
		
		
		//la grille
		oslDrawImage(m_img_grid_over);
		m_anim_grid_under->draw();
		
		//dessin du contenu de la grille, pieces qui explosent pas
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				AbstractPiece *piece=mco->getPiece();
				if(piece!=NULL) //PIECE NON NULLE
				{
					if(!mco->isExploding())
					{
						Pos pos=grid->getContentPosition(i, j); 
						
						if(12-j<=m_waterlevel) //piece sous l'eau
						{
							//mco->getPiece()->drawXY(pos->x,pos->y,m_fxmgr_waves->getAt(j));
							if(piece->getType()==PIECE_TYPE_NORMAL && m_waterlevel!=13)
							{
								//oslSetAlpha(OSL_FX_TINT, m_color_pieces_underwater);
								m_fx_color_pieces_underwater->activate();
							}
		
							if(mco->getMCOContent()!=NULL && (m_waterlevel!=13 || piece->getType()==PIECE_TYPE_COMBO || piece->getType()==PIECE_TYPE_LEVEL))
								mco->getMCOContent()->draw(m_fxmgr_waves->getAt(j));
							//if(m_waterlevel==13 && piece->getType()==PIECE_TYPE_NORMAL)
							else
								piece->drawXY(pos.x,pos.y, m_fxmgr_waves->getAt(j));
							
							oslSetAlpha(OSL_FX_DEFAULT, 0);
						}
						else //piece au dessus de l'eau
						{
							if(piece->getType()==PIECE_TYPE_LEVEL && mco->getMCOContent()!=NULL)
								mco->getMCOContent()->draw();
							else
								piece->drawXY(pos.x,pos.y);
						}
					}
				}
			}	
		}
		
		//dessin du contenu de la grille, pieces qui explosent
		for(int i=0;i<grid->getNbCols();i++)
		{
			for(int j=0;j<grid->getNbLines();j++)
			{
				MatrixContentObject *mco=grid->getAt(i,j);
				AbstractPiece *piece=mco->getPiece();
				if(piece!=NULL) //PIECE NON NULLE
				{
					if(mco->isExploding())
					{
						Pos pos=grid->getContentPosition(i, j); 
						if(12-j<=m_waterlevel) //piece sous l'eau
						{
							//mco->getPiece()->drawXY(pos->x,pos->y,m_fxmgr_waves->getAt(j));
							if(piece->getType()==PIECE_TYPE_NORMAL && !mco->isColorExploding() && m_waterlevel!=13)
							{
								//oslSetAlpha(OSL_FX_TINT, m_color_pieces_underwater);
								m_fx_color_pieces_underwater->activate();
							}
		
							if(mco->getMCOContent()!=NULL && !mco->isColorExploding() && (m_waterlevel!=13 || piece->getType()==PIECE_TYPE_LEVEL))
								mco->getMCOContent()->draw(m_fxmgr_waves->getAt(j));
							else
								piece->drawXY(pos.x,pos.y, m_fxmgr_waves->getAt(j));
		
							oslSetAlpha(OSL_FX_DEFAULT, 0);
						}
						else //piece au dessus de l'eau
						{
							if((piece->getType()==PIECE_TYPE_COMBO || piece->getType()==PIECE_TYPE_LEVEL) && mco->getMCOContent()!=NULL)
								mco->getMCOContent()->draw();
							else
								piece->drawXY(pos.x,pos.y);
						}
					}
				}
			}	
		}
	}
}

void LevelUnderwater::drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc)
{
	if(!isPaused())
	{
		//nextPair->drawXY(61, 30, grid, APPLY_FX_ON_NONE);
		drawNextPair(nextPair, grid);
	
		/* BUBBLES */
	  	list<CondemnedParticleGeneratorPoint *>::iterator generators_it;
	  	for(generators_it=m_partgen_bubbles->begin(); generators_it != m_partgen_bubbles->end(); generators_it++)
	  	{
	  		CondemnedParticleGeneratorPoint *generator=*generators_it;
	  		//il est condamné
	  		if(generator->isCondemned())
	  		{
	  			generator->draw();
	  		}
	  	}
		
		//generateurs de la jauge de combos
		m_gen_gauge_left_water->draw();
		m_gen_gauge_left_bubbles->draw();
		m_gen_gauge_right_water->draw();
		m_gen_gauge_right_bubbles->draw();
	}
	else //isPaused()
	{
		for(unsigned int i=0;i<5;i++)
		{
			OSL_IMAGE *img=m_img_pause[i];
			fx::drawXY(img, img->x, img->y+m_watery, m_fxmgr_pause->getAt(i));
		}
	}
	
	m_partgen_bubbles_frontback[1]->draw();
	
	m_fx_underwater_filter->activate();
	fx::drawXY(
		m_img_underwater_filter, 0, 
		m_watery
	);
	oslSetAlpha(OSL_FX_DEFAULT, 0);

	if(!isPaused())
	{
		m_songbanner_pirates->draw();
		m_songbanner_smoker->draw();
		
		m_comboDrawer->drawXY(240, 180);
	
	
		//oslDrawString(10, 200, toString(m_partgen_bubbles->size()).c_str());
	}
}

void LevelUnderwater::drawPiece(AbstractPiece *piece, int grid_x, int grid_y, GameGrid *grid, AbstractLocalEffect *fx_trans)
{
	Pos pos=grid->getContentPosition(grid_x, grid_y);
	if(12-grid_y<=m_waterlevel) //elle est sous l'eau
	{
		//oslSetAlpha(OSL_FX_TINT, m_color_pieces_underwater);
		m_fx_color_pieces_underwater->activate();
		fx::drawXY(animFromShape(piece->getShape())->getImage(), pos.x, pos.y, m_fxmgr_waves->getAt(grid_y), fx_trans);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	else //elle est hors de l'eau
	{
		piece->drawXY(pos.x, pos.y, fx_trans);
	}
}

void LevelUnderwater::drawNextPair(Pair *nextPair, GameGrid *grid)
{		
	//nextPair->drawXY(m_pos_nextpair.x, m_pos_nextpair.y+m_watery, grid, NULL, APPLY_FX_ON_BOTH, m_fx_nextpair_shake);
	fx::drawXY(m_img_buoy, m_img_buoy->x, m_img_buoy->y+m_watery, m_fx_nextpair_shake);
	AbstractPiece *piece0=nextPair->getPiece(0);
	AbstractPiece *piece1=nextPair->getPiece(1);
	
	piece0->drawXY(m_pos_nextpair.x, m_pos_nextpair.y+m_watery, m_fx_nextpair_shake, m_fx_nextpair_resize);
	
	piece1->drawXY(
		m_pos_nextpair.x+int(GAMEDEF_PIECE_WIDTH*m_nextpair_size_coeff), 
		m_pos_nextpair.y+m_watery, 
		m_fx_nextpair_shake, m_fx_nextpair_resize
	);
}

void LevelUnderwater::start_specific()
{
	oslPlaySound(m_music_pirates, 0);
	m_scenario_pirates->restart();
	m_fxmgr_waves->start();
}

bool LevelUnderwater::hasEvent(int event)
{
	if(m_scenario_pirates->hasEvent(event))
		return true;
	
	if(m_scenario_smoker->hasEvent(event))
		return true;
	
	return false;
}

void LevelUnderwater::waterUp()
{
	m_fx_water_up->restart();
	oslPlaySound(m_snd_water_up, 3);
}

void LevelUnderwater::waterDown()
{
	m_fx_water_down->restart();
	oslPlaySound(m_snd_water_down, 4);
}

Anim *LevelUnderwater::animFromShape(int shape)
{
	switch(shape)
	{
		case PIECE_SHAPE_CIRCLE:
		{
			return m_anim_circle;
			break;	
		}
		case PIECE_SHAPE_CROSS:
		{
			return m_anim_cross;
			break;	
		}
		case PIECE_SHAPE_DIAMOND:
		{
			return m_anim_diamond;
			break;	
		}
		case PIECE_SHAPE_TRIANGLE:
		{
			return m_anim_triangle;
			break;	
		}
		case PIECE_SHAPE_STAR:
		{
			return m_anim_star;
			break;	
		}
		case PIECE_SHAPE_SQUARE:
		{
			return m_anim_square;
			break;	
		}
		default:
		{
			fatalError("no such shape in level underwater");
			return NULL;
		}
	}
	return NULL;
}

int LevelUnderwater::calculateWaterPixPosY()
{
	int waterY=272-(7+m_waterlevel*21);
	if(m_fx_water_up->isRunning())
	{
		waterY+=m_fx_water_up->getPosY();
	}
	if(m_fx_water_down->isRunning())
	{
		waterY+=m_fx_water_down->getPosY();
	}
	return waterY;
}

AbstractMCOContent *LevelUnderwater::newMCOContentSpecial(int type)
{
	switch(type)
	{
		case LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_BUBBLE:
		{
			return new MCOContentSimple(new Anim(*m_anim_special_bubble_explode));
		}
		case LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_SQUARE:
		{
			DJMCOContentLayered *mcoc=new DJMCOContentLayered();
			mcoc->addLayer(new Anim(*m_anim_special_square_constant), false)->start();
			mcoc->addLayerExplosion(new Anim(*m_anim_special_square_explode), false);
			return mcoc;
		}
		case LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_TRIANGLE:
		{
			DJMCOContentLayered *mcoc=new DJMCOContentLayered();
			mcoc->addLayer(new Anim(*m_anim_special_triangle_constant), false)->start();
			mcoc->addLayerExplosion(new Anim(*m_anim_special_triangle_explode), false);
			return mcoc;
		}
	}
	
	return NULL;
}

AbstractMCOContent *LevelUnderwater::newMCOContentComboBubbles(int nbCombos)
{
	MCOContentCombo *mcocc=new MCOContentCombo(new Anim(*m_anim_combo_explode), new Anim(*m_anim_combo_shine), m_img_combo_numbers, nbCombos);
	
	CondemnedParticleGeneratorPoint *partgen_combo_constant=m_partgen_combo_constant->clone();
	CondemnedParticleGeneratorPoint *partgen_combo_decrement=m_partgen_combo_decrement->clone();
	CondemnedParticleGeneratorPoint *partgen_combo_explode=m_partgen_combo_explode->clone();
	
	m_partgen_bubbles->push_back(partgen_combo_constant);
	m_partgen_bubbles->push_back(partgen_combo_decrement);
	m_partgen_bubbles->push_back(partgen_combo_explode);
	
	return new MCOContentFlagged(new MCOContentComboBubbles(mcocc, partgen_combo_constant, partgen_combo_decrement, partgen_combo_explode));
}

void LevelUnderwater::destroyParticlesOverWater(AbstractParticleGenerator *gen)
{
	//int waterY=getWaterPixPosY();
  			
	//on detruit les bulles qui dépassent de la surface
	list<Particle *> *particles=gen->getParticles();
	list<Particle *>::iterator it;
	for (it=particles->begin() ; it != particles->end(); /*it++*/ )
  	{
  		Particle *particle=*it;
  		int particleY=particle->getYwithFX();
  		if(particleY<=m_watery)
  		{
  			it=gen->destroyParticle(it);
  		}
  		else
  		{
  			it++;
  		}
  	}
}

void LevelUnderwater::destroyParticlesUnderWater(AbstractParticleGenerator *gen)
{
	//on detruit les bulles qui dépassent de la surface
	list<Particle *> *particles=gen->getParticles();
	list<Particle *>::iterator it;
	for (it=particles->begin() ; it != particles->end(); /*it++*/ )
  	{
  		Particle *particle=*it;
  		int particleY=particle->getYwithFX();
  		if(particleY>=m_watery)
  		{
  			it=gen->destroyParticle(it);
  		}
  		else
  		{
  			it++;
  		}
  	}
}

int LevelUnderwater::getNbSpecialExplosions()
{
	int ret=m_nb_specialexplosions;
	m_nb_specialexplosions=0;
	return ret;
}

void LevelUnderwater::explodePiecesSpecialWithType(GameGrid *grid, int type)
{
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			AbstractPiece *p=mco->getPiece();
			
			if(p!=NULL && p->getType()==PIECE_TYPE_LEVEL)
			{
				if(p->specialEvent(PIECE_EVENT_LEVEL_GETID)==type)
				{
					if(!mco->isExploding())
					{
						mco->startExplosion();
						m_nb_specialexplosions++;
					}
				}
			}
		}
	}
}

bool LevelUnderwater::pieceSpecialTypeIsPresent(GameGrid *grid, int type)
{
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			AbstractPiece *p=mco->getPiece();
			
			if(p!=NULL && p->getType()==PIECE_TYPE_LEVEL)
			{
				if(p->specialEvent(PIECE_EVENT_LEVEL_GETID)==type)
				{
					return true;
				}
			}
		}
	}
	
	return false;
}

void LevelUnderwater::setGridClipping(int watery)
{
	if(watery-m_img_grid_over->y>0)
		oslSetImageTile(m_img_grid_over, m_img_grid_over->offsetX0, m_img_grid_over->offsetY0, m_img_grid_over->offsetX1, watery-m_img_grid_over->y);
	else
		oslSetImageTile(m_img_grid_over, m_img_grid_over->offsetX0, m_img_grid_over->offsetY0, m_img_grid_over->offsetX1, 0);
	//m_img_grid_over->offsetY1=watery-m_img_grid_over->y;
	
	//int y=watery-m_anim_grid_under->getImage()->y;
	OSL_IMAGE *img=m_anim_grid_under->getImage();
	int height=img->sizeY-watery+m_img_grid_over->y;
	//m_anim_grid_under->getImage()->offsetY0=m_anim_grid_under->getImage()->sizeY-height;
	if(img->sizeY-height>0)
	{
		oslSetImageTile(img, img->offsetX0, img->sizeY-height, img->offsetX1, img->offsetY1);
		m_anim_grid_under->setY(watery);
	}
	else	
	{
		oslSetImageTile(img, img->offsetX0, 0, img->offsetX1, img->offsetY1);
		m_anim_grid_under->setY(m_img_grid_over->y);
	}
	
	
}

void LevelUnderwater::update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{
	m_animals->update();
	m_anim_underwater_bg->update();
	m_anim_waves->update();
	m_fxmgr_pause->update();
	m_partgen_bubbles_frontback[0]->update();
	m_partgen_bubbles_frontback[1]->update();
	destroyParticlesOverWater(m_partgen_bubbles_frontback[0]);
	destroyParticlesOverWater(m_partgen_bubbles_frontback[1]);
}

//appelé uniquement si le level n'est pas en pause.
void LevelUnderwater::update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans)
{		
	bool sound_combo=false;
	bool sound_explode=false;
	
	//POSITION DE L'EAU
	m_fx_water_up->update();
	m_fx_water_down->update();
	if(m_fx_water_up->finished())
	{
		m_fx_water_up->reset();
		m_waterlevel++;
		m_square_user=false;
	}
	if(m_fx_water_down->finished())
	{
		m_fx_water_down->reset();
		m_waterlevel--;
		m_triangle_user=false;
	}
	m_watery=calculateWaterPixPosY();
	
	//poissons
	m_animals->setWaterPosition(m_watery);
	m_animals->update();
	
	//cas special comme pour la pdrom: on utilise des anims séparées
	for(int i=0;i<grid->getNbCols();i++)
	{
		for(int j=0;j<grid->getNbLines();j++)
		{
			MatrixContentObject *mco=grid->getAt(i,j);
			AbstractPiece *p=mco->getPiece();
			Pos pos=grid->getContentPosition(i,j);
			if(p==NULL)
			{
				if(mco->getMCOContent()!=NULL)
				{
					mco->deleteMCOContent();
				}
			}
			else //piece non null
			{
				if(p->getType()==PIECE_TYPE_NORMAL)
				{
					//on cree si une piece n'en a pas
					if(mco->getMCOContent()==NULL)
					{
						Anim *anim=animFromShape(p->getShape());
						mco->setMCOContent(new MCOContentSimple(new Anim(*anim)));
					}				
				}
				else if(p->getType()==PIECE_TYPE_COMBO)
				{
					//remplir si c'est une piece combo
					if(mco->getMCOContent()==NULL)
					{
						int nbCombos=p->specialEvent(PIECE_EVENT_COMBO_GET);
						mco->setMCOContent(newMCOContentComboBubbles(nbCombos));
					}
					
					//on met a jour le nombre de combos
					int nbCombosPiece=p->specialEvent(PIECE_EVENT_COMBO_GET);
					int nbCombosMCOContent=mco->getMCOContent()->specialEvent(MCOCONTENT_EVENT_COMBO_GETNB);
					if(nbCombosPiece!=nbCombosMCOContent)
					{
						mco->getMCOContent()->specialEvent(MCOCONTENT_EVENT_COMBO_DECREMENT);
						if(pos.y>m_watery)
						{
							sound_combo=true;
						}
					}
				}
				else if(p->getType()==PIECE_TYPE_LEVEL)
				{
					int id=p->specialEvent(PIECE_EVENT_LEVEL_GETID);
					if(mco->getMCOContent()==NULL)
					{
						if(id==LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_BUBBLE)
							mco->setMCOContent(newMCOContentSpecial(LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_BUBBLE));
						else if(id==LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_SQUARE)
							mco->setMCOContent(newMCOContentSpecial(LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_SQUARE));
						else if(id==LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_TRIANGLE)
							mco->setMCOContent(newMCOContentSpecial(LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_TRIANGLE));
						else
							fatalError("LevelUnderwater: une piece speciale a un id inconnu");
					}	
					if(12-j>m_waterlevel)
					{
						if(!mco->isExploding() && id==LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_BUBBLE)
						{
							mco->startExplosion();
							m_nb_specialexplosions++;
						}
					}
				}
				
				if(mco->getMCOContent()!=NULL)
				{
					AbstractMCOContent *mcoc=mco->getMCOContent();
					
					//on démarre si la piece explose
					if(mco->isExploding() && !mcoc->isRunning())
					{
						if(p->getType()==PIECE_TYPE_COMBO && pos.y>m_watery && !mcoc->specialEvent(MCOCONTENT_EVENT_ISFLAGGED))
						{
							sound_explode=true;
							sound_combo=true;
							mcoc->specialEvent(MCOCONTENT_EVENT_SET_FLAG_TRUE);
						}
						mcoc->start();
					}
				}
				
			}
			
			
			if(mco->getMCOContent()!=NULL) //peu importe le cas, on update
			{
				mco->getMCOContent()->setX(pos.x);
				mco->getMCOContent()->setY(pos.y);
				mco->getMCOContent()->update();
			}
		}
	}
	
	if(sound_combo)
	{
		oslPlaySound(m_snd_combo, 6);
	}
	if(sound_explode)
	{
		//message("explode");
		oslPlaySound(m_snd_explode, 7);
	}
		
	
	
	if(m_part==LEVEL_UNDERWATER_PART_PIRATES)
		m_songbanner_pirates->update();
	else
		m_songbanner_smoker->update();
	m_scenario_pirates->update();
	m_scenario_smoker->update();
	m_fxmgr_waves->update();
	m_fx_underwater_filter->update();
	m_fx_color_pieces_underwater->update();
	m_anim_grid_under->update();
	m_anim_underwater_bg->update();
	m_anim_waves->update();
	m_fx_nextpair_shake->update();
	
	setGridClipping(m_watery);
	
	if(m_part==LEVEL_UNDERWATER_PART_PIRATES && m_scenario_pirates->getTime()>=LEVEL_UNDERWATER_DURATION_PIRATES)
	{
		m_part=LEVEL_UNDERWATER_PART_SMOKER;
		oslPlaySound(m_music_smoker, 2);
		m_scenario_smoker->restart();
		m_water_upspeed=m_water_upspeed_smoker;
	}	
	  	
  	/* BUBBLES */
  	list<CondemnedParticleGeneratorPoint *>::iterator generators_it;
  	for(generators_it=m_partgen_bubbles->begin(); generators_it != m_partgen_bubbles->end(); /*generators_it++*/ )
  	{
  		CondemnedParticleGeneratorPoint *generator=*generators_it;
  		//generator->update();
  		//il est condamné
  		bool condemned=generator->isCondemned();
  		if(condemned)
  		{
  			if(generator->getCurrentNbParticles()==0) //et il a plus aucune bulle
  			{
  				//on le detruit
  				generators_it=m_partgen_bubbles->erase(generators_it);
  				delete generator;
  				generator=NULL;
  			}
  			else
  			{
  				//condamné, mais avec encore des bulles, la piece combo ne l'updatera plus car elle est detruite, on s'en charge
	  			generator->update();
	  			generators_it++;
  			}
  		}
  		if(generator!=NULL && generator->getCurrentNbParticles()>0) //il a encore des bulles, condamné ou pas
  		{
  			//on detruit les bulles qui dépassent de la surface
  			destroyParticlesOverWater(generator);
  		}
  		if(!condemned)
  		{
  			generators_it++;
  		}
  	}
	
	//generateurs front et back
	m_partgen_bubbles_frontback[0]->update();
	m_partgen_bubbles_frontback[1]->update();
	destroyParticlesOverWater(m_partgen_bubbles_frontback[0]);
	destroyParticlesOverWater(m_partgen_bubbles_frontback[1]);
	
	m_counter++;
	if(m_counter>=m_water_upspeed && !m_fx_water_up->isRunning() && !m_fx_water_up->finished() && isWaterMovementOk(cc))
	{
		m_counter=0;
		waterUp();
	}
	
	//generateurs de la jauge de combos
	//oslDrawFillRect(400, 270-int((cc->getTime()*260)/cc->getTimeTotalAvailable()), 415, 270, RGB(255,0,0));
//	if(m_comboDrawer->getComboNum()<cc->getCombos()) //nouvelle explosion
//	{
//		m_gen_gauge_left_water->generationOff();
//		m_gen_gauge_left_bubbles->generationOff();
//		m_gen_gauge_right_water->generationOff();
//		m_gen_gauge_right_bubbles->generationOff();
//		if(m_gauge_side)
//		{
//			m_gen_gauge_left_water->generationOn();
//			m_gen_gauge_left_bubbles->generationOn();
//		}
//		else
//		{
//			m_gen_gauge_right_water->generationOn();
//			m_gen_gauge_right_bubbles->generationOn();
//		}
		//m_gauge_side=!m_gauge_side;
//	}
	
	if(cc->getTime()<=0)
	{
		m_gen_gauge_left_water->generationOff();
		m_gen_gauge_left_bubbles->generationOff();
		m_gen_gauge_right_water->generationOff();
		m_gen_gauge_right_bubbles->generationOff();
	}
	else
	{
//		if(m_gauge_side)
//		{
				m_gen_gauge_left_water->generationOn();
				m_gen_gauge_left_bubbles->generationOn();
//		}
//		else
//		{
				m_gen_gauge_right_water->generationOn();
				m_gen_gauge_right_bubbles->generationOn();
//		}
	}
	
	int gaugey=272-(cc->getTime()*272)/cc->getTimeTotalAvailable();
	m_gen_gauge_left_water->setY(gaugey);
	m_gen_gauge_right_water->setY(gaugey);
	m_gen_gauge_left_bubbles->setY(gaugey);
	m_gen_gauge_right_bubbles->setY(gaugey);
	
	
	m_gen_gauge_left_water->update();
	m_gen_gauge_left_bubbles->update();
	m_gen_gauge_right_water->update();
	m_gen_gauge_right_bubbles->update();
	destroyParticlesOverWater(m_gen_gauge_left_bubbles);
	destroyParticlesOverWater(m_gen_gauge_right_bubbles);
	destroyParticlesUnderWater(m_gen_gauge_left_water);
	destroyParticlesUnderWater(m_gen_gauge_right_water);
	
	
	m_comboDrawer->sync(cc);
	m_comboDrawer->update();
	
	//évènements du scénario
	if(hasEvent(1)) //1eres basses
	{
		m_dropspeed=m_dropspeed_normal;
	}
	if(hasEvent(2)) //cymballes fortes
	{
		if(m_difficulty==GAMEDEF_DIFFICULTY_EASY)
			m_addline++;
		else
			m_addline+=2;
	}
	if(hasEvent(101)) //transition debut smoker
	{
		m_fx_underwater_filter->start();
		m_fx_color_pieces_underwater->start();
	}
	
	
	
}

void LevelUnderwater::pause_specific() 
{
	if(m_part==LEVEL_UNDERWATER_PART_PIRATES)
	{
		oslPauseSound(m_music_pirates, 1);
		m_scenario_pirates->pause();
	}
	else
	{
		oslPauseSound(m_music_smoker, 1);
		m_scenario_smoker->pause();
	}
}

void LevelUnderwater::resume_specific()
{
	if(m_part==LEVEL_UNDERWATER_PART_PIRATES)
	{
		oslPauseSound(m_music_pirates, 0);
		m_scenario_pirates->start();
	}
	else
	{
		oslPauseSound(m_music_smoker, 0);
		m_scenario_smoker->start();
	}
}

list<AbstractPiece *> *LevelUnderwater::getNextLine()
{
	list<AbstractPiece *> *nextline=m_linepattern->getNextLine();
	
	list<AbstractPiece *>::iterator it;
	for(it=nextline->begin();it!=nextline->end();it++)
	{
		AbstractPiece *piece=*it;
		if(piece!=NULL && piece->getType()==PIECE_TYPE_LEVEL)
		{
			int type=piece->specialEvent(PIECE_EVENT_LEVEL_GETID);
			//les bulles explosent + vite que la normale
			if(type==LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_BUBBLE)
			{
				piece->setExplosionTime(m_piece_special_explosion_time);
			}
			//pendant smoker, aucune piece square ne peut apparaitre d'en bas
			else if(type==LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_SQUARE && m_part==LEVEL_UNDERWATER_PART_SMOKER)
			{
				piece->specialEvent(PIECE_EVENT_LEVEL_ID_REDUCE); //ca devient un triangle
			}
		}
	}
	
	return nextline;
}

bool LevelUnderwater::getAddLineEvent() //dit quand une ligne doit monter, dû au scénario
{
	if(m_addline>0)
	{
		m_addline--;
		return true;
	}
	return false;
}

int LevelUnderwater::getDropSpeed() //la vitesse a un instant t (elle peut etre modifiée par le scénario)
{
	return m_dropspeed;
}

int LevelUnderwater::getUpSpeed() //la vitesse ou les lignes montent a un instant t
{
	return m_upspeed;
}

void LevelUnderwater::buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid)
{
	if(!isPaused())
	{
		if(key==OSL_KEY_TRIANGLE)
		{
			if(m_waterlevel>m_water_min && !m_fx_water_down->isRunning() && !m_fx_water_down->finished())
			{
				waterDown();
				m_addline++;
				explodePiecesSpecialWithType(grid, LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_TRIANGLE);
				m_triangle_user=true;
			}
		}
		else if(key==OSL_KEY_SQUARE)
		{
			if(m_waterlevel<12 && !m_fx_water_up->isRunning() && !m_fx_water_up->finished() && pieceSpecialTypeIsPresent(grid, LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_SQUARE))
			{
				waterUp();
				m_counter=0;
				explodePiecesSpecialWithType(grid, LEVEL_UNDERWATER_PIECE_SPECIAL_TYPE_SQUARE);
				m_square_user=true;
			}
		}
	}
}

bool LevelUnderwater::finished()
{
	if(m_part==LEVEL_UNDERWATER_PART_SMOKER && m_scenario_smoker->getTime()>=LEVEL_UNDERWATER_DURATION_SMOKER)
	{
		return true;
	}
	return false;
}

float LevelUnderwater::getDuration()
{
	return LEVEL_UNDERWATER_DURATION_PIRATES+LEVEL_UNDERWATER_DURATION_SMOKER;
}

bool LevelUnderwater::isGameOver()
{
	if(m_waterlevel>=13)
		return true;
	return false;
}

bool LevelUnderwater::isWaterMovementOk(ComboCounter *cc)
{
	if(!cc->isEvent())
		return true;
	
	//else cc isevent!
	if(m_part==LEVEL_UNDERWATER_PART_PIRATES)
	{
		if(m_water_move_busy_pirates)
			return true;
	}
	else //smoker
	{
		if(m_water_move_busy_smoker)
			return true;
	}
	
	return false;
}






























