#ifndef LEVELUNDERWATER_H_
#define LEVELUNDERWATER_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/ComboNumberDrawer.h"
#include "code/game/EventsScenario.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/Pos.h"
#include "code/framework/anim/Anim.h"
#include "code/game/grid/AbstractMCOContent.h"
#include "code/game/levels/utils/DJMCOContentSimple.h"
#include "code/game/levels/impl/underwater/CondemnedParticleGeneratorPoint.h"
#include "code/game/levels/impl/underwater/CondemnedParticleGeneratorSegment.h"
#include "code/framework/Properties.h"
#include "code/game/levels/impl/underwater/MCOContentComboBubbles.h"
#include "code/game/levels/impl/underwater/fauna/AnimalManager.h"

using namespace std;

class LevelUnderwater: public AbstractLevel
{
	private:
		OSL_SOUND *m_music_pirates;
		OSL_SOUND *m_music_smoker;
		
		//la grille
		OSL_IMAGE *m_img_grid_over;
		Anim *m_anim_grid_under;
		
		LinePattern *m_linepattern;
		SongBanner *m_songbanner_pirates;
		SongBanner *m_songbanner_smoker;
		ComboNumberDrawer * m_comboDrawer;
		EventsScenario *m_scenario_pirates;
		EventsScenario *m_scenario_smoker;
		
		int m_part;
		int m_addline;
		
		int m_dropspeed;
		int m_upspeed;
		int m_dropspeed_normal;
		
		int m_water_min;
		
		//water level
		int m_waterlevel; //niveau actuel, si == 13 c'est perdu
		TranslationEffect *m_fx_water_up;
		TranslationEffect *m_fx_water_down;
		int m_water_upspeed;
		int m_water_upspeed_smoker;
		int m_counter;
		
		OSL_SOUND *m_snd_water_up;
		OSL_SOUND *m_snd_water_down;
		OSL_SOUND *m_snd_combo;
		OSL_SOUND *m_snd_explode;
		void waterUp();
		void waterDown();
		LocalEffectManager *m_fxmgr_waves;
		
		//filtre sous l'eau
		OSL_IMAGE *m_img_underwater_filter;
		ProgressiveTintEffect *m_fx_underwater_filter;
		//OSL_COLOR m_color_pieces_underwater;
		ProgressiveTintEffect *m_fx_color_pieces_underwater;
		
		//les animations séparées
		Anim *m_anim_circle;
		Anim *m_anim_cross;
		Anim *m_anim_diamond;
		Anim *m_anim_square;
		Anim *m_anim_star;
		Anim *m_anim_triangle;
		Anim *m_anim_combo_explode;
		Anim *m_anim_combo_shine; 
		OSL_IMAGE **m_img_combo_numbers;//tableau de pointeurs
		Anim *m_anim_special_bubble_explode;
		Anim *m_anim_special_square_explode;
		Anim *m_anim_special_square_constant;
		Anim *m_anim_special_triangle_explode;
		Anim *m_anim_special_triangle_constant;
		
		void initPieceAnims(Properties *levelprops);
		Anim *animFromShape(int shape);
		AbstractMCOContent *newMCOContentSpecial(int type);
		
		//les generateurs de bulles, qui doivent vivre même si la piece n'existe plus
		list<CondemnedParticleGeneratorPoint *> *m_partgen_bubbles;
		CondemnedParticleGeneratorPoint *m_partgen_combo_constant;
		CondemnedParticleGeneratorPoint *m_partgen_combo_decrement;
		CondemnedParticleGeneratorPoint *m_partgen_combo_explode;
		
		void initPartgenCombos(Properties *props);
		
		//generateurs de bulles devant et derriere
		ParticleGeneratorSegment* m_partgen_bubbles_frontback[2];
		
		void destroyParticlesOverWater(AbstractParticleGenerator *gen);
		void destroyParticlesUnderWater(AbstractParticleGenerator *gen);
		
		//les generateurs d'eau / de bulles représentant la jauge de combo
		ParticleGeneratorPoint *m_gen_gauge_left_water;
		ParticleGeneratorPoint *m_gen_gauge_left_bubbles;
		ParticleGeneratorPoint *m_gen_gauge_right_water;
		ParticleGeneratorPoint *m_gen_gauge_right_bubbles;
		//bool m_gauge_side; //on n'affiche pas les 2 cotés en même temps mais l'un apres l'autre
		void initGaugeGenerators(Properties *props);
		
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void update_pause_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); //pause, pour par exemple utiliser spliff politics
		void resume_specific();
		
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		
		void drawPiece(AbstractPiece *piece, int grid_x, int grid_y, GameGrid *grid, AbstractLocalEffect *fx_trans=NULL);
		
		bool hasEvent(int event);
		
		int calculateWaterPixPosY();
		int m_watery;
		
		AbstractMCOContent *newMCOContentComboBubbles(int nbCombos);
		
		int m_nb_specialexplosions;
		
		float m_piece_special_explosion_time;
		
		void explodePiecesSpecialWithType(GameGrid *grid, int type);
		bool pieceSpecialTypeIsPresent(GameGrid *grid, int type);
		
		//GameGrid *m_grid;
		
		void setGridClipping(int watery);
		
		bool m_water_move_busy_pirates;
		bool m_water_move_busy_smoker;
		bool isWaterMovementOk(ComboCounter *cc);
		
		//le joueur a demandé à l'eau de monter/descendre
		bool m_triangle_user;
		bool m_square_user;
		
		OSL_COLOR m_color_sky_top;
		OSL_COLOR m_color_sky_bottom;
		
		Anim *m_anim_underwater_bg;
		
		//les poissons
		AnimalManager *m_animals;
		
		//vagues
		Anim *m_anim_waves;
		
		//next pair
		Pos m_pos_nextpair;
		OSL_IMAGE *m_img_buoy;
		ShakeEffect *m_fx_nextpair_shake;
		SizeWowEffect *m_fx_nextpair_resize;
		float m_nextpair_size_coeff;
		
		void drawNextPair(Pair *nextPair, GameGrid *grid);
		
		//pause
		LocalEffectManager *m_fxmgr_pause;
		OSL_IMAGE *m_img_pause[5];
		
		
	
	public:
		LevelUnderwater(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~LevelUnderwater();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		bool finished();
		list<AbstractPiece *> *getNextLine();
		float getDuration();
		bool isGameOver();
		int getNbSpecialExplosions();
		void reset();
};

#endif /*LEVELUNDERWATER_H_*/
