#ifndef LEVELVOLTIGEUR_H_
#define LEVELVOLTIGEUR_H_

#include <oslib/oslib.h>
#include "code/game/levels/abstract/AbstractLevel.h"
#include "code/game/grid/GameGrid.h"
#include "code/game/Pair.h"
#include "code/game/levels/LinePattern.h"
#include "code/game/pieces/abstract/AbstractPiece.h"
#include <list>
#include "code/framework/time/Delayer.h"
#include "code/game/SongBanner.h"
#include "code/game/combos/ComboCounter.h"
#include "code/game/combos/ComboNumberDrawer.h"
#include "code/game/EventsScenario.h"
#include "code/framework/particles/complete/impl/ParticleGeneratorSegment.h"
#include "code/framework/fx/EffectsLib.h"


using namespace std;

class LevelVoltigeur : public AbstractLevel
{
	private:
			
		OSL_IMAGE *m_gridImage;
		LinePattern *m_linepattern;
			
		void start_specific(); //démarre la musique, le timer etc
		void update_specific(ComboCounter *cc, GameGrid *grid, Pair *nextPair, Pair *currentPair, SerialLocalEffect *fx_trans);
		void pause_specific(); 
		void resume_specific();
				
		SongBanner *m_songbanner;
		
		ComboNumberDrawer * m_comboDrawer;
		
		EventsScenario *m_scenario;
		OSL_SOUND* m_music;
		
		int m_dropspeed;
		int m_upspeed;
		
		int m_addline; //ajoute 1 qd on veut qu'une ligne monte
		
		RotationEffect *m_fx_rot;
		SerialLocalEffect *m_fx_move_hor;
		SerialLocalEffect *m_fx_move_vert;
		SerialLocalEffect *m_fx_move_hor_late;
		SerialLocalEffect *m_fx_move_vert_late;
		
		ShakeEffect *m_fx_shake_pair;
		
		
		int m_inversion_num; //a quelle inversion on en est? (elles sont différentes)
		ShakeEffect *m_fx_shake;
		
		int m_dropspeedNormal;
		int m_dropSpeedSlow;
		
		SizeWowEffect *m_fx_wow;
						
		void drawBefore(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		void drawAfter(GameGrid *grid, Pair *nextPair, ComboCounter *cc);
		
		int m_difficulty;
		
	public:
		LevelVoltigeur(int difficulty=GAMEDEF_DIFFICULTY_NORMAL, int nbPlayers=1);
		~LevelVoltigeur();
		
		//hérité:
		int getDropSpeed(); //la vitesse a un instant t (elle peut etre modifiée par le scénario)
		int getUpSpeed(); //la vitesse ou les lignes montent a un instant t
		void draw(GameGrid *grid, Pair *currentPair, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, AbstractPiece *pieceConnected, Pos pieceConnectedPos, AbstractPiece *pieceUnconnected, Pos pieceUnconnectedPos, Pair *nextPair, SerialLocalEffect *fx_trans, ComboCounter *cc, Scorer *scorer);
		void draw(GameGrid *grid, Pair *nextPair, ComboCounter *cc, Scorer *scorer);
		string getCode();
		
		void buttonPushEvent(OSL_KEY_BITS key, GameGrid *grid);
		
		//LinePattern *getLevelPieces(); 
		bool getAddLineEvent(); //dit quand une ligne doit monter, dû au scénario
		
		bool finished();
		
		list<AbstractPiece *> *getNextLine();
		
		float getDuration();
};

#endif /*LEVELVOLTIGEUR_H_*/
