#include "code/game/levels/utils/DJMCOContentLayered.h"

DJMCOContentLayered::DJMCOContentLayered(OSL_COLOR color)
{
	m_layers_normal=new vector<Anim *>();
	m_colorized_normal=new vector<bool>();
	
	m_layers_explosion=new vector<Anim *>();
	m_colorized_explosion=new vector<bool>();
	
	m_color=color;
	
	m_color_comp=new int[4];
	oslRgbaGet8888(m_color, m_color_comp[0], m_color_comp[1], m_color_comp[2], m_color_comp[3]);
	m_alpha = new OSL_ALPHA_PARAMS;
	m_curColor=new u8[4];
	m_newColor=new u8[4];
	
	m_exploding=false;
	m_x=0;
	m_y=0;
}

DJMCOContentLayered::~DJMCOContentLayered()
{
	for(unsigned int i=0;i<m_layers_normal->size();i++)
	{
		delete m_layers_normal->at(i);
	}
	delete m_layers_normal;
	delete m_colorized_normal;
	
	for(unsigned int i=0;i<m_layers_explosion->size();i++)
	{
		delete m_layers_explosion->at(i);
	}
	delete m_layers_explosion;
	delete m_colorized_explosion;
	
	delete[] m_color_comp;
	delete m_alpha;
	delete[] m_curColor;
	delete[] m_newColor;
}

bool DJMCOContentLayered::isExploding()
{
	return m_exploding;
}

Anim *DJMCOContentLayered::addLayer(Anim *aclone, bool colorized)
{
	m_layers_normal->push_back(aclone);
	m_colorized_normal->push_back(colorized);
	return aclone;
}

Anim *DJMCOContentLayered::addLayerExplosion(Anim *aclone, bool colorized)
{
	m_layers_explosion->push_back(aclone);
	m_colorized_explosion->push_back(colorized);
	return aclone;
}

OSL_COLOR DJMCOContentLayered::getColor()
{
	return m_color;
}

void DJMCOContentLayered::setColor(OSL_COLOR color)
{
	m_color=color;
	oslRgbaGet8888(m_color, m_color_comp[0], m_color_comp[1], m_color_comp[2], m_color_comp[3]);
}

//drawable
void DJMCOContentLayered::draw(AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	drawXY(m_x, m_y, fx1, fx2, fx3, fx4, fx5);
}

void DJMCOContentLayered::drawXY(int x, int y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	oslGetAlphaEx(m_alpha);
	oslRgbaGet8888(m_alpha->coeff1, m_curColor[0], m_curColor[1], m_curColor[2], m_curColor[3]);
	m_alpha->effect=OSL_FX_ALPHA | OSL_FX_TINT;
	for(int i=0;i<4;i++)
	{	
		m_newColor[i]=u8(float(m_curColor[i]*m_color_comp[i])/255.0f);
	}
	int oldcoeff1=m_alpha->coeff1;
	OSL_COLOR newcoeff1=RGBA(m_newColor[0], m_newColor[1], m_newColor[2], m_newColor[3]);
		
	vector<Anim *> *anims;
	vector<bool> *colorized;
	if(!m_exploding)
	{
		anims=m_layers_normal;
		colorized=m_colorized_normal;
	}
	else
	{
		anims=m_layers_explosion;
		colorized=m_colorized_explosion;
	}
	
	//on dessine
	for(unsigned int i=0;i<anims->size();i++)
	{
		if(colorized->at(i))
		{
			m_alpha->coeff1=newcoeff1;
			oslSetAlphaEx(m_alpha);
		}
		
		fx::drawXY(anims->at(i)->getImage(), x, y, fx1,fx2,fx3,fx4,fx5);
		
		//on remet comme avant
		m_alpha->coeff1=oldcoeff1;
		oslSetAlphaEx(m_alpha);
	}
}

//playable
void DJMCOContentLayered::update()
{
	if(!m_exploding)
	{
		for(unsigned int i=0;i<m_layers_normal->size();i++)
		{
			m_layers_normal->at(i)->update();
		}
	}
	else
	{
		for(unsigned int i=0;i<m_layers_explosion->size();i++)
		{
			m_layers_explosion->at(i)->update();
		}
	}
}

void DJMCOContentLayered::start()
{
	m_exploding=true;
	for(unsigned int i=0;i<m_layers_explosion->size();i++)
	{
		m_layers_explosion->at(i)->start();
	}
}

void DJMCOContentLayered::stop()
{
	if(!m_exploding)
	{
		for(unsigned int i=0;i<m_layers_normal->size();i++)
		{
			m_layers_normal->at(i)->stop();
		}
	}
	else
	{
		for(unsigned int i=0;i<m_layers_explosion->size();i++)
		{
			m_layers_explosion->at(i)->stop();
		}
	}
}

void DJMCOContentLayered::reset()
{
	m_exploding=false;
	for(unsigned int i=0;i<m_layers_explosion->size();i++)
	{
		m_layers_explosion->at(i)->reset();
	}
	for(unsigned int i=0;i<m_layers_normal->size();i++)
	{
		m_layers_normal->at(i)->reset();
	}
}

void DJMCOContentLayered::restart()
{
	m_exploding=false;
	for(unsigned int i=0;i<m_layers_explosion->size();i++)
	{
		m_layers_explosion->at(i)->restart();
	}
	for(unsigned int i=0;i<m_layers_normal->size();i++)
	{
		m_layers_normal->at(i)->restart();
	}
}

bool DJMCOContentLayered::isRunning()
{
	return m_exploding;
}


//positionable
void DJMCOContentLayered::setX(int x)
{
	m_x=x;
}

void DJMCOContentLayered::setY(int y)
{
	m_y=y;
}

int DJMCOContentLayered::getX()
{
	return m_x;
}

int DJMCOContentLayered::getY()
{
	return m_y;
}

State *DJMCOContentLayered::savestate(State * state)
{
	state->m_x=m_x;
	state->m_y=m_y;
	return state;
}

void DJMCOContentLayered::restorestate(State * state)
{
	m_x=state->m_x;
	m_y=state->m_y;
}

int DJMCOContentLayered::specialEvent(int event)
{
	return 0;
}
























