#include "code/game/menus/pdr4c/PDR4CSlideshow.h"
#include "code/game/gamedef.h"
#include "code/framework/utils.h"
#include "code/framework/check/loaders.h"

using namespace utils;

PDR4CSlideshow::PDR4CSlideshow(string properties)
{
	Properties *props=new Properties(properties);
	
	m_nbimg=props->getIntValue("slides.number");
	m_curimg=0;
	
	string folder=props->getStrValue("slides.folder");
	
	m_list=new vector<string>();
	m_list->reserve(m_nbimg);
	for(int i=0;i<m_nbimg;i++)
	{
		m_list->push_back(folder+"slide"+toString(i+1)+".png");
	}
	m_img=fwkLoadImageFilePNG(m_list->at(0), OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	
	m_music=fwkLoadSoundFileMP3(props->getStrValue("song.file"), OSL_FMT_STREAM);
	oslSetSoundLoop(m_music, true);
	m_songbanner=new SongBanner(props->getIntValue("songbanner.y"), "res/fonts/bmpFonts/russelsquare/15/nosmooth/", "normal", "italic", props->getStrValue("songbanner.text1"), props->getStrValue("songbanner.text2"), props->getStrValue("songbanner.text3"), props->getStrValue("songbanner.text4"));
	
	m_songbanner_color=RGB(props->getIntValue("songbanner.color.r"), props->getIntValue("songbanner.color.g"), props->getIntValue("songbanner.color.b"));
	
	delete props;
}

PDR4CSlideshow::~PDR4CSlideshow()
{
	delete m_list;
	oslDeleteSound(m_music);
	delete m_songbanner;
}

int PDR4CSlideshow::go()
{
	bool skip=false;
	
	oslPlaySound(m_music, 1);
	
	while(true)
	{
		//touches
		oslReadKeys();
		if (osl_pad.pressed.start || osl_pad.pressed.triangle) //Exit 
		{
			return MENU_MAIN;
		}
		else if(osl_pad.pressed.cross || osl_pad.pressed.circle || osl_pad.pressed.right || osl_pad.pressed.R)
		{
			if(m_curimg<m_nbimg-1)
			{
				m_curimg++;
				oslDeleteImage(m_img);
				m_img=fwkLoadImageFilePNG(m_list->at(m_curimg), OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_4444);
			}
		}
		else if(osl_pad.pressed.square || osl_pad.pressed.left || osl_pad.pressed.L)
		{
			if(m_curimg>0)
			{
				m_curimg--;
				oslDeleteImage(m_img);
				m_img=fwkLoadImageFilePNG(m_list->at(m_curimg), OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_4444);
			}
		}
		else if(osl_pad.pressed.select)
		{
			screenshot();
		}
		
		//update
		m_songbanner->update();
		
		if (!skip)
		{
			//dessin
			oslStartDrawing();
			oslDrawImage(m_img);	
			oslSetAlpha(OSL_FX_TINT, m_songbanner_color);	
			m_songbanner->draw();
			oslSetAlpha(OSL_FX_DEFAULT, 0);
			oslEndDrawing();
		}
		oslEndFrame();
		skip = oslSyncFrame();
		
	}
	return 0;
}



















