#include "code/game/menus/qj/QJCredits.h"
#include "code/game/menus/CreditsRoll.h"
#include "code/framework/utils.h"
#include "code/framework/Properties.h"
#include "code/game/gamedef.h"
#include "code/framework/check/loaders.h"

#include <string>
using namespace std;
using namespace utils;

QJCredits::QJCredits()
{
	m_music=fwkLoadSoundFileMP3("res/qj/credits/pedia.mp3", OSL_FMT_STREAM);
	oslSetSoundLoop(m_music, true);
	
	m_roll=new CreditsRoll("res/qj/credits/qj.credits");
	
	m_img_bordertop=fwkLoadImageFilePNG("res/qj/credits/border_t.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	
	m_img_borderbottom=fwkLoadImageFilePNG("res/qj/credits/border_b.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageSetCenter(m_img_borderbottom, 0, m_img_borderbottom->sizeY);
	oslImageSetPosition(m_img_borderbottom, 0, 272/*+15*/);
	
	Properties *props=new Properties("res/qj/credits/qjcredits.properties");
	
	m_songbanner=new SongBanner(	
		GAMEDEF_SONGBANNER_Y+3, 
		"res/fonts/bmpFonts/russelsquare/13/", 
		"normal", "italic", 
		props->getStrValue("songbanner.text1"), 
		props->getStrValue("songbanner.text2"), 
		props->getStrValue("songbanner.text3"), 
		props->getStrValue("songbanner.text4")
	);
		
	
	
	m_nbImages=props->getIntValue("bg.images.nb");
	
	m_img_current=fwkLoadImageFilePNG("res/qj/credits/images/bg_0.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	m_current_img_num=0;
	
	m_fx_bg=new SerialGlobalEffect(1);
	m_fx_bg->addEffect(
		new ProgressiveAlphaEffect(
			props->getIntValue("bg.images.transition.time"), 
			props->getIntValue("bg.images.transition.time"),
			0, props->getIntValue("bg.images.alpha"), 1)
	);
	m_fx_bg->addEffect(
		new ProgressiveAlphaEffect(
			props->getIntValue("bg.images.show.time"), 
			props->getIntValue("bg.images.show.time"),
			props->getIntValue("bg.images.alpha"), 
			props->getIntValue("bg.images.alpha"), 1)
	);
	m_fx_bg->addEffect(
		new ProgressiveAlphaEffect(
			props->getIntValue("bg.images.transition.time"), 
			props->getIntValue("bg.images.transition.time"),
			props->getIntValue("bg.images.alpha"), 0, 1)
	);
	m_fx_bg->addEffect(
		new ProgressiveAlphaEffect(
			props->getIntValue("bg.images.noshow.time"), 
			props->getIntValue("bg.images.noshow.time"),
			0, 0, 1)
	);
	
	delete props;
	
	
}

QJCredits::~QJCredits()
{
	delete m_roll;
	
	oslDeleteImage(m_img_bordertop);
	oslDeleteImage(m_img_borderbottom);
	
	delete m_songbanner;
	
	delete m_img_current;
	delete m_fx_bg;
	
	oslDeleteSound(m_music);
}

void QJCredits::go()
{
	
	m_fx_bg->restart();
	oslPlaySound(m_music, 0);
	
	bool exit=false;
	bool skip=false;
	while(!exit)
	{	
		//touches
		
		exit=keys();
		
		//phase d'update
		
		m_roll->update();
		m_songbanner->update();
		m_fx_bg->update();
		if(m_fx_bg->finished())
		{
			m_fx_bg->restart();
			m_current_img_num++;
			if(m_current_img_num>=m_nbImages)
				m_current_img_num=0;
			oslDeleteImage(m_img_current);
			m_img_current=fwkLoadImageFilePNG("res/qj/credits/images/bg_"+toString(m_current_img_num)+".png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
		}
				
		if(!skip)
		{
			//phase de dessin
			oslStartDrawing();
			drawScene();
			oslEndDrawing();
		}
        oslEndFrame();
		//Synchronize to ensure the game runs at 60 fps
        skip=oslSyncFrame();
		//oslAudioVSync(); //ne sert plus, oslEndFrame le fait
		
	}
	//il a choisi une option
	return;	
}

void QJCredits::drawScene()
{
	oslClearScreen(RGB(0,0,0));
	
	m_fx_bg->activate();
	oslDrawImage(m_img_current);
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	m_roll->draw();
	oslDrawImage(m_img_bordertop);
	oslDrawImage(m_img_borderbottom);
	//m_songbanner->draw();
}

bool QJCredits::keys()
{
	oslReadKeys();
	if (osl_pad.pressed.start || osl_pad.pressed.cross || osl_pad.pressed.circle) //Exit 
	{
		return true;
	}
	if(osl_pad.pressed.up)
	{
		m_roll->setSpeed(m_roll->getSpeed()+0.05f);
		
	}
	else if(osl_pad.pressed.down)
	{
		float speed=m_roll->getSpeed();
		speed-=0.05f;
		if(speed<0.2f)
			speed=0.2f;
		m_roll->setSpeed(speed);
	}
	if(osl_pad.pressed.R)
	{
		m_fx_bg->restart();
		m_current_img_num++;
		if(m_current_img_num>=m_nbImages)
			m_current_img_num=0;
		oslDeleteImage(m_img_current);
		m_img_current=fwkLoadImageFilePNG("res/qj/credits/images/bg_"+toString(m_current_img_num)+".png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	}
	else if(osl_pad.pressed.L)
	{
		m_fx_bg->restart();
		m_current_img_num--;
		if(m_current_img_num<0)
			m_current_img_num=m_nbImages;
		oslDeleteImage(m_img_current);
		m_img_current=fwkLoadImageFilePNG("res/qj/credits/images/bg_"+toString(m_current_img_num)+".png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	}
	return false;
}

