#include "code/game/menus/qj/QJMainMenu.h"
#include <oslib/oslib.h>
#include "code/framework/bmpFont/BmpFont.h"
#include "code/framework/bmpFont/BmpText.h"
#include "code/framework/fx/EffectsLib.h"
#include "code/framework/utils.h"
#include "code/framework/anim/Anim.h"
#include "code/framework/I18nResourceBundle.h"
#include "code/game/menus/qj/QJMenuItemManager.h"
#include "code/framework/Properties.h"
#include "code/framework/check/loaders.h"

using namespace std;
using namespace utils;

#define VINYL_X 328
#define VINYL_Y 91

#define QJ_MAINMENU_MUSIC "res/qj/menu/hachiko.bgm"
#define HACHIKO_VOLUME 0x6000

//construction
QJMainMenu::QJMainMenu(string locale)
{
	//enableDebug();
	//debug("QJ Main menu");
	m_locale=locale;
	m_action=MENU_NOACTION;
	m_skip=false;
	
	//debug("font");
	BmpFont * thefont=new BmpFont("res/fonts/bmpFonts/angie/20/", "angie", OSL_PF_8888);
	//debug("menuitemmanager");
	m_menu=new QJMenuItemManager();
	m_menu->init(locale, thefont);
	//debug("delete font");
	delete thefont;
	
	//musique sons
	//debug("load music");
	m_music=fwkLoadSoundFileBGM(QJ_MAINMENU_MUSIC, OSL_FMT_NONE);
	oslSetSoundLoop(m_music, true);
	m_music->volumeLeft=0;
	m_music->volumeRight=0;
	
	//debug("load sounds");
	m_snd_move=fwkLoadSoundFileBGM("res/qj/menu/whoosh2.bgm", OSL_FMT_NONE);
	m_snd_cancel=fwkLoadSoundFileBGM("res/qj/menu/main_menu_cancel.bgm", OSL_FMT_NONE);
	m_snd_select=fwkLoadSoundFileBGM("res/qj/menu/yamanotejingle.bgm", OSL_FMT_NONE);
	
	m_snd_move->volumeLeft=0x1800;
	m_snd_move->volumeRight=0x1800;
	m_snd_cancel->volumeLeft=0x5000;
	m_snd_cancel->volumeRight=0x5000;
	m_snd_select->volumeLeft=0x5000;
	m_snd_select->volumeRight=0x5000;
	
	//debug("chan mgr");
	m_chan_mgr=new ChannelManager();
	
	//debug("props");
	Properties *props=new Properties("res/qj/menu/qjmenu.properties");
	
	//debug("fx");
	m_fx_scene_appear=new ProgressiveAlphaEffect(	props->getIntValue("fx.appear.time"), 
													props->getIntValue("fx.appear.time"), 
													props->getIntValue("fx.appear.from"), 
													props->getIntValue("fx.appear.to"), 1);
													
	m_fx_bg_transition=new SerialGlobalEffect();
	//appear
	m_fx_bg_transition->addEffect(
						new ProgressiveAlphaEffect(	props->getIntValue("fx.imagechange.transition.time"), 
													props->getIntValue("fx.imagechange.transition.time"), 
													0, 255, 1));
	//wait
	m_fx_bg_transition->addEffect(
						new ProgressiveAlphaEffect(	props->getIntValue("fx.imagechange.wait.time"), 
													props->getIntValue("fx.imagechange.wait.time"), 
													255, 255, 1));									
	
	//disappear
	m_fx_bg_transition->addEffect(
						new ProgressiveAlphaEffect(	props->getIntValue("fx.imagechange.transition.time"), 
													props->getIntValue("fx.imagechange.transition.time"), 
													255, 0, 1));
													
	//wait
	m_fx_bg_transition->addEffect(
						new ProgressiveAlphaEffect(	props->getIntValue("fx.imagechange.wait.time"), 
													props->getIntValue("fx.imagechange.wait.time"), 
													0, 0, 1));	
	
	
	//debug("images");
	//on choisi 2 images au hasard
	int nbImg=props->getIntValue("images.nb");
	int num1=utils::random(1, nbImg);
	m_img_bg1=fwkLoadImageFilePNG((char *)string("res/qj/menu/bg"+toString(num1)+".png").c_str(), OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	
	int num2=num1;
	while(num2==num1)
	{
		num2=utils::random(1, nbImg);
	}
	m_img_bg2=fwkLoadImageFilePNG((char *)string("res/qj/menu/bg"+toString(num2)+".png").c_str(), OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	
	//debug("fx disappear");
	m_fx_disappearall=new SerialGlobalEffect(1);
	m_fx_disappearall->addEffect(
		new WaitGlobalEffect(props->getIntValue("fx.vanish.wait"))
	);
	m_fx_disappearall->addEffect(
		new ProgressiveAlphaEffect(	props->getIntValue("fx.vanish.time"), 
									props->getIntValue("fx.vanish.time"), 
									255, 0, 1)
	);
	
	m_img_title=fwkLoadImageFilePNG("res/qj/menu/megadrops2.png", OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageCenterScreen(m_img_title);
	m_img_title->y=props->getIntValue("image.title.y");
	
	
	//debug("delete props");
	delete props;
	//debug("construction ok");
	
}

//destruction
QJMainMenu::~QJMainMenu()
{
	delete m_menu;
	
	oslDeleteSound(m_music);
	oslDeleteSound(m_snd_move);
	oslDeleteSound(m_snd_cancel);
	oslDeleteSound(m_snd_select);
	
	delete m_chan_mgr;
	//delete m_timer_music;
	
	delete m_fx_scene_appear;
	delete m_fx_bg_transition;
	
	oslDeleteImage(m_img_bg1);
	oslDeleteImage(m_img_bg2);
	
	delete m_fx_disappearall;
	
	oslDeleteImage(m_img_title);
}

//demarrage du menu
int QJMainMenu::go(bool debugmode)
{
	//debug("go");
	m_chan_mgr->playSoundAndReserve(m_music); //on reserve indefiniment
	m_fx_scene_appear->start();
	m_fx_bg_transition->start();
	m_fx_disappearall->restart();
	
	while(m_action==MENU_NOACTION)
	{
		//touches
		//debug("keys");
		m_action=keys();
		if(m_action==MENU_QUIT)
			return m_action;
		
		//phase d'update
		//m_anim_vinyl->update();
		//debug("update");
		m_action=m_menu->update(); 
		m_fx_scene_appear->update();
		m_fx_bg_transition->update();
		m_fx_disappearall->update();
		if(m_music->volumeLeft<HACHIKO_VOLUME)
		{
			m_music->volumeLeft+=20;
			m_music->volumeRight+=20;
			
			if(m_music->volumeLeft>HACHIKO_VOLUME)
			{
				m_music->volumeLeft=HACHIKO_VOLUME;
				m_music->volumeRight=HACHIKO_VOLUME;
			}
		}
		if (!m_skip)
		{
			//debug("draw");
			oslStartDrawing();
			drawScene();
			oslEndDrawing();
		}
		
		oslEndFrame();
	    m_skip = oslSyncFrame();
	}
	
	oslStartDrawing();
	drawScene();
	oslEndDrawing();
			
	//il a choisi une option
	return m_action;	
}

//dessins
void QJMainMenu::drawScene()
{
	oslClearScreen(RGB(0,0,0));
	
	//debug("draw image1");
	oslDrawImage(m_img_bg1);
	//debug("transition activate");
	m_fx_bg_transition->activate();
	//debug("draw image 2");
	oslDrawImage(m_img_bg2);
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	//debug("disappear all activate");
	m_fx_disappearall->activate();
	oslDrawImage(m_img_title);
	m_menu->drawXY(240, 185);
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	
	
	if(m_fx_scene_appear->isRunning())
	{
		//debug("scene appear");
		m_fx_scene_appear->activate();
		oslDrawFillRect(0,0,480,272,RGB(0,0,0));
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
}

//gestion des touches
int QJMainMenu::keys()
{
	oslReadKeys();
	if (osl_pad.pressed.start) //Exit 
	{
		//return MENU_QUIT;
	}
	
	if (osl_pad.pressed.cross)
	{
		int action=m_menu->selectAction();
		if(action==MENU_CANCEL) //cliqué sur cancel
		{
			if(m_menu->cancelAction())
			{
				m_chan_mgr->restartSound(m_snd_cancel);
			}
		}
		else if(action!=MENU_OCCUPIED)
		{
			m_chan_mgr->restartSound(m_snd_select);
		}
	}
	else if (osl_pad.pressed.square)
	{
		if(m_menu->cancelAction())
		{
			m_chan_mgr->restartSound(m_snd_cancel);
		}
	}
	
	if (osl_pad.pressed.up)
	{
		if(m_menu->previousMenuAction())
		{
			m_chan_mgr->restartSound(m_snd_move);
		}
	}
	else if (osl_pad.pressed.down)
	{
		if(m_menu->nextMenuAction())
		{
			m_chan_mgr->restartSound(m_snd_move);
		}
	}
	if (osl_pad.pressed.right)
	{
		m_menu->previousValueAction();
	}
	else if (osl_pad.pressed.left)
	{
		m_menu->nextValueAction();
	}
	
	if(osl_pad.pressed.select)
	{
		screenshot();
	}
	
	if(osl_pad.pressed.value!=0)
	{
		m_fx_disappearall->restart();
	}
	
	return MENU_NOACTION;
}




