#include "code/game/menus/qj/dialogs/AbstractDialog.h"
#include "code/framework/imgFilters/imgFilters.h"
#include "code/framework/utils.h"
#include "code/framework/check/loaders.h"
#include <string>
#include "code/framework/check/PrettyFileLogger.h"
#include "code/framework/Properties.h"

using namespace utils;
using namespace std;

AbstractDialog::AbstractDialog(string overlayfilename)
{
	PFL::logConstructionStart("AbstractDialog");
	
	Properties *props=new Properties("res/qj/dialogs/dialogs.properties");
	
	m_img_normal=getBufferCopy();
	m_img_tinted=getHSLRecoloredBufferCopy(
		props->getColorValue("buffer.tint.color.ref"), 
		props->getIntValue("buffer.tint.luminance.min"), 
		props->getIntValue("buffer.tint.luminance.max"), 
		props->getIntValue("buffer.tint.alpha")
	);
	m_img_overlay=fwkLoadImageFilePNG(overlayfilename, OSL_IN_RAM|OSL_SWIZZLED, OSL_PF_8888);
	oslImageCenterScreen(m_img_overlay);
	
	m_fx_alpha=new ProgressiveAlphaEffect(90, 90, 0, 255, 1);
	m_fx_size=new SizeWowEffect(99, 1, 99, 1, 20, 20, true, 1);
	
	m_fx_alpha2=new ProgressiveAlphaEffect(15, 15, 255, 0, 1);
	m_fx_size2=new SizeWowEffect(99, 1, 99, 1, 15, 15, false, 1);
	
	delete props;
	
	PFL::logConstructionEnd("AbstractDialog");
}

AbstractDialog::~AbstractDialog()
{
	PFL::logDestructionStart("AbstractDialog");
	oslDeleteImage(m_img_normal);
	oslDeleteImage(m_img_tinted);
	oslDeleteImage(m_img_overlay);
	PFL::logMessage("AbstractDialog : images clean");
	
	delete m_fx_alpha;
	delete m_fx_size;
	delete m_fx_alpha2;
	delete m_fx_size2;
	PFL::logDestructionEnd("AbstractDialog");
}
	
int AbstractDialog::go()
{
	m_fx_alpha->start();
	m_fx_size->start();
	
	startSpecific();
	
	int action=DIALOG_NONE;
	while(action==DIALOG_NONE) 
	{
		action=keys();
		
		
		//phase d'update
		update();
		
		
		//phase de dessin
		oslStartDrawing();
		draw();
		oslEndDrawing();
		
			
        oslEndFrame();
        oslSyncFrame();
	}
	
	m_fx_alpha2->start();
	m_fx_size2->start();
	
	while(m_fx_alpha2->isRunning())
	{
		update();
		oslStartDrawing();
		draw();
		oslEndDrawing();
		oslEndFrame();
        oslSyncFrame();
	}
	
	//for(int i=0;i<2;i++)
	//{
		
		oslStartDrawing();
		oslDrawImage(m_img_normal);
		oslEndDrawing();
		oslEndFrame();
        oslSyncFrame();
		oslStartDrawing();
		oslDrawImage(m_img_normal);
		oslEndDrawing();
			
        //oslEndFrame();
        //oslSyncFrame();
	//}
	
	return action;
}



void AbstractDialog::draw()
{
	if(m_fx_alpha2->isRunning())
	{
		oslDrawImage(m_img_normal);
		m_fx_alpha2->activate();
		oslDrawImage(m_img_tinted);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	else if(m_fx_alpha2->finished())
	{
		oslDrawImage(m_img_normal);
	}
	else // if(m_fx_alpha->isRunning() || finished)
	{
		oslDrawImage(m_img_normal);
		m_fx_alpha->activate();
		oslDrawImage(m_img_tinted);
		oslSetAlpha(OSL_FX_DEFAULT, 0);
	}
	/*
	
	
	else
	{
		oslDrawImage(m_img_tinted);
	}
	*/
	if(!m_fx_size2->finished())
	{
		fx::draw(m_img_overlay, m_fx_size->isRunning()?m_fx_size:NULL, m_fx_size2->isRunning()?m_fx_size2:NULL);
	}
	
	if(m_fx_size->finished() && !m_fx_size2->isRunning() && !m_fx_size2->finished())
	{
		drawSpecific();
	}
}

void AbstractDialog::update()
{
	updateSpecific();
	
	m_fx_alpha->update();
	m_fx_size->update();
	m_fx_alpha2->update();
	m_fx_size2->update();
}
	


















