#include "code/game/pieces/impl/PieceCombo.h"
#include "code/game/pieces/PiecesBufferFactory.h"


PieceCombo::PieceCombo(int combos)
{
	//enableDebug();
	//debug("new PieceCombo");
	m_anim_destr=PiecesBufferFactory::newComboDestrAnim();
	m_nbCombos=combos;
	//debug("PieceCombo construit ok.");
	m_anim_change=PiecesBufferFactory::newComboShineAnim();
}

PieceCombo::~PieceCombo()
{
	//debug("delete PieceCombo");
	delete m_anim_destr;
	//debug("delete PieceCombo ok");
	delete m_anim_change;
}

PieceCombo::PieceCombo(const PieceCombo &piece)
{
	m_nbCombos=piece.m_nbCombos;
	m_anim_destr=PiecesBufferFactory::newComboDestrAnim();
	m_anim_change=PiecesBufferFactory::newComboShineAnim();
}

PieceCombo& PieceCombo::operator =(const PieceCombo& piece)
{
	m_nbCombos=piece.m_nbCombos;
	m_anim_destr=PiecesBufferFactory::newComboDestrAnim();
	m_anim_change=PiecesBufferFactory::newComboShineAnim();
	
	return *this;
}

PieceCombo * PieceCombo::clone() const
{
	//debug("PieceCombo clone");
	PieceCombo *p=new PieceCombo(m_nbCombos);
	//debug("clone return");
	return p;
}

int PieceCombo::getShape()
{
	return PIECE_SHAPE_NONE;
}

int PieceCombo::getColor()
{
	return PIECE_COLOR_NONE;
}

void PieceCombo::setColor(int color)
{
	
}

void PieceCombo::setShape(int shape)
{
	
}

int PieceCombo::getType()
{
	return PIECE_TYPE_COMBO;
}

void PieceCombo::startDestruct()
{
	m_anim_destr->start();
	m_anim_change->stop();
}

void PieceCombo::setExplosionTime(float time)
{
	m_anim_destr->setDelayTotal(time);
}

void PieceCombo::drawXY(int x, int y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	//debug("dessin PieceCombo");
	if(m_nbCombos>0)
	{
		//debug("combo>0");
		fx::drawXY(PiecesBufferFactory::getComboImage(m_nbCombos), x, y, fx1, fx2, fx3, fx4, fx5);
		if(m_anim_change->isRunning())
		{
			fx::drawXY(m_anim_change->getImage(), x, y, fx1, fx2, fx3, fx4, fx5);
		}
	}
	else
	{
		//debug("combo<=0");
		if(m_anim_destr->isRunning())
			fx::drawXY(m_anim_destr->getImage(), x, y, fx1, fx2, fx3, fx4, fx5);
		else
			fx::drawXY(PiecesBufferFactory::getComboImage(0), x, y, fx1, fx2, fx3, fx4, fx5);
	}
	//debug("dessin ok");
}

void PieceCombo::update()
{
	//debug("update");
	m_anim_destr->update();
	//debug("update ok");
	m_anim_change->update();
}

bool PieceCombo::destructed()
{
	return m_anim_destr->finished();
}

bool PieceCombo::destructing()
{
	return m_anim_destr->isRunning();
}

int PieceCombo::specialEvent(int e)
{
	if(e==PIECE_EVENT_COMBO_REDUCE && m_nbCombos!=0) //il y a eu une combo
	{	
		m_nbCombos--;
		m_anim_change->restart();
		return true;
	}
	else if(e==PIECE_EVENT_COMBO_EXPLODE && m_nbCombos==0) //on lui demande d'exploser si elle est a 0
	{
		m_anim_change->stop();
		m_anim_destr->start();
		return true;
	}
	else if(e==PIECE_EVENT_COMBO_ADD) //on lui demande d'incrémenter (pour éditeur)
	{
		m_nbCombos++;
		return true;
	}
	else if(e==PIECE_EVENT_COMBO_GET)
	{
		return m_nbCombos;
	}
	
	return false;
}








