#include "code/game/pieces/impl/PieceSimple.h"
#include "code/game/pieces/PiecesBufferFactory.h"

/////////////////////
//    INSTANCE     //
/////////////////////

PieceSimple::PieceSimple(int shape, int color)
//: Trackable("PieceSimple")
{
	m_color_num=color;
	m_shape_num=shape;
	m_color=getOslColor(m_color_num);
	m_anim_destr=PiecesBufferFactory::newPieceDestrAnim(shape);
	m_random=false;
	
	m_color_comp=new int[4];
	oslRgbaGet8888(m_color, m_color_comp[0], m_color_comp[1], m_color_comp[2], m_color_comp[3]);
	m_alpha = new OSL_ALPHA_PARAMS;
	m_curColor=new u8[4];
	m_newColor=new u8[4];
}

PieceSimple::PieceSimple()
//: Trackable("PieceSimple")
{
	m_color_num=random(0,5);
	m_shape_num=random(0,5);
	m_color=getOslColor(m_color_num);
	m_anim_destr=PiecesBufferFactory::newPieceDestrAnim(m_shape_num);
	m_random=true;
	
	m_color_comp=new int[4];
	oslRgbaGet8888(m_color, m_color_comp[0], m_color_comp[1], m_color_comp[2], m_color_comp[3]);
	m_alpha = new OSL_ALPHA_PARAMS;
	m_curColor=new u8[4];
	m_newColor=new u8[4];
}

PieceSimple::~PieceSimple()
{
	delete m_anim_destr;
	delete[] m_color_comp;
	delete m_alpha;
	delete[] m_curColor;
	delete[] m_newColor;
}

void PieceSimple::setExplosionTime(float time)
{
	m_anim_destr->setDelayTotal(time);
}


PieceSimple::PieceSimple(const PieceSimple &piece)//copie
//: Trackable("PieceSimple")
{
	m_color_num=piece.m_color_num;
	m_shape_num=piece.m_shape_num;
	m_color=getOslColor(m_color_num);
	m_anim_destr=PiecesBufferFactory::newPieceDestrAnim(m_shape_num);
	m_random=piece.m_random;
	
	m_color_comp=new int[4];
	oslRgbaGet8888(m_color, m_color_comp[0], m_color_comp[1], m_color_comp[2], m_color_comp[3]);
	m_alpha = new OSL_ALPHA_PARAMS;
	m_curColor=new u8[4];
	m_newColor=new u8[4];
}

PieceSimple& PieceSimple::operator =(const PieceSimple& piece)
{
	//this->Trackable::operator=(piece);
	
	m_color_num=piece.m_color_num;
	m_shape_num=piece.m_shape_num;
	m_color=getOslColor(m_color_num);
	m_anim_destr=PiecesBufferFactory::newPieceDestrAnim(m_shape_num);
	m_random=piece.m_random;
	
	m_color_comp=new int[4];
	oslRgbaGet8888(m_color, m_color_comp[0], m_color_comp[1], m_color_comp[2], m_color_comp[3]);
	m_alpha = new OSL_ALPHA_PARAMS;
	m_curColor=new u8[4];
	m_newColor=new u8[4];
	
	return *this;
}

PieceSimple * PieceSimple::clone() const
{
	PieceSimple *p=new PieceSimple(m_shape_num, m_color_num);
	p->m_random=m_random;
	return p;
}

int PieceSimple::getShape()
{
	return m_shape_num;
}

int PieceSimple::getColor()
{
	return m_color_num;
}

void PieceSimple::setColor(int color)
{
	m_color_num=color;
	m_color=getOslColor(m_color_num);
	
	oslRgbaGet8888(m_color, m_color_comp[0], m_color_comp[1], m_color_comp[2], m_color_comp[3]);
}

void PieceSimple::setShape(int shape)
{
	m_shape_num=shape;
	Anim *old=m_anim_destr;
	m_anim_destr=PiecesBufferFactory::newPieceDestrAnim(m_shape_num);
	m_anim_destr->sync(old);
	delete old;
}

int PieceSimple::getType()
{
	return PIECE_TYPE_NORMAL;
}

void PieceSimple::startDestruct()
{
	m_anim_destr->start();
}

void PieceSimple::drawXY(int x, int y, AbstractLocalEffect *fx1, AbstractLocalEffect *fx2, AbstractLocalEffect *fx3, AbstractLocalEffect *fx4, AbstractLocalEffect *fx5)
{
	//on calcule la couleur en mélangeant l'existant
	oslGetAlphaEx(m_alpha);
	oslRgbaGet8888(m_alpha->coeff1, m_curColor[0], m_curColor[1], m_curColor[2], m_curColor[3]);
	m_alpha->effect=OSL_FX_ALPHA | OSL_FX_TINT;
	for(int i=0;i<4;i++)
	{	
		m_newColor[i]=u8(float(m_curColor[i]*m_color_comp[i])/255.0f);
	}
	int oldcoeff1=m_alpha->coeff1;
	m_alpha->coeff1=RGBA(m_newColor[0], m_newColor[1], m_newColor[2], m_newColor[3]);
	oslSetAlphaEx(m_alpha);
	
	//on dessine
	fx::drawXY(m_anim_destr->getImage(), x, y, fx1, fx2, fx3, fx4, fx5);
	
	//on remet comme avant
	m_alpha->coeff1=oldcoeff1;
	oslSetAlphaEx(m_alpha);
	
}

void PieceSimple::update()
{
	m_anim_destr->update();
}
 
bool PieceSimple::destructed()
{
	return m_anim_destr->finished();
}

bool PieceSimple::destructing()
{
	return m_anim_destr->isRunning();
}

bool PieceSimple::isRandom()
{
	return m_random;
}





