#include "code/game/score/Scorer.h"
#include "code/game/gamedef.h"
#include "code/game/score/Score.h"
#include "code/framework/utils.h"
#include "code/game/grid/GridEventResult.h"
#include "code/game/saves/GameSave.h"

using namespace utils;

Scorer::Scorer(string levelcode, int difficulty)
{
	m_score_current=0.0f;
	m_maxcombos_current=0;
	
	//Score score=ProfileManager::getBestScoreForLevel(levelcode, difficulty);
	//Score combos=ProfileManager::getBestCombosForLevel(levelcode, difficulty);
	
	GameSave *gamesave=new GameSave();
	//m_score_best_all=score.getScore();
	m_score_best_self=gamesave->getScoreForLevel(levelcode, difficulty);
	//m_score_best_name=score.getName();
		
	//m_maxcombos_best_all=combos.getScore();
	m_maxcombos_best_self=gamesave->getCombosForLevel(levelcode, difficulty);
	//m_maxcombos_best_name=combos.getName();
	
	delete gamesave;
	
	m_did_best_combos=false;
	m_did_best_score=false;
}

Scorer::~Scorer()
{
	
}


int Scorer::getScoreCurrent()
{
	return int(m_score_current);
}


int Scorer::getScoreBestSelf()
{
	return m_score_best_self;
}


int Scorer::getMaxcombosCurrent()
{
	return m_maxcombos_current;
}


int Scorer::getMaxcombosBestSelf()
{
	return m_maxcombos_best_self;
}


void Scorer::notifyFastdropFrame()
{
	m_score_current+=SCORE_POINTS_FASTDROP_PERFRAME;
	
	updateBest();
}

void Scorer::notifyExplosions(GridEventResult ger, int nbCombos)
{
	if(nbCombos<1)
		nbCombos=1;
	
	float combomult=1+(nbCombos-1)*SCORE_POINTS_COMBO_MULTIPLIER;
	
	float add=0.0f; //ce qu'on va ajouter au score
	add+=ger.getExplContact()*SCORE_POINTS_CONTACT_EXPLOSION;
	add+=ger.getExplNormal()*SCORE_POINTS_SIMPLE_EXPLOSION;
	add+=ger.getExplJoker()*SCORE_POINTS_JOKER_EXPLOSION;
	add+=ger.getExplSpecial()*SCORE_POINTS_SPECIAL_EXPLOSION;
		
	m_score_current+=(add*combomult);
	
	updateBest();
}

void Scorer::notifyComboExplosions(int nbExpl, int nbCombos)
{
	if(nbCombos<1)
		nbCombos=1;
	
	float combomult=1+(nbCombos-1)*SCORE_POINTS_COMBO_MULTIPLIER;
	m_score_current+=nbExpl*SCORE_POINTS_COMBO_EXPLOSION*combomult;
	
	updateBest();
}

void Scorer::notifyComboNumber(int nb)
{
	if(nb<1)
		nb=1;
		
	nb--;
	if(nb>m_maxcombos_current)
		m_maxcombos_current=nb;
	
	updateBest();
}

void Scorer::notifyComboDecrements(int nbDecr, int nbCombos)
{
	if(nbCombos<1)
		nbCombos=1;
	
	float combomult=1+(nbCombos-1)*SCORE_POINTS_COMBO_MULTIPLIER;
	m_score_current+=nbDecr*SCORE_POINTS_COMBO_DECREMENT*combomult;
	
	updateBest();
}

void Scorer::updateBest() //met a jour au cas ou le score ou les combos du best sont battus
{
	if(m_maxcombos_current>m_maxcombos_best_self)
	{
		m_did_best_combos=true;
	}
	
	if(m_score_current>m_score_best_self)
	{
		m_did_best_score=true;
	}
}

bool Scorer::isBestCombos()
{
	return m_did_best_combos;
}

bool Scorer::isBestScore()
{
	return m_did_best_score;
}















